#include <string.h>
#include "opora.h"
#include "itmr.h"
#include "button.h"
#include "uart.h"

/*
	      UART. 
	  115200/ 8N1.
	           .
	     "led1on "  "led1off "     VD13
*/

void CoreClkConfig(void);
void PerClkConfig(void);
void InPortConfig(void);
void OutPortConfig(void);

//  
static char test_up[] = "test_up\r\n";
static char test_down[] = "test_down\r\n";
static char test_right[] = "test_right\r\n";
static char test_left[] = "test_left\r\n";
static char test_select[] = "test_select\r\n";

int main(void)
{
	unsigned int stime;
	//    GPIO
	CoreClkConfig();
	PerClkConfig();
	itmr_Init();
	InPortConfig();
	OutPortConfig();
	//  UART1,      
	UART1Config();
	
	stime = itmr_GetTick();
	while(1)
	{											
			//     
			switch(button_Press()){
				case UP:					
					uart_Send(test_up, strlen(test_up));					
					break;					
				case DOWN: 					
					uart_Send(test_down, strlen(test_down));
					break;
				case RIGHT: 					
					uart_Send(test_right, strlen(test_right));
					break;					
				case LEFT: 
					uart_Send(test_left, strlen(test_left));
					break;					
				case SELECT:
					uart_Send(test_select, strlen(test_select));					
					break;					
				case NOPRESS:					
					break;
				default:
					break;
			}			
		
		uart_Rcv(); //  
			
		if(itmr_IsTime(stime, 1000)){									
			PORTD->RXTX ^= (1 << 0) | (1 << 7) | (1 << 8) | (1 << 11);
			stime = itmr_GetTick();		
		}						
	}
}

void CoreClkConfig(void)
{
	uint32_t temp;
	RST_CLK->PER_CLOCK |= (1 << 27);				//BKP Clock enable
	temp = BKP->REG_0E;
	temp &= 0xFFFFFFC0;
	BKP->REG_0E = temp | (5 << 3) | 5;				// SelectRI = 0x5, LOW = 0x5; (for frequency below 40 MHz);

	RST_CLK->HS_CONTROL = 0x00000001;			//HSE - On; Oscillator mode
	while((RST_CLK->CLOCK_STATUS&0x04)!=0x04);	//Wait until HSE not ready
	RST_CLK->CPU_CLOCK=0x00000002;				//CPU_C1 = 16MHz
	RST_CLK->PLL_CONTROL=(1<<2)|(2<<8);			//CPU PLL On, PLL_MULL=2
	while((RST_CLK->CLOCK_STATUS&0x02)!=0x02);
	RST_CLK->PER_CLOCK|=0x08;					//EEPROM_CTRL Clock enable
	EEPROM->CMD=1;
	RST_CLK->PER_CLOCK&=(~0x08);				//EEPROM_CTRL Clock disable
	RST_CLK->CPU_CLOCK = 0x00000106;			//CPU Clock = 3*16MHz
}

void PerClkConfig(void)
{
	RST_CLK->PER_CLOCK	|=	(1 << 21) /*PORTA*/ |
							(1 << 23) /*PORTC*/ |
							(1 << 24) /*PORTD*/ |
							(1 << 14) /*TIMER1*/ |
							(1 << 6) /*UART1*/;
	RST_CLK->TIM_CLOCK=0x01000000; /*TIMER1*/
	RST_CLK->UART_CLOCK=(1 << 24); /*UART1*/	
}

void InPortConfig(void)
{
	PORTD->FUNC &= ~((0x03 << (2 * 6)) | (0x03 << (2 * 5)) | (0x03 << (2 * 4)) | (0x03 << (2 * 3)) | (0x03 << (2 * 2)));
	PORTD->OE &= ~((1 << 6) | (1 << 5) | (1 << 4) | (1 << 3) | (1 << 2));
	PORTD->ANALOG |= (1 << 6) | (1 << 5) | (1 << 4) | (1 << 3) | (1 << 2);
	PORTD->PWR |= (0x03 << (2 * 6)) | (0x03 << (2 * 5)) | (0x03 << (2 * 4)) | (0x03 << (2 * 3)) | (0x03 << (2 * 2));				
}

void OutPortConfig(void)
{
	PORTA->FUNC |= 0x00000000;	
	PORTA->OE |= (1 << 9) | (1 << 13);
	PORTA->ANALOG |= (1 << 9) | (1 << 13);
	PORTA->PWR |= (0x3 << (9 * 2)) | (0x03 << (13 * 2));
	
	PORTC->FUNC |= 0x00000000;	
	PORTC->OE |= (1 << 8) | (1 << 9);
	PORTC->ANALOG |= (1 << 8) | (1 << 9);
	PORTC->PWR |= (0x3 << (8 * 2)) | (0x03 << (9 * 2));
	
	PORTD->FUNC |= 0x00000000;
	PORTD->RXTX |= (1 << 0) | (1 << 8);
	PORTD->OE |= (1 << 0) | (1 << 7) | (1 << 8) | (1 << 11);
	PORTD->ANALOG |= (1 << 0) | (1 << 7) | (1 << 8) | (1 << 11);
	PORTD->PWR |= (0x3 << (0 * 2)) | (0x03 << (7 * 2)) | (0x03 << (8 * 2)) | (0x03 << (11 * 2));			
}
