#include <string.h>
#include <opora.h>
#include "misc.h"
#include "lcd_drv.h"
#include "font.h"

/*
	     LCD .
	 , .    GUI.
*/

extern void lcd_drv_OnOffCtrl(unsigned char cmd);
extern void lcd_drv_SetStartLine(unsigned int line_num);
extern void lcd_drv_SetEqData(unsigned char data);

//------------------------------------------------------------
// 
static void lcd_Init(void)
{		
	PORTB->RXTX &= ~(1 << 3);	
	delay(DELAY_1040NS);			
	PORTB->RXTX |= (1 << 3);		
	delay(DELAY_10280NS);		
}

//------------------------------------------------------------
void lcd_drv_Init(void)
{
	lcd_Init();	
	lcd_drv_OnOffCtrl(1);
	lcd_drv_Clear();	
	lcd_drv_SetStartLine(0);
}

void lcd_drv_OnOffCtrl(unsigned char cmd)
{
	uint16_t t = 0;		
	PORTF->RXTX &= ~(1 << 13); //  0	
	PORTB->RXTX &= ~(1 << 1);	//  	0
	PORTF->RXTX &= ~(1 << 15);	//  0
	//----------------------------------------------
	//   1  2
	PORTD->RXTX |= (1 << 8) | (1 << 11);		// CS1 CS2	1			
	delay(DELAY_160NS);
	PORTF->RXTX |= (1 << 13);	//  1	
	delay(DELAY_160NS);		
	t = PORTB->RXTX;	
	t &= 0x00FF;
	t |= (cmd ? (ON << 8) : (OFF << 8));
	PORTB->RXTX = t;
	delay(DELAY_480NS);//####
	delay(DELAY_226NS);//###
	PORTF->RXTX &= ~(1 << 13);	//  0	
	PORTD->RXTX &= ~((1 << 8) | (1 << 11)); // CS1 CS2 0		
	delay(DELAY_8220NS);//###
}

//------------------------------------------------------------
void lcd_drv_SetStartLine(unsigned int line_num)
{	
	uint16_t t = 0;	
	PORTF->RXTX &= ~(1 << 13); //  0	
	PORTB->RXTX &= ~(1 << 1);	//  	0
	PORTF->RXTX &= ~(1 << 15);	//  0		
	//   1  2
	PORTD->RXTX |= (1 << 8) | (1 << 11);		// CS1 CS2	1	
	delay(DELAY_160NS);	//###
	PORTF->RXTX |= (1 << 13);	//  1		
	delay(DELAY_160NS);	//###			
	t = PORTB->RXTX;	
	t &= 0x00FF;
	t |= (0xC0 | (line_num & 0x3F)) << 8;
	PORTB->RXTX = t;
	delay(DELAY_480NS);//####	
	delay(DELAY_226NS);//###
	PORTF->RXTX &= ~(1 << 13);	//  0		
	PORTD->RXTX &= ~((1 << 8) | (1 << 11)); // CS1 CS2 0			
	delay(DELAY_8220NS);//###	
}

//------------------------------------------------------------
void lcd_drv_SetPage(unsigned char crystal, unsigned char page_num)
{	
	uint16_t t = 0;	
	PORTF->RXTX &= ~(1 << 13); //  0	
	PORTB->RXTX &= ~(1 << 1);	//  	0
	PORTF->RXTX &= ~(1 << 15);	//  0	
	if(!crystal)
		PORTD->RXTX |= (1 << 8);
	else
		PORTD->RXTX |= (1 << 11);	
	delay(DELAY_160NS);	//###
	PORTF->RXTX |= (1 << 13);	//  1			
	delay(DELAY_160NS);	//###???????		
	t = PORTB->RXTX;	
	t &= 0x00FF;
	t |= (0xB8 | (page_num & 0x7)) << 8;
	PORTB->RXTX = t;
	delay(DELAY_480NS);//####	
	delay(DELAY_226NS);//###
	PORTF->RXTX &= ~(1 << 13); //  0		
	if(!crystal)
		PORTD->RXTX &= ~(1 << 8);
	else
		PORTD->RXTX &= ~(1 << 11);			
	delay(DELAY_8220NS);//###
}

//------------------------------------------------------------
void lcd_drv_SetAddr(unsigned int addr)
{	
	if(addr > 127) return;	
	else{				
		uint16_t t = 0;		
		PORTF->RXTX &= ~(1 << 13); //  0		
		PORTB->RXTX &= ~(1 << 1);	//  	0
		PORTF->RXTX &= ~(1 << 15);	//  0		
		if(addr < 64)
			PORTD->RXTX |= (1 << 8);
		else
			PORTD->RXTX |= (1 << 11);
		delay(DELAY_160NS);	//###
		PORTF->RXTX |= (1 << 13);	//  1	
		delay(DELAY_160NS);	//###???????				
		t = PORTB->RXTX;	
		t &= 0x00FF;
		t |= (0x40 | (addr & 0x3F)) << 8;
		PORTB->RXTX = t;
		delay(DELAY_480NS);//####		
		delay(DELAY_226NS);//###		
		PORTF->RXTX &= ~(1 << 13); //  0					
		if(addr < 64)
			PORTD->RXTX &= ~(1 << 8);
		else
			PORTD->RXTX &= ~(1 << 11);						
		delay(DELAY_8220NS);//###
	}
}

//------------------------------------------------------------
void lcd_drv_WriteData(unsigned char data, unsigned int addr)
{	
	if(addr > 127) return;	
	else{
		uint16_t t = 0;
		unsigned char crystal_num;		
		crystal_num = addr < 64 ? 0 : 1;			
		lcd_drv_SetAddr(addr);				
		PORTF->RXTX &= ~(1 << 13); //  0			
		PORTB->RXTX &= ~(1 << 1);	//  	
		PORTF->RXTX |= (1 << 15);	// 
		if(!crystal_num)
			PORTD->RXTX |= (1 << 8);
		else
			PORTD->RXTX |= (1 << 11);		
		delay(DELAY_160NS);	//###	
		PORTF->RXTX |= (1 << 13);	//  1			
		delay(DELAY_160NS);	//###???????				
		t = PORTB->RXTX;	
		t &= 0x00FF;
		t |= data << 8;	;
		PORTB->RXTX = t;
		delay(DELAY_480NS);//####		
		delay(DELAY_226NS);//###
		PORTF->RXTX &= ~(1 << 13); //  0							
		if(!crystal_num)
			PORTD->RXTX &= ~(1 << 8);
		else
			PORTD->RXTX &= ~(1 << 11);							
		delay(DELAY_8220NS);//###
	}
}

//------------------------------------------------------------
void lcd_drv_SetEqData(unsigned char data)
{
	unsigned char page,addr;
	
	lcd_drv_SetAddr(0);
	for(page = 0; page < 8; page++){
		lcd_drv_SetPage(0, page);
		for(addr = 0; addr < 64; addr++){
			lcd_drv_WriteData(data & 0xFF, addr);			
		}
	}		

	lcd_drv_SetAddr(64);
	for(page = 0; page < 8; page++){
		lcd_drv_SetPage(1, page);
		for(addr = 64; addr < 128; addr++)
			lcd_drv_WriteData(data & 0xFF, addr);
	}		
	lcd_drv_SetAddr(0);
	lcd_drv_SetPage(0, 0);
	lcd_drv_SetAddr(64);
	lcd_drv_SetPage(1, 0);
}

//------------------------------------------------------------
void lcd_drv_SetEqDataToPage(unsigned char crystal, unsigned char page, unsigned char data)
{
	unsigned char addr;
	
	lcd_drv_SetPage(crystal, page);	
	
	if(!crystal)	// ?????? ???????? ???????
		lcd_drv_SetAddr(0);						
	else 			// ?????? ???????? ???????
		lcd_drv_SetAddr(64);

	for(addr = 0; addr < 64; addr++)
		lcd_drv_WriteData(data, addr);	
}

//------------------------------------------------------------
void lcd_drv_SetEqDataToPageAddr(unsigned char page, unsigned char start_addr, unsigned char stop_addr, unsigned char data)
{
	unsigned char addr;

	if(start_addr > stop_addr) return;

	if((start_addr < 64) && (stop_addr < 64))
		lcd_drv_SetPage(0, page);
	else{
		if((start_addr >= 64) && (stop_addr >= 64))
			lcd_drv_SetPage(1, page);			
		else{
			lcd_drv_SetPage(0, page);						
			lcd_drv_SetPage(1, page);						
		}
	}	
	lcd_drv_SetAddr(start_addr);	
			
	for(addr = start_addr; addr < stop_addr; addr++)
		lcd_drv_WriteData(data, addr);										
}

//------------------------------------------------------------
void lcd_drv_PrintChar(unsigned char page, unsigned char addr, unsigned char data, unsigned char show_mode)
{
	unsigned char cnt;
	unsigned int off = data * 6;
	
	lcd_drv_SetPage(0, page);
	lcd_drv_SetPage(1, page);
	lcd_drv_SetAddr(addr);

	if(show_mode){
		for(cnt = 0; cnt < 6;)
			lcd_drv_WriteData(~Font[off + cnt++], addr++);	
	}	
	else{			
		for(cnt = 0; cnt < 6;)
			lcd_drv_WriteData(Font[off + cnt++], addr++);	
	}
}

//------------------------------------------------------------
void lcd_drv_PrintString(unsigned char page, unsigned char addr, unsigned char* string, unsigned char show_mode)
{
	unsigned int str_ind, str_length;
		
	if((str_length = strlen((char*)string)) > 21)
		str_length = 21;	
	
	lcd_drv_ClearPageAddr(page, addr, addr + str_length * 6 + 6);

	if(addr < 64){
		lcd_drv_SetAddr(addr);
		lcd_drv_SetAddr(64);
	}
	else{
		lcd_drv_SetAddr(0);
		lcd_drv_SetAddr(addr);		
	}
	
	lcd_drv_SetPage(0, page);	
	lcd_drv_SetPage(1, page);

	for(str_ind = 0; str_ind < str_length; str_ind++, addr += 6)
		lcd_drv_PrintChar(page, addr, string[str_ind], show_mode);	 
}

//------------------------------------------------------------
#define MAX_NUM_CHAR_CNT	11
void lcd_drv_PrintNum(unsigned char page, unsigned char addr, unsigned int val, unsigned char show_mode)
{
	unsigned char temp[MAX_NUM_CHAR_CNT];	
	unsigned int cnt;	
		
	cnt = uint2text(val, temp, MAX_NUM_CHAR_CNT - 1);
	temp[MAX_NUM_CHAR_CNT - 1] = 0;
	memcpy(temp, temp + MAX_NUM_CHAR_CNT - cnt - 1, (cnt + 1) * sizeof(unsigned char));
											
	lcd_drv_PrintString(page, addr, temp, show_mode);
}

//------------------------------------------------------------
void lcd_drv_Point(unsigned int x, unsigned int y)
{
	unsigned char page = y / 8;

	lcd_drv_SetPage(0, page);
	lcd_drv_SetPage(1, page);	
			
	lcd_drv_WriteData(1 << (y - 8 * page),x);
}

//------------------------------------------------------------
void lcd_drv_Byte(unsigned char page, unsigned char addr, unsigned char val)
{
	lcd_drv_SetPage(0, page);
	lcd_drv_SetPage(1, page);			

	lcd_drv_WriteData(~val, addr);
}
