#include "MDR32F9Qx_config.h"
#include <MDR32F9Qx_port.h>
#include <MDR32F9Qx_rst_clk.h>
#include "MDR32F9Qx_arinc429t.h"
#include "MDR32F9Qx_uart.h"
#include "MDR32F9Qx_arinc429r.h"
#include "MDR32F9Qx_eeprom.h"

//	  
#define DELAY_TICKS 1000000

#define HSE_ERROR 	(1ul << 0)
#define PLL_ERROR 	(1ul << 1)
#define LSE_ERROR 	(1ul << 2)
#define HSE2_ERROR 	(1ul << 3)
#define RTC_WAIT1_ERROR 	(1ul << 4)
#define RTC_WAIT2_ERROR 	(1ul << 5)


//    ,  
void Delay(int waitTicks);
void ClockConfigure ( void );

volatile uint32_t ReadBytes = 0;
uint32_t ReadArray[64];
static unsigned int Error = 0;

void ARINC429T_CONTROL4(uint32_t NewState)
{
	MDR_ARINC429T_TypeDef * ARINC429Tx;

	ARINC429Tx = MDR_ARINC429T;
	
	ARINC429Tx->CONTROL4 = NewState;
}

void ARINC429R_CONTROL9(uint32_t NewState)
{
	MDR_ARINC429R_TypeDef * ARINC429Rx;

	ARINC429Rx = MDR_ARINC429R;
	
	ARINC429Rx->CONTROL9 = NewState;
}

//   ,    
int main()
{
	static ARINC429T_InitChannelTypeDef	ARINC429T_InitChanelStruct;
	static ARINC429R_InitChannelTypeDef	ARINC429R_InitChanelStruct;
	//uint8_t CMDkey = 0;
	static uint32_t TestData[64] = { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
            						 0xCC, 0x0C, 0x0D, 0xCD, 0x0F, 0xCF, 0xCE, 0x0E, 0x0A, 0xCA, 0xCB, 0x0B, 0xC9, 0x09, 0x08, 0xC8,
            						 0xD8, 0x18, 0x19, 0xD9, 0x1B, 0xDB, 0xDA, 0x1A, 0x1E, 0xDE, 0xDF, 0x1F, 0xDD, 0x1D, 0x1C, 0xDC,
            						 0x14, 0xD4, 0xD5, 0x15, 0xD7, 0x17, 0x16, 0xD6, 0xD2, 0x12, 0x13, 0xD3, 0x11, 0xD1, 0xD0, 0x10 };
	uint32_t i;
												 
  uint32_t Data_CH3 = 0xF0;
	uint32_t Data_CH4 = 0xF0;
	
	ClockConfigure();
	//    (-)  GPIO
  PORT_InitTypeDef GPIOInitStruct;
	PORT_InitTypeDef PORT_InitStructure;

  //     C
  RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTC, ENABLE);
	RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTF, ENABLE);
	RST_CLK_PCLKcmd(RST_CLK_PCLK_ARINC429T, ENABLE);

  //     (-)    
  PORT_StructInit(&GPIOInitStruct);
  
  //         
  GPIOInitStruct.PORT_Pin        = PORT_Pin_9;
  GPIOInitStruct.PORT_OE         = PORT_OE_OUT;
  GPIOInitStruct.PORT_SPEED      = PORT_SPEED_SLOW;
  GPIOInitStruct.PORT_MODE       = PORT_MODE_DIGITAL;
  
  //       PORTC.
  PORT_Init(MDR_PORTC, &GPIOInitStruct);
	
	
												 
	/* Init ARINC429T --------------------------------------------------------*/

	PORT_DeInit(MDR_PORTF);
	PORT_StructInit(&PORT_InitStructure);
	PORT_InitStructure.PORT_Pin = PORT_Pin_7 | PORT_Pin_8 | PORT_Pin_9 | PORT_Pin_10; 
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_OVERRID;
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;
	PORT_InitStructure.PORT_GFEN = PORT_GFEN_OFF;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;                    
	PORT_InitStructure.PORT_PULL_DOWN = PORT_PULL_DOWN_OFF;
	PORT_InitStructure.PORT_PULL_UP = PORT_PULL_UP_OFF;
	PORT_InitStructure.PORT_PD = PORT_PD_DRIVER;
	PORT_Init(MDR_PORTF, &PORT_InitStructure);
	
	/* Reset all ARINC429T settings */
	ARINC429T_DeInit();

	/* Set the HCLK division factor = 80 for ARINC429T */
	ARINC429T_BRG_Init(80);

	/* ARINC429T channel 3&4 configuration */
	ARINC429T_InitChanelStruct.ARINC429T_CLK = ARINC429T_CLK_100_KHz;
	ARINC429T_InitChanelStruct.ARINC429T_DIV = 0;
	ARINC429T_InitChanelStruct.ARINC429T_EN_PAR = ENABLE;
	ARINC429T_InitChanelStruct.ARINC429T_ODD = ARINC429T_ODD_ADD_TO_ODD;
	ARINC429T_ChannelInit(ARINC429T_CHANNEL3, &ARINC429T_InitChanelStruct);
	ARINC429T_ChannelInit(ARINC429T_CHANNEL4, &ARINC429T_InitChanelStruct);

	/* Enables ARINC429T channel 3&4 */
	ARINC429T_ChannelCmd(ARINC429T_CHANNEL3, ENABLE);
	ARINC429T_ChannelCmd(ARINC429T_CHANNEL4, ENABLE);
	
	ARINC429T_CONTROL4(0x0C); // CHANNEL4_ENSYNC, CHANNEL3_ENSYNC - ENABLE
	
	
	
	
	
	/* Init ARINC429R --------------------------------------------------------*/
	PORT_StructInit(&PORT_InitStructure);
	PORT_InitStructure.PORT_Pin = PORT_Pin_4 | PORT_Pin_5 | PORT_Pin_6 | PORT_Pin_7; 
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_ALTER;
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_OE = PORT_OE_IN;
	PORT_InitStructure.PORT_GFEN = PORT_GFEN_OFF;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;                    
	PORT_InitStructure.PORT_PULL_DOWN = PORT_PULL_DOWN_OFF;
	PORT_InitStructure.PORT_PULL_UP = PORT_PULL_UP_OFF;
	PORT_InitStructure.PORT_PD = PORT_PD_DRIVER;
	PORT_Init(MDR_PORTB, &PORT_InitStructure);
	
	/* Reset all ARINC429R settings */
	ARINC429R_DeInit();

	/* Set the HCLK division factor = 80 for ARINC429R */
	ARINC429R_BRG_Init(80);

	/* ARINC429T channel 3&4 configuration */
	ARINC429R_InitChanelStruct.ARINC429R_CLK = ARINC429R_CLK_100_KHz;
	ARINC429R_InitChanelStruct.ARINC429R_DIV = 0;
	ARINC429R_ChannelInit(ARINC429R_CHANNEL3, &ARINC429R_InitChanelStruct);
	ARINC429R_ChannelInit(ARINC429R_CHANNEL4, &ARINC429R_InitChanelStruct);

	/* Enables ARINC429T channel 3&4 */
	ARINC429R_ChannelCmd(ARINC429R_CHANNEL3, ENABLE);
	ARINC429R_ChannelCmd(ARINC429R_CHANNEL4, ENABLE);
	
	//ARINC429R_ITConfig(ARINC429R_IT_INT_DR, ENABLE);

	
	
	ARINC429R_CONTROL9(0x0C); // CHANNEL4_ENSYNC, CHANNEL3_ENSYNC - ENABLE


	//DEBUG_PRINTF("Init ARINC429T CHANNEL2 ... Ok\r\n");											 
	i = 0;
	
	
	ARINC429T_SendData(ARINC429T_CHANNEL3,Data_CH3);	
	Delay(100);

	ARINC429T_SendData(ARINC429T_CHANNEL4,Data_CH4);	
	Delay(100);

  //      -  	
  while (1)
  {
		//    PD0
		//     "0",      "1"
    if (PORT_ReadInputDataBit (MDR_PORTC, PORT_Pin_9) == 0)
    {	
			PORT_SetBits(MDR_PORTC, PORT_Pin_9);	// LED
    }	

		while(ARINC429R_GetFlagStatus(ARINC429R_CHANNEL3,ARINC429R_FLAG_DR) == 0)
		{
		}
		
		Data_CH3 = ARINC429R_ReceiveDataDirect(ARINC429R_CHANNEL3);
		
		ARINC429T_SendData(ARINC429T_CHANNEL3,Data_CH3);	
		Delay(100);
		
		while(ARINC429R_GetFlagStatus(ARINC429R_CHANNEL4,ARINC429R_FLAG_DR) == 0)
		{
		}
		
		Data_CH4 = ARINC429R_ReceiveDataDirect(ARINC429R_CHANNEL4);

		ARINC429T_SendData(ARINC429T_CHANNEL4,Data_CH4);	
		Delay(100);
		
		
    //    PD0
    //    = "1",   "0"
    if (PORT_ReadInputDataBit (MDR_PORTC, PORT_Pin_9) == 1)
    {
			PORT_ResetBits(MDR_PORTC, PORT_Pin_9);
    };
    Delay(10000);
  }      
}

//    ,        
void Delay(int waitTicks)
{
  int i;
  for (i = 0; i < waitTicks; i++)
  {
    __NOP();
  }	
}

void error_SetError(unsigned int error)
{
	Error |= error;
}

/**
  * @brief	Configure CPU clock.
  * @param	None
  * @retval None
  */
void ClockConfigure ( void )
{
	// Enable HSE (High Speed External) clock
  RST_CLK_HSEconfig(RST_CLK_HSE_ON);
  if (RST_CLK_HSEstatus() == ERROR) error_SetError(HSE_ERROR);//while (1);  	
		
  // Configures the CPU_PLL clock source 
  RST_CLK_CPU_PLLconfig(RST_CLK_CPU_PLLsrcHSEdiv1, RST_CLK_CPU_PLLmul6);

  // Enables the CPU_PLL 
  RST_CLK_CPU_PLLcmd(ENABLE);
  if (RST_CLK_CPU_PLLstatus() == ERROR) error_SetError(PLL_ERROR);//while (1);  

  // Enables the RST_CLK_PCLK_EEPROM 
  RST_CLK_PCLKcmd(RST_CLK_PCLK_EEPROM, ENABLE);
  // Sets the code latency value 
  EEPROM_SetLatency(EEPROM_Latency_4);

  // Select the CPU_PLL output as input for CPU_C3_SEL
  RST_CLK_CPU_PLLuse(ENABLE);
  // Set CPUClk Prescaler 
  RST_CLK_CPUclkPrescaler(RST_CLK_CPUclkDIV1);

  // Select the CPU clock source 
  RST_CLK_CPUclkSelection(RST_CLK_CPUclkCPU_C3);

  // Enables the RTCHSE clock on all ports 
  //RST_CLK_PCLKcmd(ALL_PORTS_CLK, ENABLE);

  MDR_RST_CLK->PER_CLOCK = 0xFFFFFFFF;
}

