/******************************************************************************/
/*!
 *  @file     M95M02.h
 *  @author   Clint Stevenson (Casco Logix)
 *  @license  
 *
 *  The M95M02 is a SPI-interface EEPROM chip with 2Mbit capacity
 *
 *  This is the header file for the M95M02 Arduino library
 *
 *  @section  HISTORY
 *  
 *  v1.0 - First release
 *  v1.1 - Bitbucket Issue #1:
 *		   Fixed data types in read and write to uint32_t to support full page
 *         read/writes as well as reading/writing the entire chip. 
 *		   Bitbucket Issue #2, Issue #3:
 *		   Added proposed code for accessing the EEPROM's ID page.
 */
/******************************************************************************/


#ifndef M95M02_H
#define M95M02_H



#define M95M02_PAGE_SIZE			(256)
#define M95M02_NUM_PAGES			(1024)
#define M95M02_CAPACITY_BYTES		(M95M02_PAGE_SIZE * M95M02_NUM_PAGES)

unsigned char M95M02_write (unsigned int address, unsigned char data);
unsigned char M95M02_write_buffer (unsigned int address, unsigned char * pBuffer, unsigned int numBytes) ;
unsigned char M95M02_read (unsigned int address);
unsigned char M95M02_read_buffer (unsigned int address, unsigned char * pBuffer, unsigned int numBytes);
	unsigned char M95M02_getStatus (void) ;
	void M95M02_readIDPage (unsigned char address, unsigned char * pBuffer, unsigned short size);
	void M95M02_writeIDPage (unsigned char address, unsigned char * pBuffer, unsigned short size);
	void M95M02_readLockStatus (unsigned char * data);
	void M95M02_lockID (void);
	void M95M02_commandWREN (void);
unsigned char eeprom_id(void);
#endif // M95M02_H
