
// Program realization of I2C interface

#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"
#include "itmr.h"

#define I2C_PORT			MDR_PORTE
#define RST_I2C_PORT	RST_CLK_PCLK_PORTE
#define I2C_SCL_PIN 	PORT_Pin_3
#define I2C_SDA_PIN 	PORT_Pin_4



static void i2c_port_in (unsigned int pin, MDR_PORT_TypeDef * port);
static void i2c_port_out (unsigned int pin, MDR_PORT_TypeDef *  port);

#define I2CSWM_DELAY 10000


#define I2CSWM_SCL_IN			i2c_port_in(I2C_SCL_PIN, I2C_PORT)
#define I2CSWM_SCL_OUT		i2c_port_out(I2C_SCL_PIN, I2C_PORT)
#define I2CSWM_SETSCL()		PORT_WriteBit(I2C_PORT, I2C_SCL_PIN, Bit_SET)
#define I2CSWM_CLEARSCL() PORT_WriteBit(I2C_PORT, I2C_SCL_PIN, Bit_RESET)
#define I2CSWM_GETSCL()		(PORT_ReadInputData(I2C_PORT) & I2C_SCL_PIN)

#define I2CSWM_SDA_IN			i2c_port_in(I2C_SDA_PIN, I2C_PORT)
#define I2CSWM_SDA_OUT		i2c_port_out(I2C_SDA_PIN, I2C_PORT)
#define I2CSWM_SETSDA()		PORT_WriteBit(I2C_PORT, I2C_SDA_PIN, Bit_SET)
#define I2CSWM_CLEARSDA()	PORT_WriteBit(I2C_PORT, I2C_SDA_PIN, Bit_RESET)
#define I2CSWM_GETSDA()		(PORT_ReadInputData(I2C_PORT) & I2C_SDA_PIN)

#define true	1
#define false	0
#define NULL 0

static const unsigned char I2CSWM_DIRECTION_TX = 0;
static const unsigned char I2CSWM_DIRECTION_RX = 1;
static const unsigned char I2CSWM_NACK = 1;
static const unsigned char I2CSWM_ACK = 0;

PORT_InitTypeDef PORT_InitStructure;

/*
*/
static void __i2cswm___delay( int n )
{

while( --n > 0 );
	
}

/*
*/


void i2c_Init(void)
{	
	RST_CLK_PCLKcmd(RST_I2C_PORT, ENABLE);

	PORT_InitStructure.PORT_OE = PORT_OE_OUT;	
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	PORT_InitStructure.PORT_PULL_UP = PORT_PULL_UP_OFF;
	PORT_InitStructure.PORT_PULL_DOWN = PORT_PULL_DOWN_OFF;
		
	PORT_InitStructure.PORT_Pin = I2C_SCL_PIN | I2C_SDA_PIN;	
	PORT_Init(I2C_PORT, &PORT_InitStructure);	
}

static void i2c_port_in (unsigned int pin, MDR_PORT_TypeDef * port)
{
	PORT_InitStructure.PORT_OE = PORT_OE_IN;	
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	
	PORT_InitStructure.PORT_Pin = pin;
	PORT_Init(I2C_PORT, &PORT_InitStructure);	
}

static void i2c_port_out (unsigned int pin, MDR_PORT_TypeDef *  port)
{
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;	
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	
	PORT_InitStructure.PORT_Pin = pin;
	PORT_Init(I2C_PORT, &PORT_InitStructure);	
}


static void __i2cswm___start()
{
I2CSWM_SETSDA();
I2CSWM_SETSCL();

I2CSWM_CLEARSDA();
__i2cswm___delay(I2CSWM_DELAY);
I2CSWM_CLEARSCL();
}


/*
*/
static void __i2cswm___stop()
{
I2CSWM_CLEARSDA();
I2CSWM_SETSCL();
__i2cswm___delay(I2CSWM_DELAY);

I2CSWM_SETSDA();
}

/*
*/
static unsigned char __i2cswm___clock()
{
unsigned char l, Bit_SET;
I2CSWM_SDA_IN;
I2CSWM_SETSCL();
__i2cswm___delay(I2CSWM_DELAY);

l = I2CSWM_GETSDA() == Bit_SET ? 1 : 0;

I2CSWM_CLEARSCL();
__i2cswm___delay(I2CSWM_DELAY);
I2CSWM_SDA_OUT;
return l;
}

/*
*/
static unsigned char __i2cswm___sendb(unsigned char b)
{
	unsigned char i;
unsigned char db = 0x80;
for(i = 0; i < 8; i++)
{
if( b & db )
{
I2CSWM_SETSDA();
}else
{
I2CSWM_CLEARSDA();
}
__i2cswm___clock();
db >>= 1;
}

/* Receive ACK */
I2CSWM_SETSDA(); 
return __i2cswm___clock();
}


/*
*/
static unsigned char __i2cswm___recvb( unsigned char ack )
{
unsigned char rb = 0;
unsigned char db = 0x80;
	unsigned char i;
for(i = 0; i < 8; i++)
{
if( __i2cswm___clock() )
rb |= db;
db >>= 1;
}

if( ack == I2CSWM_ACK )
{

I2CSWM_CLEARSDA();
__i2cswm___clock();
I2CSWM_SETSDA();
}else
{

I2CSWM_SETSDA();

__i2cswm___clock();
}
return rb;
}



/*
*/
void __i2cswm___reset()
{int i;
I2CSWM_SETSDA();
for(i = 0; i < 15; i++)
{
I2CSWM_SETSCL();
__i2cswm___delay(I2CSWM_DELAY);
I2CSWM_CLEARSCL();
__i2cswm___delay(I2CSWM_DELAY);
}
I2CSWM_SETSDA();
I2CSWM_SETSCL();
}



/*
*/
int __i2cswm___recvex(const unsigned short saddr, unsigned char* lpdata1, unsigned int size1, unsigned char* lpdata2, unsigned int size2)
{int i;
__i2cswm___start();
if( lpdata1 != NULL )
{
if( __i2cswm___sendb( saddr | I2CSWM_DIRECTION_TX) == I2CSWM_NACK )
{
__i2cswm___stop();
return false;
}

for(i = 0; i < size1; i++)
{
if( __i2cswm___sendb(*lpdata1++) == I2CSWM_NACK )
{
__i2cswm___stop();
return false;
}
}
__i2cswm___start();
}

if( lpdata2 != NULL )
{
if( __i2cswm___sendb( saddr | I2CSWM_DIRECTION_RX) == I2CSWM_NACK )
{
__i2cswm___stop();
return false;
}
I2CSWM_SETSCL();
for(i = 1; i < size2; i++)
{
*lpdata2 = __i2cswm___recvb( I2CSWM_ACK );
lpdata2++;
}
*lpdata2 = __i2cswm___recvb( I2CSWM_NACK );
lpdata2++;
}
__i2cswm___stop();
return true;
}



/*
*/
int __i2cswm___sendex(const uint16_t saddr, uint8_t* lpdata1, uint32_t size1, uint8_t* lpdata2, uint32_t size2)
{int i;
__i2cswm___start();
if( lpdata1 != NULL )
{
if( __i2cswm___sendb( saddr | I2CSWM_DIRECTION_TX) == I2CSWM_NACK )
{
__i2cswm___stop();
return false;
}

for(i = 0; i < size1; i++)
{
if( __i2cswm___sendb(*lpdata1++) == I2CSWM_NACK )
{
__i2cswm___stop();
return false;
}
}
}

if( lpdata2 != NULL )
{
for(i = 0; i < size2; i++)
{
if( __i2cswm___sendb(*lpdata2++) == I2CSWM_NACK )
{
__i2cswm___stop();
return false;
}
}
}
__i2cswm___stop();
return true;
}


unsigned char mcu_i2c_write_byte(uint8_t device_address, uint8_t lpdata1)
{

__i2cswm___start();
if( lpdata1 != NULL )
{
if( __i2cswm___sendb( device_address | I2CSWM_DIRECTION_TX) == I2CSWM_NACK )
{
__i2cswm___stop();
return false;
}

if( __i2cswm___sendb(lpdata1) == I2CSWM_NACK )
{
__i2cswm___stop();
return false;
}

}
__i2cswm___stop();
return true;
}

unsigned char mcu_i2c_write_two_byte (uint8_t device_address, uint8_t lpdata1, uint8_t lpdata2)
{

__i2cswm___start();
if( lpdata1 != NULL )
{
if( __i2cswm___sendb( device_address | I2CSWM_DIRECTION_TX) == I2CSWM_NACK )
{
__i2cswm___stop();
return false;
}

if( __i2cswm___sendb(lpdata1) == I2CSWM_NACK )
{
__i2cswm___stop();
return false;
}
}
if( lpdata2 != NULL )
{
if( __i2cswm___sendb(lpdata2) == I2CSWM_NACK )
{
__i2cswm___stop();
return false;
}
}
__i2cswm___stop();
return true;
}

uint8_t mcu_i2c_read_byte(uint8_t saddr)
{	
	uint8_t data;
__i2cswm___start();

if( __i2cswm___sendb( saddr | I2CSWM_DIRECTION_TX) == I2CSWM_NACK )
{
__i2cswm___stop();
return false;
}
__i2cswm___start();
I2CSWM_SETSCL();
data = __i2cswm___recvb( I2CSWM_ACK );
__i2cswm___stop();
return data;
}

// Example
/*
uint_fast8_t eeprom_read(uint16_t addr, uint8_t* buf, size_t size)
{
addr = SWP(addr);

uint_fast8_t rep_cnt = 3;
uint_fast8_t rc = FALSE;
while( --rep_cnt > 0 && rc == FALSE )
{
rc = i2c_recvex(EEPROM_SADDR, (uint8_t*)&addr, sizeof(addr), buf, size);
}
return rc;
}


uint_fast8_t eeprom_write(uint16_t addr, uint8_t* buf, size_t size)
{
addr = SWP(addr);
uint_fast8_t rep_cnt = 3;
uint_fast8_t rc = FALSE;
while( --rep_cnt > 0 && rc == FALSE )
{
rc = i2c_sendex(EEPROM_SADDR, (uint8_t*)&addr, sizeof(addr), buf, size);
}
return rc;
}*/