
/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2008
 *
 *    File name   : sd_ll_spi1.h
 *    Description : SD/MMC low level SPI1 driver
 *
 *    History :
 *    1. Date        : April 10, 2008
 *       Author      : Stanimir Bonev
 *       Description : Create
 *
 *    $Revision: #2 $
 **************************************************************************/
#include "multiplexer.h"
#include "spi.h"
#include "itmr.h"

#define _SSP_FIFO_SIZE 8

void SdDly_1ms (unsigned int Delay)
{
	unsigned char i;
	for (i=0;i<Delay;i++)
	{
		itmr_Delay(1);
	}
}

/*************************************************************************
 * Function Name: SdPowerOn
 * Parameters: none
 * Return: none
 *
 * Description: Set power off state
 *
 *************************************************************************/
void SdPowerOn (void)
{
  SdDly_1ms(1);
}
/*************************************************************************
 * Function Name: SdPowerOff
 * Parameters: none
 * Return: none
 *
 * Description: Set power off state
 *
 *************************************************************************/
void SdPowerOff (void)
{
  SdDly_1ms(1);
}
/*************************************************************************
 * Function Name: SdChipSelect
 * Parameters: Boolean Select
 * Return: none
 *
 * Description: SD/MMC Chip select control
 * Select = true  - Chip is enable
 * Select = false - Chip is disable
 *
 *************************************************************************/
void SdChipSelect (void)
{
	MICROSD_CS;
}
/*************************************************************************
 * Function Name: SdPresent
 * Parameters: none
 * Return: Boolean - true cart present
 *                 - false cart no present
 *
 * Description: SD/MMC precent check
 *
 *************************************************************************/
/*Boolean SdPresent (void)
{
  int x;
  GPIO_InitTypeDef GPIO_InitStructure;

  // Configure the MMC_CS pin as input
  GPIO_InitStructure.GPIO_Pin = MMC_CS_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(MMC_CS_PORT, &GPIO_InitStructure);
  
  x = GPIO_ReadInputDataBit(MMC_CS_PORT,MMC_CS_PIN);

  GPIO_ResetBits(MMC_CS_PORT, MMC_CS_PIN);
  // Configure the MMC_CS pin as output
  GPIO_InitStructure.GPIO_Pin = MMC_CS_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(MMC_CS_PORT, &GPIO_InitStructure);
  return x;
}*/

/*************************************************************************
 * Function Name: SdWriteProtect
 * Parameters: none
 * Return: Boolean - true cart is protected
 *                 - false cart no protected
 *
 * Description: SD/MMC Write protect check
 *
 *************************************************************************/
/*Boolean SdWriteProtect (void)
{
  return false;
}*/

/*************************************************************************
 * Function Name: SdSetClockFreq
 * Parameters: Int32U Frequency
 * Return: Int32U
 *
 * Description: Set SPI ckl frequency
 *
 *************************************************************************/
/*unsigned int SdSetClockFreq (unsigned int Frequency)
{
RCC_ClocksTypeDef RCC_Clocks;
unsigned int Div;

  RCC_GetClocksFreq(&RCC_Clocks);

  for(Div = 0; Div < 8; Div++)
  {
    if(Frequency * (2<<Div) > RCC_Clocks.PCLK1_Frequency)
    {
      break;
    }
  }

  if(8 <= Div) return (-1UL);

  unsigned int tmpreg;
  tmpreg = MMC_SPI->CR1;
  tmpreg &= ~SPI_BaudRatePrescaler_256;
  tmpreg |= Div<<3;
  MMC_SPI->CR1 = tmpreg;
  // Return real frequency
  return(RCC_Clocks.PCLK1_Frequency/(2<<Div));
}*/

/*************************************************************************
 * Function Name: SdInit
 * Parameters: none
 * Return: none
 *
 * Description: Init SPI, Cart Present, Write Protect and Chip select pins
 *
 *************************************************************************/
/*void SdInit (void)
{
  SPI_InitTypeDef   SPI_InitStructure;
  GPIO_InitTypeDef  GPIO_InitStructure;

  // SPI and GPIO enable clock
  RCC_AHB1PeriphClockCmd(MMC_CS_CLK | MMC_SPI_CLK | MMC_SPI_GPIO_CLK, ENABLE);
  RCC_APB1PeriphClockCmd(MMC_SPI_CLK, ENABLE);

  GPIO_PinAFConfig(MMC_SPI_GPIO, MMC_PIN_SCK_SOURCE, GPIO_AF_SPI3);
  GPIO_PinAFConfig(MMC_SPI_GPIO, MMC_PIN_MISO_SOURCE, GPIO_AF_SPI3);
  GPIO_PinAFConfig(MMC_SPI_GPIO, MMC_PIN_MOSI_SOURCE, GPIO_AF_SPI3);

  // Init CS pin
  GPIO_InitStructure.GPIO_Pin = MMC_CS_PIN;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT; // input to enable card detect
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(MMC_CS_PORT, &GPIO_InitStructure);

  // init SCK, MISO and MOSI pins
  GPIO_InitStructure.GPIO_Pin = MMC_PIN_SCK | MMC_PIN_MOSI | MMC_PIN_MISO;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(MMC_SPI_GPIO, &GPIO_InitStructure);

  SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_256;
  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 7;
  SPI_Init(MMC_SPI, &SPI_InitStructure);
  
  // Clock Freq. Identification Mode < 400kHz
  SdSetClockFreq(IdentificationModeClock);
  
  
  SPI_Cmd(MMC_SPI, ENABLE);

}*/

/*************************************************************************
 * Function Name: SdTranserByte
 * Parameters: Int8U ch
 * Return: Int8U
 *
 * Description: Read byte from SPI
 *
 *************************************************************************/
unsigned char SdTranserByte (unsigned char ch)
{
 unsigned short rb;
	rb = SendAndRcvByte(ch);
	return rb;
}

/*************************************************************************
 * Function Name: SdSendBlock
 * Parameters: pInt8U pData, Int32U Size
 *
 * Return: void
 *
 * Description: Read byte from SPI
 *
 *************************************************************************/
void SdSendBlock (unsigned char pData, unsigned int Size)
{
	unsigned int Dummy;
  unsigned int OutCount = Size;
  while (OutCount--)
  {
		SendAndRcvByte(pData++);
    Dummy = SendAndRcvByte(0);
  }
}

/*************************************************************************
 * Function Name: SdReceiveBlock
 * Parameters: pInt8U pData, Int32U Size
 *
 * Return: void
 *
 * Description: Read byte from SPI
 *
 *************************************************************************/
void SdReceiveBlock (unsigned char pData, unsigned int Size)
{
unsigned int InCount = Size;
  while (InCount--)
  {
		SendAndRcvByte(0xFF);
    pData += SendAndRcvByte(0);
  }
}

/*************************************************************************
 * Function Name: SdDly_1ms
 * Parameters: Int32U Delay
 * Return: none
 *
 * Description: Delay [msec]
 *
 *************************************************************************/

