/* Which SPI use */
#define _SD_SPI 3

#if _SD_SPI == 1
	#define SD_SPI       SPI1
	#define SD_CS_PIN    GPIO_Pin_4     // PA4
	#define SD_SCK_PIN   GPIO_Pin_5     // PA5
	#define SD_MISO_PIN  GPIO_Pin_6     // PA6
	#define SD_MOSI_PIN  GPIO_Pin_7     // PA7
	#define SD_PORT      GPIOA
	#define SD_CS_PORT   SD_PORT
#elif _SD_SPI == 2
	#define SD_SPI       SPI2
	#define SD_CS_PIN    GPIO_Pin_12    // PB12
	#define SD_SCK_PIN   GPIO_Pin_13    // PB13
	#define SD_MISO_PIN  GPIO_Pin_14    // PB14
	#define SD_MOSI_PIN  GPIO_Pin_15    // PB15
	#define SD_PORT      GPIOB
	#define SD_CS_PORT   SD_PORT
#elif _SD_SPI == 3
	#define SD_SPI       SPI3
	#define SD_CS_PIN    GPIO_Pin_6     // PB6
	#define SD_SCK_PIN   GPIO_Pin_3     // PB3  (JTDO)
	#define SD_MISO_PIN  GPIO_Pin_4     // PB4  (NJTRST)
	#define SD_MOSI_PIN  GPIO_Pin_5     // PB5
	#define SD_PORT      GPIOB
	#define SD_CS_PORT   SD_PORT
#endif


/* SDIO Commands  Index */
#define SD_CMD_GO_IDLE_STATE                       ((unsigned char)0)
#define SD_CMD_SEND_OP_COND                        ((unsigned char)1)
#define SD_CMD_ALL_SEND_CID                        ((unsigned char)2)
#define SD_CMD_SET_REL_ADDR                        ((unsigned char)3) /* SD_CMD_SEND_REL_ADDR for SD Card */
#define SD_CMD_SET_DSR                             ((unsigned char)4)
#define SD_CMD_SD_CMD_SEN_OP_COND                  ((unsigned char)5)
#define SD_CMD_HS_SWITCH                           ((unsigned char)6)
#define SD_CMD_SEL_DESEL_CARD                      ((unsigned char)7)
#define SD_CMD_HS_SEND_EXT_CSD                     ((unsigned char)8)
#define SD_CMD_SEND_CSD                            ((unsigned char)9)
#define SD_CMD_SEND_CID                            ((unsigned char)10)
#define SD_CMD_READ_DAT_UNTIL_STOP                 ((unsigned char)11) /* SD Card doesn't support it */
#define SD_CMD_STOP_TRANSMISSION                   ((unsigned char)12)
#define SD_CMD_SEND_STATUS                         ((unsigned char)13)
#define SD_CMD_HS_BUSTEST_READ                     ((unsigned char)14)
#define SD_CMD_GO_INACTIVE_STATE                   ((unsigned char)15)
#define SD_CMD_SET_BLOCKLEN                        ((unsigned char)16)
#define SD_CMD_READ_SINGLE_BLOCK                   ((unsigned char)17)
#define SD_CMD_READ_MULT_BLOCK                     ((unsigned char)18)
#define SD_CMD_HS_BUSTEST_WRITE                    ((unsigned char)19)
#define SD_CMD_WRITE_DAT_UNTIL_STOP                ((unsigned char)20) /* SD Card doesn't support it */
#define SD_CMD_SET_BLOCK_COUNT                     ((unsigned char)23) /* SD Card doesn't support it */
#define SD_CMD_WRITE_SINGLE_BLOCK                  ((unsigned char)24)
#define SD_CMD_WRITE_MULT_BLOCK                    ((unsigned char)25)
#define SD_CMD_PROG_CID                            ((unsigned char)26) /* reserved for manufacturers */
#define SD_CMD_PROG_CSD                            ((unsigned char)27)
#define SD_CMD_SET_WRITE_PROT                      ((unsigned char)28)
#define SD_CMD_CLR_WRITE_PROT                      ((unsigned char)29)
#define SD_CMD_SEND_WRITE_PROT                     ((unsigned char)30)
#define SD_CMD_SD_ERASE_GRP_START                  ((unsigned char)32) /* To set the address of the first write
                                                                  block to be erased. (For SD card only) */
#define SD_CMD_SD_ERASE_GRP_END                    ((unsigned char)33) /* To set the address of the last write block of the
                                                                  continuous range to be erased. (For SD card only) */
#define SD_CMD_ERASE_GRP_START                     ((unsigned char)35) /* To set the address of the first write block to be erased.
                                                                  (For MMC card only spec 3.31) */

#define SD_CMD_ERASE_GRP_END                       ((unsigned char)36) /* To set the address of the last write block of the
                                                                  continuous range to be erased. (For MMC card only spec 3.31) */

#define SD_CMD_ERASE                               ((unsigned char)38)
#define SD_CMD_FAST_IO                             ((unsigned char)39) /* SD Card doesn't support it */
#define SD_CMD_GO_IRQ_STATE                        ((unsigned char)40) /* SD Card doesn't support it */
#define SD_CMD_LOCK_UNLOCK                         ((unsigned char)42)
#define SD_CMD_APP_CMD                             ((unsigned char)55)
#define SD_CMD_GEN_CMD                             ((unsigned char)56)
#define SD_CMD_READ_OCR                            ((unsigned char)58) /* Read OCR register */
#define SD_CMD_CRC_ON_OFF                          ((unsigned char)59) /* On/Off CRC check by SD Card */
#define SD_CMD_NO_CMD                              ((unsigned char)64)

/* Following commands are SD Card Specific commands.
   SD_CMD_APP_CMD should be sent before sending these
   commands. */
#define SD_CMD_APP_SD_SET_BUSWIDTH                 ((unsigned char)6)  /* For SD Card only */
#define SD_CMD_SD_APP_STAUS                        ((unsigned char)13) /* For SD Card only */
#define SD_CMD_SD_APP_SEND_NUM_WRITE_BLOCKS        ((unsigned char)22) /* For SD Card only */
#define SD_CMD_SD_APP_OP_COND                      ((unsigned char)41) /* For SD Card only */
#define SD_CMD_SD_APP_SET_CLR_CARD_DETECT          ((unsigned char)42) /* For SD Card only */
#define SD_CMD_SD_APP_SEND_SCR                     ((unsigned char)51) /* For SD Card only */
#define SD_CMD_SD_CMD_RW_DIRECT                    ((unsigned char)52) /* For SD I/O Card only */
#define SD_CMD_SD_CMD_RW_EXTENDED                  ((unsigned char)53) /* For SD I/O Card only */

/* Following commands are SD Card Specific security commands.
   SD_CMD_APP_CMD should be sent before sending these commands. */
#define SD_CMD_SD_APP_GET_MKB                      ((unsigned char)43) /* For SD Card only */
#define SD_CMD_SD_APP_GET_MID                      ((unsigned char)44) /* For SD Card only */
#define SD_CMD_SD_APP_SET_CER_RN1                  ((unsigned char)45) /* For SD Card only */
#define SD_CMD_SD_APP_GET_CER_RN2                  ((unsigned char)46) /* For SD Card only */
#define SD_CMD_SD_APP_SET_CER_RES2                 ((unsigned char)47) /* For SD Card only */
#define SD_CMD_SD_APP_GET_CER_RES1                 ((unsigned char)48) /* For SD Card only */
#define SD_CMD_SD_APP_SECURE_READ_MULTIPLE_BLOCK   ((unsigned char)18) /* For SD Card only */
#define SD_CMD_SD_APP_SECURE_WRITE_MULTIPLE_BLOCK  ((unsigned char)25) /* For SD Card only */
#define SD_CMD_SD_APP_SECURE_ERASE                 ((unsigned char)38) /* For SD Card only */
#define SD_CMD_SD_APP_CHANGE_SECURE_AREA           ((unsigned char)49) /* For SD Card only */
#define SD_CMD_SD_APP_SECURE_WRITE_MKB             ((unsigned char)48) /* For SD Card only */

/* Masks for R6 Response */
#define SD_CHECK_PATTERN                ((unsigned int)0x000001AA)

/* Supported memory cards */
#define SD_UNKNOWN_SD_CARD              ((unsigned int)0x0)
#define SD_STD_CAPACITY_SD_CARD_V1_0    ((unsigned int)0x1)
#define SD_STD_CAPACITY_SD_CARD_V2_0    ((unsigned int)0x2)
#define SD_MULTIMEDIA_CARD              ((unsigned int)0x3)
#define SD_HIGH_CAPACITY_SD_CARD        ((unsigned int)0x4)


extern unsigned char  SD_CardType;
extern unsigned int SD_CardCapacity;
extern unsigned char  SD_MaxBusClkFreq;
extern unsigned char  SD_CSD[16];
extern unsigned char  SD_CID[16];
extern unsigned char  SD_sector[512];
extern unsigned short SD_CRC16_rcv;
extern unsigned short SD_CRC16_cmp;


void SD_Init(void);
unsigned char SD_SendRecv(unsigned char data);
unsigned char SD_SendCmd(unsigned char cmd, unsigned int arg);
unsigned char SD_CardInit(void);
unsigned char SD_Read_CSD(void);
unsigned char SD_Read_CID(void);
unsigned char SD_Read_Block(unsigned int addr);
void SD_TEST (void);
