
#include "MDR1986VE1T.h"
#include "i2c_soft.h"
#include "multiplexer.h"
#include "bmp085.h"
#include "itmr.h"
#include <math.h>
#include "uart.h"

#define BAROMETER_I2C_ADDRESS 0xEE

#define BMP180_CTRL_MEAS_REG			(0xF4)
#define BMP180_ADC_OUT_REG				(0xF6)

/* temperature measurement */
#define BMP180_T_MEASURE			(0x2E)
/* pressure measurement*/
#define BMP180_P_MEASURE			(0x74)
#define BMP085_SETTINGS_OSS		1

#define RD_BMP085_ADDR		0xEF

static uint32_t rawTempValue;

void read_calibr_coefficients(void);
int sprintf( char *, const char *, ...);
int32_t CalculateRealPreassure(int32_t rawTemp, int32_t rawPressure);
/*
enum bmp085_settings
{
	BMP085_SETTINGS_DEVICE_ADDRESS = 0x77, // адрес
	BMP085_SETTINGS_OSS = 0, //
	BMP085_SETTINGS
};*/


	typedef struct {
		uint16_t raw[11];	
		int16_t ac1;
		int16_t ac2;
		int16_t ac3;
		uint16_t ac4;
		uint16_t ac5;
		uint16_t ac6;
		int16_t b1;
		int16_t b2;
		int16_t mb;
		int16_t mc;
		int16_t md;
		int32_t b5;
	
} bmp085_calibration_coefficients_type;

struct bmp085_type
{
	// калибровочные коэффициенты
	bmp085_calibration_coefficients_type calibration_coefficients;

	// нескомпенсированная температура
	int16_t uncompensated_temperature;

	// нескомпенсированное  давление
	int32_t uncompensated_pressure;
};

struct bmp085_type bmp085;

//------------------------------------------------------------------------------
uint8_t bmp085_init(void)
{
	read_calibr_coefficients();
	
	return  bmp085.calibration_coefficients.ac1;
}
void read_calibr_coefficients(void)
{		
	bmp085.calibration_coefficients.ac1 = mcu_i2c_read_Int(0xEE, 0xAA);
	bmp085.calibration_coefficients.ac2 = mcu_i2c_read_Int(0xEE, 0xAC);	
	bmp085.calibration_coefficients.ac3 = mcu_i2c_read_Int(0xEE, 0xAE);	
	bmp085.calibration_coefficients.ac4 = mcu_i2c_read_Int(0xEE, 0xB0);	
	bmp085.calibration_coefficients.ac5 = mcu_i2c_read_Int(0xEE, 0xB2);
	bmp085.calibration_coefficients.ac6 = mcu_i2c_read_Int(0xEE, 0xB4);
	bmp085.calibration_coefficients.b1 = mcu_i2c_read_Int(0xEE, 0xB6);
	bmp085.calibration_coefficients.b2 = mcu_i2c_read_Int(0xEE, 0xB8);
	bmp085.calibration_coefficients.mb = mcu_i2c_read_Int(0xEE, 0xBA);
	bmp085.calibration_coefficients.mc = mcu_i2c_read_Int(0xEE, 0xBC);
	bmp085.calibration_coefficients.md = mcu_i2c_read_Int(0xEE, 0xBE);
	
}
void BMP180_StartPressureMeasure(void) {
	mcu_i2c_write_two_byte(BAROMETER_I2C_ADDRESS, BMP180_CTRL_MEAS_REG, BMP180_P_MEASURE | (BMP085_SETTINGS_OSS << 6));
}

void BMP180_StartTemperatureMeasure(void) {
	mcu_i2c_write_two_byte(BAROMETER_I2C_ADDRESS, BMP180_CTRL_MEAS_REG, BMP180_T_MEASURE);
}
//------------------------------------------------------------------------------
int32_t bmp085_read_pressure(void)
{	int32_t pData;
	 
    uint8_t dataRead[3];
    uint32_t rawValue;
			BMP180_StartPressureMeasure();
	itmr_Delay(30);
	
    mcu_i2c_read_len(BAROMETER_I2C_ADDRESS, BMP180_ADC_OUT_REG, dataRead, 3);
    rawValue = dataRead[0]<<(8+BMP085_SETTINGS_OSS) | dataRead[1]<<(BMP085_SETTINGS_OSS) | dataRead[2]>>(8-BMP085_SETTINGS_OSS);

    pData = CalculateRealPreassure(rawTempValue, rawValue);

    return pData;
}
int16_t bmp085_read_temperature(void)
{
    int16_t status = 0;
  // uint8_t dataRead[2];
	MULTI_CS_HIGH;
		BMP180_StartTemperatureMeasure();
		itmr_Delay(5);
    status = mcu_i2c_read_Int(BAROMETER_I2C_ADDRESS, BMP180_ADC_OUT_REG);
    rawTempValue = status;

    return status;
}
void coefficients_read (void)
{
	char buffer[32];
	
	uart_SendStr("---cal data start---");
  uart_SendStr("ac1:");
	sprintf(buffer, " %d \n\r", bmp085.calibration_coefficients.ac1);
	uart_SendBuf(buffer,32);

	uart_SendStr("ac2:");
	sprintf(buffer, " %d \n\r", bmp085.calibration_coefficients.ac2);
	uart_SendBuf(buffer,32);

	uart_SendStr("ac3:");
	sprintf(buffer, " %d \n\r", bmp085.calibration_coefficients.ac3);
	uart_SendBuf(buffer,32);

	uart_SendStr("ac4:");
	sprintf(buffer, " %d \n\r", bmp085.calibration_coefficients.ac4);
	uart_SendBuf(buffer,32);

	uart_SendStr("ac5:");
	sprintf(buffer, " %d \n\r", bmp085.calibration_coefficients.ac5);
	uart_SendBuf(buffer,32);

	uart_SendStr("ac6:");
	sprintf(buffer, " %d \n\r", bmp085.calibration_coefficients.ac6);
	uart_SendBuf(buffer,32);

	uart_SendStr("b1:");
	sprintf(buffer, " %d \n\r", bmp085.calibration_coefficients.b1);
	uart_SendBuf(buffer,32);

	uart_SendStr("b2:");
	sprintf(buffer, " %d \n\r", bmp085.calibration_coefficients.b2);
	uart_SendBuf(buffer,32);
/*
	uart_SendStr("b5:");
	sprintf(buffer, " %d \n\r", bmp085.calibration_coefficients.b5);
	uart_SendBuf(buffer,32);
*/
	uart_SendStr("mb:");
	sprintf(buffer, " %d \n\r", bmp085.calibration_coefficients.mb);
	uart_SendBuf(buffer,32);

	uart_SendStr("mc:");
	sprintf(buffer, " %d \n\r", bmp085.calibration_coefficients.mc);
	uart_SendBuf(buffer,32);

	uart_SendStr("md:");
	sprintf(buffer, " %d \n\r", bmp085.calibration_coefficients.md);
	uart_SendBuf(buffer,32);

  uart_SendStr("---cal data end---");
}

 unsigned int bmp085_getPressure(unsigned char high)
	 {   
  long TruePressure;
	int Pa;

  TruePressure = bmp085_read_pressure(); 
  Pa = TruePressure / pow((1 - (float)high / 4433000), 5.255);
  // converting from float to int32_t truncates toward zero, 1010.999985 becomes 1010 resulting in 1 Pa error (max).  
  // Note that BMP085 abs accuracy from 700...1100hPa and 0..+65ยบC is +-100Pa (typ.)
		 
 
		 return Pa;
}

//------------------------------------------------------------------------------

uint8_t bmp085_readID(void)
{	uint8_t id=0;
	id = mcu_i2c_write_byte(0xEE, 0xD0);
	if (id != 85)
	{
		id = mcu_i2c_write_byte(0xEE, 0xD0);
	}
	return id;
}

int32_t CalculateRealPreassure(int32_t rawTemp, int32_t rawPressure) {
	int32_t X1, X2, X3, B3, B6, p;
	uint32_t B4, B7;

	X1 = (rawTemp - bmp085.calibration_coefficients.ac6) * bmp085.calibration_coefficients.ac5 / 32768;
	X2 = bmp085.calibration_coefficients.mc * 2048 / (X1 + bmp085.calibration_coefficients.md);
	B6 = X1 + X2 - 4000;

	X1 = (bmp085.calibration_coefficients.b2 * (B6 * B6 / 4096)) / 2048;
	X2 = bmp085.calibration_coefficients.ac2 * B6 / 2048;
	X3 = X1 + X2;
	B3 = (((bmp085.calibration_coefficients.ac1 * 4 + X3) << BMP085_SETTINGS_OSS) + 2) / 4;

	X1 = bmp085.calibration_coefficients.ac3 * B6 / 8192;
	X2 = (bmp085.calibration_coefficients.b1 * ( B6 * B6 / 4096)) / 65536;
	X3 = ((X1 + X2) + 2) / 4;
	B4 = bmp085.calibration_coefficients.ac4 * (uint32_t)(X3 + 32768) / 32768;
	B7 = ((uint32_t)rawPressure - B3) * (50000 >> BMP085_SETTINGS_OSS);
	if (B7 < 0x80000000) {
		p = (B7 * 2) / B4;
	}
	else {
		p = (B7 / B4) * 2;
	}
	X1 = (p / 256) * (p / 256);
	X1 = (X1 * 3038) / 65536;
	X2 = (-7357 * p) / 65536;
	p = p + (X1 + X2 +3791) / 16;

	return p;
}
float CalcAltitudeFromPressure(int32_t pressure) {
	float tmp = powf((float)pressure / 101325.0f, 1.0f/5.255f);
	float altitude = 44330.0f * ( 1.0f - tmp);

	return altitude;	//in meters 
}
int32_t get_pressure_mmHg(void)
{
	MULTI_CS_HIGH;
	return bmp085_read_pressure()/133;
}
void test_barometer(void)
{	
	char buffer[32];
	short temperature;
	int pressure, altitude;
  
	MULTI_CS_HIGH;
	
	bmp085_init();
	BMP180_StartPressureMeasure();
	
		pressure = get_pressure_mmHg();
		altitude = CalcAltitudeFromPressure(pressure*133);
		temperature = bmp085_read_temperature();
		temperature = (float)(temperature/1000);
		sprintf(buffer, " %d \n\r", altitude	);
		
		uart_SendBuf(buffer,32);
		
		itmr_Delay(100);
	
}
void barometr_init(void)
{
	MULTI_CS_HIGH;
	
	bmp085_init();
	BMP180_StartPressureMeasure();
	}
