/*
  Arduino Mario Bros Tunes
  With Piezo Buzzer and PWM

  by: Dipto Pratyaksa
*/
 #include "MDR32F9Qx_port.h"
 #include "MDR32F9Qx_rst_clk.h"
 #include "buzzer_melody.h"
 #include "misc.h"
 #include "wdt.h"

void buzz(int targetPin, long frequency, long length);

//Mario main theme melody
int melody[] = {
  NOTE_E5, NOTE_E5, 0, NOTE_E5,
  0, NOTE_C5, NOTE_E5, 0,
  NOTE_G5, 0, 0,  0,
  NOTE_G4, 0, 0, 0,
 
  NOTE_C5, 0, 0, NOTE_G4,
  0, 0, NOTE_E4, 0,
  0, NOTE_A4, 0, NOTE_B4,
  0, NOTE_AS4, NOTE_A4, 0,
 
  NOTE_G4, NOTE_E5, NOTE_G5,
  NOTE_A5, 0, NOTE_F5, NOTE_G5,
  0, NOTE_E5, 0, NOTE_C5,
  NOTE_D5, NOTE_B4, 0, 0,
 
  NOTE_C5, 0, 0, NOTE_G4,
  0, 0, NOTE_E4, 0,
  0, NOTE_A4, 0, NOTE_B4,
  0, NOTE_AS4, NOTE_A4, 0,
 
  NOTE_G4, NOTE_E5, NOTE_G5,
  NOTE_A5, 0, NOTE_F5, NOTE_G5,
  0, NOTE_E5, 0, NOTE_C5,
  NOTE_D5, NOTE_B4, 0, 0
};
//Mario main them tempo
int tempo[] = {
  12, 12, 12, 12,
  12, 12, 12, 12,
  12, 12, 12, 12,
  12, 12, 12, 12,
 
  12, 12, 12, 12,
  12, 12, 12, 12,
  12, 12, 12, 12,
  12, 12, 12, 12,
 
  9, 9, 9,
  12, 12, 12, 12,
  12, 12, 12, 12,
  12, 12, 12, 12,
 
  12, 12, 12, 12,
  12, 12, 12, 12,
  12, 12, 12, 12,
  12, 12, 12, 12,
 
  9, 9, 9,
  12, 12, 12, 12,
  12, 12, 12, 12,
  12, 12, 12, 12,
};
//Underworld melody
int underworld_melody[] = {
  NOTE_C3, NOTE_C4, NOTE_A2, NOTE_A3,
  NOTE_AS2, NOTE_AS3, 0,
  0,
  NOTE_C3, NOTE_C4, NOTE_A2, NOTE_A3,
  NOTE_AS2, NOTE_AS3, 0,
  0,
  NOTE_F2, NOTE_F3, NOTE_D2, NOTE_D3,
  NOTE_DS2, NOTE_DS3, 0,
  0,
  NOTE_F2, NOTE_F3, NOTE_D2, NOTE_D3,
  NOTE_DS2, NOTE_DS3, 0,
  0, NOTE_DS3, NOTE_CS3, NOTE_D3,
  NOTE_CS3, NOTE_DS3,
  NOTE_DS3, NOTE_GS2,
  NOTE_G2, NOTE_CS3,
  NOTE_C3, NOTE_FS3, NOTE_F3, NOTE_E2, NOTE_AS3, NOTE_A3,
  NOTE_GS3, NOTE_DS3, NOTE_B2,
  NOTE_AS2, NOTE_A2, NOTE_GS2,
  0, 0, 0
};
//Underwolrd tempo
int underworld_tempo[] = {
  12, 12, 12, 12,
  12, 12, 6,
  3,
  12, 12, 12, 12,
  12, 12, 6,
  3,
  12, 12, 12, 12,
  12, 12, 6,
  3,
  12, 12, 12, 12,
  12, 12, 6,
  6, 18, 18, 18,
  6, 6,
  6, 6,
  6, 6,
  18, 18, 18, 18, 18, 18,
  10, 10, 10,
  10, 10, 10,
  3, 3, 3
};
 /*
void loop()
{
  //sing the tunes
  sing(1);
  sing(1);
  sing(2);
}*/
int song = 0;
 
void sing(int s) {
  // iterate over the notes of the melody:
	int thisNote, noteDuration, pauseBetweenNotes, size;

  song = s;
  if (song == 2) {
   
    int size = sizeof(underworld_melody) / sizeof(int);
    for ( thisNote = 0; thisNote < size; thisNote++) {
 
      // to calculate the note duration, take one second
      // divided by the note type.
      //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
       noteDuration = 1000 / underworld_tempo[thisNote];
 
      buzz(melodyPin, underworld_melody[thisNote], noteDuration);
 
      // to distinguish the notes, set a minimum time between them.
      // the note's duration + 30% seems to work well:
      pauseBetweenNotes = noteDuration * 1.30;
      itmr_Delay(pauseBetweenNotes);
 
      // stop the tone playing:
      buzz(melodyPin, 0, noteDuration);
			wdt_ReLoad();
    }
 
  } else {
 
    
     size = sizeof(melody) / sizeof(int);
    for (thisNote = 0; thisNote < size; thisNote++) {
 
      // to calculate the note duration, take one second
      // divided by the note type.
      //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
      noteDuration = 1000 / tempo[thisNote];
 
      buzz(melodyPin, melody[thisNote], noteDuration);
 
      // to distinguish the notes, set a minimum time between them.
      // the note's duration + 30% seems to work well:
      pauseBetweenNotes = noteDuration * 1.30;
      itmr_Delay(pauseBetweenNotes);
 
      // stop the tone playing:
      buzz(melodyPin, 0, noteDuration);
			wdt_ReLoad();
    }
  }
}
 
void buzz(int targetPin, long frequency, long length) {
  long delayValue, i, numCycles;

	
  delayValue = 1000000 / frequency; // calculate the delay value between transitions
  //// 1 second's worth of microseconds, divided by the frequency, then split in half since
  //// there are two phases to each cycle
  numCycles = frequency * length/1000; // calculate the number of cycles for proper timing
  
	//// multiply frequency, which is really cycles per second, by the number of seconds to
  //// get the total number of cycles to produce
  for (i = 0; i < numCycles; i++) { // for the calculated length of time...
    PORT_WriteBit(MDR_PORTB, PORT_Pin_2, Bit_SET); // write the buzzer pin high to push out the diaphram
    delay(delayValue*10); // wait for the calculated delay value
    PORT_WriteBit(MDR_PORTB, PORT_Pin_2, Bit_RESET); // write the buzzer pin low to pull back the diaphram
    delay(delayValue*10); // wait again or the calculated delay value
  }
  
 
}
