
#include "ds1339_RTC.h"
#include "i2c_soft.h"
#include "uart.h"
#include "itmr.h"
#include "multiplexer.h"
#include "buzzer_melody.h"

#define BCD2BIN(val) ((val/16*10)+(val%16))
#define BIN2BCD(val) ((val/10*16)+(val%10))





// Cumulative number of days elapsed at the start of each month, assuming a normal (non-leap) year.
const unsigned int monthdays[] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
static unsigned char stime = 0;

void rtc_write_time(unsigned char reg, unsigned char time)
{	
	i2c_start();
	i2c_wr_U8(DEVICE_ADDR_WR);
	i2c_wr_U8(reg);
	i2c_wr_U8(BIN2BCD(time));
	i2c_stop();
}


void rtc_null ( void)
{	
	PRESSURE_CS;
		i2c_start();
	i2c_wr_U8(DEVICE_ADDR_WR);
	i2c_wr_U8(0);
	
	i2c_wr_U8( 0);
	i2c_wr_U8( 0);
	i2c_wr_U8( 0);
	i2c_wr_U8( 0);
	i2c_wr_U8( 0);
	i2c_wr_U8( 0);
	i2c_wr_U8( 0);
	i2c_stop();
}
void rtc_set(unsigned char date, unsigned char month, unsigned char year, unsigned char hour, unsigned char min, unsigned char sec)
{
	i2c_start();
	i2c_wr_U8(DEVICE_ADDR_WR);
	i2c_wr_U8(0);
	
	i2c_wr_U8( BIN2BCD(sec));
	i2c_wr_U8( BIN2BCD(min));
	i2c_wr_U8( BIN2BCD(hour));
	i2c_wr_U8( BIN2BCD(7));
	i2c_wr_U8( BIN2BCD(date));
	i2c_wr_U8( BIN2BCD(month));
	i2c_wr_U8( BIN2BCD(year));
	i2c_stop();
	itmr_Delay(100);
	ds1339_read_time();
}
void alarm_set(unsigned char hour, unsigned char min, unsigned char sec)
{
	i2c_start();
	i2c_wr_U8(DEVICE_ADDR_WR);
	i2c_wr_U8(0x07);
	
	i2c_wr_U8( BIN2BCD(sec));
	i2c_wr_U8( BIN2BCD(min));
	i2c_wr_U8( BIN2BCD(hour));
	i2c_wr_U8( BIN2BCD(0x80));		// alarm whem hours, minutes and seconds matches
	
	i2c_stop();	
}
unsigned char alarm_status (void)
{ unsigned char status = 0;
	
	if (itmr_IsTime(stime, 1000ul)) {
	i2c_start();
	i2c_wr_U8(DEVICE_ADDR_WR);
	i2c_wr_U8(0x0F);
	i2c_start();
	i2c_wr_U8(DEVICE_ADDR_RD);
	status = i2c_rd_U8(1);
	
	i2c_stop();
	if ((status & 0x11) == 0x01) return 1;		// alarm match
	else return 0;
	}
	else return 0;
	
}
void clear_alarm(void)
{
	i2c_start();
	i2c_wr_U8(DEVICE_ADDR_WR);
	i2c_wr_U8(0x0F);
	i2c_wr_U8(0);
	i2c_stop();
	stime = itmr_GetTick();
}
unsigned char rtc_get_time (unsigned char reg)
{	unsigned char time = 0;
	
	i2c_start();
	i2c_wr_U8(DEVICE_ADDR_WR);
	i2c_wr_U8(reg);
	i2c_start();
	i2c_wr_U8(DEVICE_ADDR_RD);
	time = i2c_rd_U8(1);
	
	i2c_stop();
	return BCD2BIN(time);
}


TTime ds1339_read_time (void)
{ TTime rtc;
	PRESSURE_CS;
	rtc.sec = rtc_get_time(RTC_SEC_REG_ADDR)&0x7F;
	rtc.min = rtc_get_time(RTC_MIN_REG_ADDR)&0x7F;
	rtc.hour = rtc_get_time(RTC_HR_REG_ADDR)&0x3F;
	rtc.date = rtc_get_time(RTC_DATE_REG_ADDR)&0x3F;
	rtc.month = rtc_get_time(RTC_MON_REG_ADDR)&0x1F;
	rtc.year = rtc_get_time(RTC_YR_REG_ADDR)&0xFF;
	
	rtc.msec = 0;
	return rtc;
}
void system_time_synchronize(void)
{	TTime rtc;
	
	rtc = ds1339_read_time();
	rtc.year = rtc.year+2000;
	rtc_TimeSet(&rtc);
}
void alarm_Work(void)
{
	if (alarm_status()) {
		sing(1);
		clear_alarm();
	}
}
