#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"
#include "itmr.h"

#define SDA_LN          PORT_Pin_4                     //SDA
#define I2C_SDA_PORT    MDR_PORTE                          

#define SCL_LN          PORT_Pin_3                      //SCL
#define I2C_SCL_PORT   	MDR_PORTE                          

#define sda_read			(PORT_ReadInputData(I2C_SDA_PORT) & SDA_LN)
#define scl_read			(PORT_ReadInputData(I2C_SDA_PORT) & SCL_LN)

#define pause			100

#define IN_LN           0                               
#define OUT_LN          1                               

uint8_t i2c_error;                           			
PORT_InitTypeDef PORT_InitStructure;

void i2c_init(void);
void i2c_start(void);
void i2c_stop(void);
void i2c_tx(uint8_t byte);
uint8_t i2c_rx(uint8_t last_byte);
uint8_t in_sda(void);
void sda_io(uint8_t io_c);
void scl_set(uint8_t set_c);
void sda_set(uint8_t set_c);
unsigned char mcu_i2c_write_byte(uint8_t device_address, uint8_t lpdata1);
void mcu_i2c_write_two_byte (uint8_t device_address, uint8_t lpdata1, uint8_t lpdata2);


void i2c_init(void)
	{
	
	RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTE, ENABLE);

	PORT_InitStructure.PORT_OE = PORT_OE_IN;	
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	PORT_InitStructure.PORT_PULL_UP = PORT_PULL_UP_OFF;
	PORT_InitStructure.PORT_PULL_DOWN = PORT_PULL_DOWN_OFF;
		
	PORT_InitStructure.PORT_Pin = SDA_LN | SCL_LN;	
	PORT_Init(I2C_SDA_PORT, &PORT_InitStructure);
	i2c_error=0; 
	
	}

unsigned char in_sda(void)
	{
		
  	if((PORT_ReadInputData(I2C_SDA_PORT) & SDA_LN)) return 1;
  		else return 0;
	}

	void sda_io(uint8_t io_c)
	{
	uint32_t i = pause;

	if (io_c==IN_LN) 
		{
		PORT_InitStructure.PORT_OE = PORT_OE_IN;	
		PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
		PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
		PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	
		PORT_InitStructure.PORT_Pin = SDA_LN;
		PORT_Init(I2C_SDA_PORT, &PORT_InitStructure);	
		}
		else
		{
					PORT_InitStructure.PORT_OE = PORT_OE_OUT;	
		PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
		PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
		PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	
		PORT_InitStructure.PORT_Pin = SDA_LN;
		PORT_Init(I2C_SDA_PORT, &PORT_InitStructure);	
			PORT_WriteBit(I2C_SDA_PORT, SDA_LN, 0);
		}
	while (--i > 0) __NOP();
	}

void scl_set(uint8_t set_c)
	{
	uint32_t i = pause;

	if(set_c)
		{
			PORT_InitStructure.PORT_OE = PORT_OE_IN;	
		PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
		PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
		PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	
		PORT_InitStructure.PORT_Pin = SCL_LN;
		PORT_Init(I2C_SCL_PORT, &PORT_InitStructure);	
			
		}
    	else 
			{
					PORT_InitStructure.PORT_OE = PORT_OE_OUT;	
					PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
					PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
					PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	
					PORT_InitStructure.PORT_Pin = SCL_LN;
					PORT_Init(I2C_SCL_PORT, &PORT_InitStructure);	
					PORT_WriteBit(I2C_SDA_PORT, SCL_LN, 0);
			}
	while (--i > 0) __NOP();
	}

void sda_set(uint8_t set_c)
	{
	uint32_t i = pause/2;

	if(set_c)
		{
		PORT_InitStructure.PORT_OE = PORT_OE_IN;	
		PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
		PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
		PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	
		PORT_InitStructure.PORT_Pin = SDA_LN;
		PORT_Init(I2C_SDA_PORT, &PORT_InitStructure);	
		}
    	else 
			{
			PORT_InitStructure.PORT_OE = PORT_OE_OUT;	
			PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
			PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
			PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	
			PORT_InitStructure.PORT_Pin = SDA_LN;
			PORT_Init(I2C_SDA_PORT, &PORT_InitStructure);	
			PORT_WriteBit(I2C_SDA_PORT, SDA_LN, 0);
				
			}

	while (--i > 0) __NOP();
	}

void i2c_start(void)
	{
	scl_set(1);  	
  	sda_set(0);
	scl_set(0);  	
	}
void i2c_start_read(void)
	{
		scl_set(1);  	
  	sda_set(0);
		scl_set(0); 
		sda_set(1);		
		scl_set(1);
		
	}

void i2c_stop(void)
	{
  	sda_set(0);
  	scl_set(1);
  	sda_set(1);	
 	}
void scl_clock(void)
{
	scl_set(1);
	scl_set(0);
}

void i2c_wr_bit(uint8_t bit)
{	
	if (bit) sda_set(1); else sda_set(0);
	scl_set(1);
	scl_set(0);
	
}
uint8_t i2c_rd_bit(void)
{	uint8_t bit;
	
	scl_set(1);
	bit = in_sda();
	scl_set(0);
	return bit;
}
uint8_t i2c_wr_U8(uint8_t byte)
{	uint8_t i;
	for (i = 8; i;i--) {
	i2c_wr_bit(byte & 0x80);
	byte <<= 1;
	}

	sda_set(1);
	return (i2c_rd_bit());
}
uint8_t i2c_rd_U8(uint8_t ack)
{	uint8_t i;
	uint8_t result = 0;
	for (i = 8; i; i--) {
	result <<= 1;
	result |= !!(i2c_rd_bit());
	}
	i2c_wr_bit(ack);
	return (result);
}
unsigned char mcu_i2c_write(uint8_t device_address)
{
	uint8_t success;
	i2c_start();
	success = i2c_wr_U8(device_address);	
	i2c_stop();
	return success;
  //itmr_Delay(5);	//delay between write and read...not less 4ms
	
}
unsigned char mcu_i2c_write_byte(uint8_t device_address, uint8_t lpdata1)
{ 
 uint8_t success = 1, data_read = 0;
	i2c_start();
	success = i2c_wr_U8(device_address);
	if (success == 0)
	{
		success = i2c_wr_U8(lpdata1);
	}
	else return 1;
	i2c_start();
	success = i2c_wr_U8(device_address | 0x01);
	if (success  == 0) {
	
		data_read = i2c_rd_U8(0);
		return data_read;
	}
	else return 2;
	i2c_stop();
	
}
unsigned short mcu_i2c_read_Int(uint8_t device_address, uint8_t lpdata1)
{ 
 uint8_t msb, lsb;

		i2c_start();
		i2c_wr_U8(device_address);
		i2c_wr_U8(lpdata1);
		i2c_start();
		i2c_wr_U8(device_address | 0x01);
		msb = i2c_rd_U8(1);
		i2c_stop();
		i2c_start();
		i2c_wr_U8(device_address);
		i2c_wr_U8(lpdata1 + 0x01);
		i2c_start();
		i2c_wr_U8(device_address | 0x01);
		lsb = i2c_rd_U8(1);
		i2c_stop();
		return ((msb << 8) | lsb);
}
void mcu_i2c_write_two_byte (uint8_t device_address, uint8_t lpdata1, uint8_t lpdata2)
{ 
	i2c_start();
	i2c_wr_U8(device_address);
	i2c_wr_U8(lpdata1);	
	i2c_wr_U8(lpdata2);
	i2c_stop();

	itmr_Delay(5);	//delay between write and read...not less 4ms
}
uint8_t mcu_i2c_read_byte(void)
{unsigned char data;
 
	i2c_start();	
	data = i2c_rd_U8(0);		
	
	i2c_stop();
	
	return data;
}
void mcu_i2c_read_len(uint8_t device_address, uint8_t reg, uint8_t dataRead[3], uint8_t len)
{	uint8_t i;
	i2c_start();
	i2c_wr_U8(device_address);
	i2c_wr_U8(reg);
	i2c_start();
	i2c_wr_U8(device_address | 0x01);
	for (i=0;i<2;i++)
	{
		dataRead[i] = i2c_rd_U8(0);
	}
	dataRead[i] = i2c_rd_U8(1);
	i2c_stop();
}
