
#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"
#include "MDR32F9Qx_ssp.h"
#include "spi.h"
#include "sflash.h"
#include "disp.h"

void spi_Ports_Init(void)
{
	
	PORT_InitTypeDef PORT_InitStructure;
	
	RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTC, ENABLE);	

	
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_ALTER;
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;
	
	PORT_InitStructure.PORT_PULL_UP = PORT_PULL_UP_OFF;
	PORT_InitStructure.PORT_PULL_DOWN = PORT_PULL_DOWN_OFF;
	
	PORT_InitStructure.PORT_Pin = (PORT_Pin_5 | PORT_Pin_7);	
	PORT_Init(MDR_PORTC, &PORT_InitStructure);	
	
	
	PORT_InitStructure.PORT_OE = PORT_OE_IN;
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_ALTER;
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;
	
	PORT_InitStructure.PORT_Pin = (PORT_Pin_6);	
	PORT_Init(MDR_PORTC, &PORT_InitStructure);
	
}
void spi_DeInit(void)
{
	SSP_InitTypeDef sSSP;
	RST_CLK_PCLKcmd(RST_CLK_PCLK_SSP1, ENABLE);	
	SSP_DeInit(MDR_SSP1);
	SSP_BRGInit(MDR_SSP1, SSP_HCLKdiv2); // 
	
	SSP_StructInit(&sSSP);

	sSSP.SSP_SCR = 0;
	sSSP.SSP_CPSDVSR = 2;
	sSSP.SSP_Mode = SSP_ModeMaster;
	sSSP.SSP_WordLength = SSP_WordLength8b;
	sSSP.SSP_SPH = SSP_SPH_2Edge;
	sSSP.SSP_SPO = SSP_SPO_High;
	sSSP.SSP_FRF = SSP_FRF_SPI_Motorola;
	sSSP.SSP_HardwareFlowControl = SSP_HardwareFlowControl_SSE;
	SSP_Init(MDR_SSP1, &sSSP);	
	SSP_Cmd(MDR_SSP1, ENABLE);	
	
}
void spi_Interface_Init(void)
{
	SSP_InitTypeDef sSSP;
	
	SSP_DeInit(MDR_SSP1);
	SSP_BRGInit(MDR_SSP1,SSP_HCLKdiv1);		
	
	SSP_StructInit(&sSSP);

  sSSP.SSP_SCR  = 0x20;  //20
  sSSP.SSP_CPSDVSR = 2;
  sSSP.SSP_Mode = SSP_ModeMaster;
  sSSP.SSP_WordLength = SSP_WordLength8b;
  sSSP.SSP_SPH = SSP_SPH_1Edge;//SSP_SPH_1Edge;
  sSSP.SSP_SPO = SSP_SPO_Low;//SSP_SPO_Low;
  sSSP.SSP_FRF = SSP_FRF_SPI_Motorola;
  sSSP.SSP_HardwareFlowControl = SSP_HardwareFlowControl_SSE;
  SSP_Init (MDR_SSP1, &sSSP);
	SSP_DMACmd(MDR_SSP1, (SSP_DMA_TXE | SSP_DMA_RXE), DISABLE);
	
	SSP_Cmd(MDR_SSP1, ENABLE);	

}

unsigned char SSP_SendByte(unsigned char byte)
{while (SSP_GetFlagStatus(MDR_SSP1, SSP_FLAG_TFE ) == RESET);
	SSP_SendData(MDR_SSP1, byte);
return 0;
}
unsigned char SSP_ReceiveByte(unsigned char byte)
{unsigned short rb;
	while (SSP_GetFlagStatus(MDR_SSP1, SSP_FLAG_RNE ) == RESET);
	rb = SSP_ReceiveData(MDR_SSP1);	
	return rb;
}
unsigned char SendAndRcvByte(unsigned char byte)
{
	unsigned short rb;
	while (SSP_GetFlagStatus(MDR_SSP1, SSP_FLAG_TFE ) == RESET);
	SSP_SendData(MDR_SSP1, byte);
	while (SSP_GetFlagStatus(MDR_SSP1, SSP_FLAG_RNE ) == RESET);
	rb = SSP_ReceiveData(MDR_SSP1);	
	return rb;
}

 void spi_Init(void)
{
	spi_Ports_Init();
	spi_Interface_Init();	
	
}
void spi_SendBuffer(unsigned char* buf, unsigned int len) {

  while (len--) {
    SSP_SendByte(*buf++);
  }
}
void spi_ReadBuffer(unsigned char* buf, unsigned int len) {

  while (len--) {
    *buf++ = SendAndRcvByte(0xff);
  }
}
unsigned char sFLASH_SendByte(unsigned char byte)
 {
   /*!< Loop while DR register in not emplty */
   while (SSP_GetFlagStatus(MDR_SSP1, SSP_FLAG_TFE ) == RESET);
 
   /*!< Send byte through the SPI1 peripheral */
   SSP_SendData(MDR_SSP1, byte);
 
   /*!< Wait to receive a byte */
   while (SSP_GetFlagStatus(MDR_SSP1, SSP_FLAG_RNE ) == RESET);
 
   /*!< Return the byte read from the SPI bus */
   return SSP_ReceiveData(MDR_SSP1);
 }	 
 
 
void SPIx_SetSpeed(unsigned char divider)
{
	SSP_InitTypeDef sSSP;
	
	SSP_DeInit(MDR_SSP1);
	if (divider == 0) {
		SSP_BRGInit(MDR_SSP1,SSP_HCLKdiv128);		
	}
	else
	{
		SSP_BRGInit(MDR_SSP1,SSP_HCLKdiv1);		
	}
	SSP_StructInit(&sSSP);

  sSSP.SSP_SCR  = 0x20;  //20
  sSSP.SSP_CPSDVSR = 2;
  sSSP.SSP_Mode = SSP_ModeMaster;
  sSSP.SSP_WordLength = SSP_WordLength8b;
  sSSP.SSP_SPH = SSP_SPH_1Edge;//SSP_SPH_1Edge;
  sSSP.SSP_SPO = SSP_SPO_Low;//SSP_SPO_Low;
  sSSP.SSP_FRF = SSP_FRF_SPI_Motorola;
  sSSP.SSP_HardwareFlowControl = SSP_HardwareFlowControl_SSE;
  SSP_Init (MDR_SSP1, &sSSP);
	SSP_DMACmd(MDR_SSP1, (SSP_DMA_TXE | SSP_DMA_RXE), DISABLE);
	
	SSP_Cmd(MDR_SSP1, ENABLE);	

}
