#include <string.h>
#include "opora.h"
#include "eth_drv.h"
#include "eth_def.h"
#include "eth_prop.h"
#include "eth_misc.h"
#include "error.h"
#include "itmr.h"

static TEthSettings EthSetting;
static TEthAddr EthernetMAC;
static TIP4Addr EthernetIP;
static TIP4Addr SubNetMask;
static TIP4Addr Gateway;

//---------------------------------------------------------------------------------------------
//    
#define EnterCSEth()								\
						ETHERNET->IMR = 0;			\
						ETHERNET->IFR = 0xFFFF		

//   
//   
#define LeaveCSEth()	ETHERNET->IMR |= bETH_IMR_RF_OK /*| bETH_IMR_XF_OK | bETH_IMR_XF_ERR | bETH_IMR_OVF	| bETH_IMR_CRC_ERR*/

//---------------------------------------------------------------------------------------------
//    PHY  MAC 
//---------------------------------------------------------------------------------------------
void ethernet_LowLevelInit(void)
{
	unsigned int stime = itmr_GetTick();
	//    
	//  
	RST_CLK->HS_CONTROL |= (1 << 2);													// HSE2 - On, Oscillator mode
	while((RST_CLK->CLOCK_STATUS & (1 << 3)) != (1 << 3)){		// Wait until HSE2 not ready
		if(itmr_IsTime(stime, 10000ul)){
			error_SetError(HSE2_ERROR);
			break;
		}
	}
	RST_CLK->ETH_CLOCK |= (1 << 24) | (1 << 27) | (3 << 28);	// PHY_CLK_SEL = HSE2
	//     -     25
	PORTF->ANALOG &= 0xFFFC;		
	
	ethprop_GetMAC(&EthernetMAC);
	EthSetting.mac[0] = EthernetMAC.addr[0];
	EthSetting.mac[1] = EthernetMAC.addr[1];
	EthSetting.mac[2] = EthernetMAC.addr[2];
	EthSetting.mac[3] = EthernetMAC.addr[3];
	EthSetting.mac[4] = EthernetMAC.addr[4];
	EthSetting.mac[5] = EthernetMAC.addr[5];
	
	ethprop_GetIP(&EthernetIP);
	EthSetting.ip[0] = EthernetIP.d8[0];
	EthSetting.ip[1] = EthernetIP.d8[1];
	EthSetting.ip[2] = EthernetIP.d8[2];
	EthSetting.ip[3] = EthernetIP.d8[3];	
	
	ethprop_GetMask(&SubNetMask);
	EthSetting.mask[0] = SubNetMask.d8[0];
	EthSetting.mask[1] = SubNetMask.d8[1];
	EthSetting.mask[2] = SubNetMask.d8[2];
	EthSetting.mask[3] = SubNetMask.d8[3];	
	
	ethprop_GetGate(&Gateway);
	EthSetting.gateway[0] = Gateway.d8[0];	
	EthSetting.gateway[1] = Gateway.d8[1];	
	EthSetting.gateway[2] = Gateway.d8[2];	
	EthSetting.gateway[3] = Gateway.d8[3];		
}

//---------------------------------------------------------------------------------------------
//   PHY      
void ethernet_PHYInit(unsigned char addr, unsigned char mode)
{
	unsigned int ctrl;				

	ctrl = ETHERNET->PHY_CTRL;	//    
	ctrl &= mPHY_CTRL_MODE|mPHY_CTRL_PHYADD;	//     
	ctrl |= (addr << PHY_CTRL_PHYADD) | (mode << PHY_CTRL_MODE) | bPHY_BN_nRST;
	
	ETHERNET->PHY_CTRL = ctrl; //  

	while(!(ETHERNET->PHY_STATUS & bPHY_STAT_READY)); //  
}

//---------------------------------------------------------------------------------------------
//    
void ethernet_PHYState(void)
{		
	unsigned short status = ETHERNET->PHY_STATUS;
	//   
	//uart_Send((char*)&status, sizeof(status));
	if(!(ETHERNET->PHY_STATUS & bPHY_STAT_LED1))	PORTE->SETTX = 1 << 6;			
	else											PORTE->CLRTX = 1 << 6;

	//   
	if(!(ETHERNET->PHY_STATUS & bPHY_STAT_LED3))	PORTE->SETTX = 1 << 7;
	else											PORTE->CLRTX = 1 << 7;			
}

//---------------------------------------------------------------------------------------------
void ethernet_MACConfig(void)
{
//	unsigned char cnt;

	ethernet_LowLevelInit();
	
	ethernet_PHYInit(PHY_ADDR, EthSetting.speed);
	
	ETHERNET->MAC_T = MAKEWORD(EthSetting.mac[1], EthSetting.mac[0]);	
	ETHERNET->MAC_M = MAKEWORD(EthSetting.mac[3], EthSetting.mac[2]);
	ETHERNET->MAC_H = MAKEWORD(EthSetting.mac[5], EthSetting.mac[4]);

	ethernet_MACReset();
	//   
	// -   
	LeaveCSEth();	
/*
	itmr_Delay(3000);
	//  arp    ip  
	//   ""  		
	for(cnt = 0; cnt < ARP_START_CNT; cnt++)
		ARP_START();					
*/	
}

//---------------------------------------------------------------------------------------------
void ethernet_MACReset(void)
{
	//    
	ETHERNET->G_CFG |= bETH_CTRL_XRST | bETH_CTRL_RRST;			

	memset((unsigned int*)ETH_RAM_BASE_ADDR, 0, ETH_RAM_FULL_SIZE * sizeof(unsigned char));

	//      
	ETHERNET->Delimiter = ETH_RAM_SIZE_R;

	// HASH -   
	ETHERNET->HASH0 = 0;
	ETHERNET->HASH1 = 0;
	ETHERNET->HASH2 = 0;
	ETHERNET->HASH3 = 0x8000;	
	//-------------------------------------------------------
	//     
	ETHERNET->IPG = 0x0060;
	//  BAG  JitterWnd
	ETHERNET->PSC = PSC_VAL(1);	  //0x50
	//   
	ETHERNET->BAG = BAG_PERIOD(40); //0x200	//  100
	//    
	ETHERNET->JitterWnd = BAG_PERIOD(2); //0x5	//  5
	//-------------------------------------------------------
	//   
	ETHERNET->R_CFG = bETH_R_CFG_EN |					//   
	(EVNT_MODE_RFIFO_NO_FULL << ETH_R_CFG_EVNT_MODE)|	//  -   	
	bETH_R_CFG_UCA_EN |									//    MAC    MAC_Address
	bETH_R_CFG_BCA_EN |									//   MAC 
	bETH_R_CFG_AC_EN; //0x8406							//   

	//  
	ETHERNET->X_CFG = bETH_X_CFG_EN |						//    
	(EVNT_MODE_XFIFO_NEAR_EMPTY << ETH_X_CFG_EVNT_MODE)| 	//  -   
	bETH_X_CFG_PAD_EN |										//   PAD'
	bETH_X_CFG_PRE_EN |										//  
	bETH_X_CFG_CRC_EN | 									//  CRC
	bETH_X_CFG_IPG_EN |										//   
	EXCH_ATTEMPT;	//0x81FA								//   
	
	ETHERNET->G_CFG = bETH_CTRL_RRST |		//  
	bETH_CTRL_XRST |						//  
	COLL_WND;								//   

	//     
	EnterCSEth();	
	
	ETHERNET->R_Head = 0;				//       
	ETHERNET->X_Tail = ETH_RAM_SIZE_R; 	//     

	ETHERNET->G_CFG &= ~(bETH_CTRL_XRST | bETH_CTRL_RRST);	//RRST=0, XRST=0   
	NVIC_EnableIRQ(ETHERNET_IRQn);		
}

//---------------------------------------------------------------------------------------------
//     
unsigned int ethernet_ReadRAM(void* buf)
{		
	unsigned int head, tail;	//  ,      ,  	
	unsigned int* src;			//  
	unsigned int* dst; 	 		//  
	unsigned short tmp[2];		//     
	unsigned short space[2];	//     
	unsigned int i, size;	
		
	//   ,      return 0	
	if(!(ETHERNET->STAT & mETH_STAT_R_CNT))
		return 0;

	head = ETHERNET->R_Head;	// 
	tail = ETHERNET->R_Tail;	// 	

	if(head < tail){
		space[0] = tail - head;
		space[1] = 0;
	}
	else{
		space[0] = ETH_RAM_SIZE_R - head;
		space[1] = tail;
	}

	src = (unsigned int*)(ETH_RAM_BASE_ADDR + head);
	dst = (unsigned int*)buf;
	
	*((unsigned int*)tmp) = *src++;	//      ,   
	space[0] -= sizeof(unsigned int);	//     ,         
	
	if(((unsigned int)src & 0xFFFF) > (ETH_RAM_SIZE_R - 1))
		src = (unsigned int*)ETH_RAM_BASE_ADDR;
		
	size = (tmp[0] + sizeof(unsigned int) - 1)/sizeof(unsigned int); //    

	if(tmp[0] <= space[0]){ //     
		for(i = 0; i < size; i++)	
			*dst++ = *src++;						
	}					
	else{	//     
		size -= (space[0] >> 2);
		for(i = 0; i < (space[0] >> 2); i++)
			*dst++ = *src++;				
		src = (unsigned int*)ETH_RAM_BASE_ADDR;
		for(i = 0; i < size; i++)
			*dst++ = *src++;					
	}
	if(((unsigned int)src & 0xFFFF) > (ETH_RAM_SIZE_R - 1))
		src = (unsigned int*)ETH_RAM_BASE_ADDR;	
	
	ETHERNET->R_Head = (unsigned int)src & 0xFFFF;
	ETHERNET->STAT -= (1 << ETH_STAT_R_CNT);	//   	
	PORTA->RXTX ^= (1 << 9);	
	return tmp[0];		
}

//---------------------------------------------------------------------------------------------
//    
unsigned int ethernet_WriteRAM(void* buf, unsigned int size)
{	
	unsigned int head, tail;	//  ,      ,  
	unsigned short tmp[2];
	unsigned int tmp32;
	unsigned int* src; 
	unsigned int* dst;
	unsigned short i;	
	
	head = ETHERNET->X_Head;	// 
	tail = ETHERNET->X_Tail;	//	
		
	//      
	if(head > tail){			
		tmp[0] = head - tail;
		tmp[1] = 0;	
	}
	else{		
		tmp[0] = ETH_RAM_FULL_SIZE - tail;
		tmp[1] = head - ETH_RAM_SIZE_X;
	}					

	//     ,    (  2-   )
	if(size > (tmp[0] + tmp[1] - 2 * sizeof(unsigned int))){				
		return 0;			
	}

	tmp32 = size;

	src = buf;
	dst = (unsigned int*)(ETH_RAM_BASE_ADDR + tail); // 		

	*dst++ = tmp32;
	tmp[0] -= sizeof(unsigned int);

	if(dst >= (unsigned int*)(ETH_RAM_BASE_ADDR + ETH_RAM_FULL_SIZE))
		dst = (unsigned int*)ETH_RAM_BASE_X;
	

	tmp32 = (size + sizeof(unsigned int) - 1)/sizeof(unsigned int);	

	if(size <= tmp[0]){		
		for(i = 0; i < tmp32; i++){
			*dst++ = *src++;			
		}						
	}	
	else{				
		tmp32 -= tmp[0]/sizeof(unsigned int);
			
		for(i = 0; i < tmp[0]/sizeof(unsigned int); i++)
			*dst++ = *src++;
		dst = (unsigned int*)ETH_RAM_BASE_X;
		for(i = 0; i < tmp32; i++)
			*dst++ = *src++;		
	}	
	
	if(dst >= (unsigned int*)(ETH_RAM_BASE_ADDR + ETH_RAM_FULL_SIZE)){		
		dst = (unsigned int*)ETH_RAM_BASE_X;
	}

	tmp32 = 0;	
	*dst++ = tmp32;	

	if(dst >= (unsigned int*)(ETH_RAM_BASE_ADDR + ETH_RAM_FULL_SIZE)){	
		dst = (unsigned int*)ETH_RAM_BASE_X;
	}
		
	ETHERNET->X_Tail = (unsigned int)dst;								
	PORTA->RXTX ^= (1 << 13);
	return size;
}

//---------------------------------------------------------------------------------------------
//    -   
void ETHERNET_Handler(void)
{		
	//  
	unsigned int intr = ETHERNET->IFR;		
	ETHERNET->IFR = intr;
}

char HasMessage(void){
	return (ETHERNET->R_Head != ETHERNET->R_Tail) ? 1 : 0;
}

//---------------------------------------------------------------------------------------------
void ethernet_GetSettings(TEthSettings* settings)
{
	memcpy(settings, &EthSetting, sizeof(TEthSettings));
}

void ethernet_SetSettings(TEthSettings* settings)
{
	memcpy(&EthSetting, settings, sizeof(TEthSettings));
			
	EthernetMAC.addr[0] = EthSetting.mac[0];
	EthernetMAC.addr[1] = EthSetting.mac[1];
	EthernetMAC.addr[2] = EthSetting.mac[2];
	EthernetMAC.addr[3] = EthSetting.mac[3];
	EthernetMAC.addr[4] = EthSetting.mac[4];
	EthernetMAC.addr[5] = EthSetting.mac[5];
	ethprop_SetMAC(&EthernetMAC);
	
	EthernetIP.d8[0] = EthSetting.ip[0];
	EthernetIP.d8[1] = EthSetting.ip[1];
	EthernetIP.d8[2] = EthSetting.ip[2];
	EthernetIP.d8[3] = EthSetting.ip[3];
	ethprop_SetIP(&EthernetIP);	
	
	SubNetMask.d8[0] = EthSetting.mask[0];
	SubNetMask.d8[1] = EthSetting.mask[1];
	SubNetMask.d8[2] = EthSetting.mask[2];
	SubNetMask.d8[3] = EthSetting.mask[3];
	ethprop_SetMask(&SubNetMask);	
	
	Gateway.d8[0] = EthSetting.gateway[0];
	Gateway.d8[1] = EthSetting.gateway[1];
	Gateway.d8[2] = EthSetting.gateway[2];
	Gateway.d8[3] = EthSetting.gateway[3];	
	ethprop_SetGate(&Gateway);	
	
	ethernet_MACConfig();
}
