#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"
#include "MDR32F9Qx_ebc.h"
#include "nand_flash.h"

#include "itmr.h"

#define READ_1st	0x01
#define READ_2nd	0x30

#define READ_FOR_COPY_BACK_1st	0x00
#define READ_FOR_COPY_BACK_2nd	0x35

#define READ_ID	0x90

#define RESET	0xFF

#define PAGE_PROGRAM_START	0x80
#define PAGE_PROGRAM_STOP	0x10

#define COPY_BACK_PROGRAM_START	0x85
#define COPY_BACK_PROGRAM_STOP	0x10

#define BLOCK_ERASE_START	0x60
#define BLOCK_ERASE_STOP	0xD0

#define RANDOM_DATA_IN	0x85

#define RANDOM_DATA_OUT_START	0x05
#define RANDOM_DATA_OUT_STOP	0xE0

#define READ_STATUS	0x70

#define NF_PageSize 1024
#define NF_Status_Fail 0x00
#define buffer_size NF_PageSize + 64
#define MNUM	0x77

unsigned int buffer[buffer_size];

static unsigned int NF_AddressCycles[4];
static EBC_InitTypeDef EBC_InitStruct;


static unsigned int nand_cmd_phase(unsigned char cmd_start, 
																		unsigned char cmd_stop, 
																		unsigned char cycle_cnt, 
																		unsigned char with_stop)
{
	return (MNUM << 24) | ((cycle_cnt & 0x7) << 21) | ((with_stop & 1) << 20) | (cmd_stop << 11) | (cmd_start << 3);
}

static unsigned int nand_data_phase(unsigned char cmd_stop, 																		
																		unsigned char with_stop,
																		unsigned char cs)																			
{
	return (MNUM << 24) | ((cs & 1) << 21) | ((with_stop & 1) << 20) | (1 << 19) | (cmd_stop << 11);
}																			

static unsigned int nand_read_id(void)
{
	return nand_cmd_phase(READ_ID, 0, 1, 0);
}

static unsigned int nand_read_status(void)
{
	return nand_cmd_phase(READ_STATUS, 0, 0, 0);
}

static unsigned int nand_block_erase(void)
{
	return nand_cmd_phase(BLOCK_ERASE_START, BLOCK_ERASE_STOP, 2, 1);
}

static unsigned int nand_read_page(void)
{
	return nand_cmd_phase(READ_1st, 0, 4, 0);
}

static unsigned int nand_write_page_first(void)
{
	return nand_cmd_phase(PAGE_PROGRAM_START, 0, 4, 0);
}

static unsigned int nand_write_page_second(void)
{
	return nand_cmd_phase(PAGE_PROGRAM_STOP, 0, 0, 0);
}

//Generate Address
void generate_address(unsigned int column_address, unsigned int row_address, unsigned int buffer_bytes[4])
{ buffer_bytes[0] = 0;
	buffer_bytes[1] = 0;
	buffer_bytes[2] = 0;
	buffer_bytes[3] = 0;
	
	//1 cycle
	buffer_bytes[0] = (unsigned int)(column_address    & 0xFF)      ;
	//2 cycle
	buffer_bytes[1] = (unsigned int)((column_address   & 0x0F) >> 8);
	//3 cycle
	buffer_bytes[2] = (unsigned int)(row_address       & 0xFF)      ;
	//4 cycle
	buffer_bytes[3] = (unsigned int)((row_address      & 0xFF) >> 8);		
} 
//------------------------------------------------------------
void nand_flash_Init(void)
{
	PORT_InitTypeDef PORT_InitStructure;
	
	RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTA | RST_CLK_PCLK_PORTC | RST_CLK_PCLK_PORTD | RST_CLK_PCLK_PORTE | RST_CLK_PCLK_PORTF, ENABLE);			
	// portA - data/cmd bus	
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_MAIN;
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;
	
	PORT_InitStructure.PORT_Pin = PORT_Pin_0 | PORT_Pin_1 | PORT_Pin_2 | PORT_Pin_3 |  
																PORT_Pin_4 | PORT_Pin_5 | PORT_Pin_6 | PORT_Pin_7;	
	PORT_Init(MDR_PORTA, &PORT_InitStructure);	
	
	// 0 - NWR, 1 - NRD, 2 - CLKO
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_MAIN;
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;
	
	PORT_InitStructure.PORT_Pin = PORT_Pin_0 | PORT_Pin_1 | PORT_Pin_2;	
	PORT_Init(MDR_PORTC, &PORT_InitStructure);	
	
	// BUSY
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_ALTER;
	PORT_InitStructure.PORT_Pin = PORT_Pin_15;	
	PORT_Init(MDR_PORTC, &PORT_InitStructure);	
	//PORT_WriteBit(MDR_PORTC, PORT_Pin_1, Bit_SET);	//Read Enable
	// cmd lanch
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_ALTER;
	PORT_InitStructure.PORT_Pin = PORT_Pin_1;	
	PORT_Init(MDR_PORTD, &PORT_InitStructure);	
	
	// CS
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	PORT_InitStructure.PORT_Pin = PORT_Pin_0;	
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;	
	PORT_Init(MDR_PORTE, &PORT_InitStructure);	
	PORT_WriteBit(MDR_PORTE, PORT_Pin_0, Bit_RESET);
	
	// write protect
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	PORT_InitStructure.PORT_Pin = PORT_Pin_2;	
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;	
	PORT_Init(MDR_PORTF, &PORT_InitStructure);	
	PORT_WriteBit(MDR_PORTF, PORT_Pin_2, Bit_SET);
	
	// ext bus config
	RST_CLK_PCLKcmd(RST_CLK_PCLK_EBC, ENABLE);
	EBC_StructInit(&EBC_InitStruct);
	EBC_InitStruct.EBC_Mode					 = EBC_MODE_NAND;
  EBC_InitStruct.EBC_NandTrc       = EBC_NAND_CYCLES_6HCLK;
  EBC_InitStruct.EBC_NandTwc       = EBC_NAND_CYCLES_6HCLK;
  EBC_InitStruct.EBC_NandTrea      = EBC_NAND_CYCLES_4HCLK;
  EBC_InitStruct.EBC_NandTwp       = EBC_NAND_CYCLES_3HCLK;
  EBC_InitStruct.EBC_NandTwhr      = EBC_NAND_CYCLES_6HCLK;
  EBC_InitStruct.EBC_NandTalea		 = EBC_NAND_CYCLES_10HCLK;
  EBC_InitStruct.EBC_NandTrr       = EBC_NAND_CYCLES_2HCLK;	
	//EBC_InitStruct.EBC_UseMemRegion  = ENABLE;
	EBC_Init(&EBC_InitStruct);	
/*
	EBC_MemRegionStructInit(&EBC_MemRegionInitStruct);
	EBC_MemRegionInit(&EBC_MemRegionInitStruct, EBC_MEM_REGION_70000000);	
*/	
}

unsigned int nand_flash_ReadID(void)	
//void nand_flash_ReadID(char write_id)	
{ 
	unsigned char id[5];
	unsigned int cmd_addr, data_addr;

	cmd_addr = nand_read_id();
	data_addr = nand_data_phase(0, 0, 0);

	*(volatile unsigned char*)cmd_addr = 0x00;
	id[0] = *(volatile unsigned char*)data_addr;
	id[1] = *(volatile unsigned char*)data_addr;
	id[2] = *(volatile unsigned char*)data_addr;
	id[3] = *(volatile unsigned char*)data_addr;
	id[4] = *(volatile unsigned char*)data_addr;	
	 
	return (id[1] << 24) | (id[2] << 16) | (id[3] << 8) | id[4];
	
}

unsigned int nand_flash_ReadStatus(void)
{
	unsigned int cmd_addr, data_addr;
	unsigned char status;
	
	cmd_addr = nand_read_status();
	data_addr = nand_data_phase(0, 0, 0);
	
	*(volatile unsigned char*)cmd_addr = 0;
	status = *(volatile unsigned char*)data_addr;
	
	return status;
}

void nand_flash_WritePage(unsigned short col, unsigned short row, unsigned char* buf, unsigned int buf_size)
{
	unsigned int cmd_addr, data_addr, i, status;
	unsigned int buffer_bytes[4];
	generate_address(col, row, buffer_bytes);
	
	cmd_addr = nand_write_page_first();
	data_addr = nand_data_phase(PAGE_PROGRAM_STOP, 0, 0);
	
	
	*(volatile unsigned char*)cmd_addr = buffer_bytes[0];
	*(volatile unsigned char*)cmd_addr = buffer_bytes[1];
	*(volatile unsigned char*)cmd_addr = buffer_bytes[2];
	*(volatile unsigned char*)cmd_addr = buffer_bytes[3];
	
		for(i = 0; i < buf_size - 1; i++)
	{
			*(volatile unsigned char*)data_addr = buf[i];
	}
	data_addr = nand_data_phase(PAGE_PROGRAM_STOP, 1, 0);	
	*(volatile unsigned char*)data_addr = buf[buf_size - 1];
	
  while (EBC_GetBusyStatus() != SET) {;}
	status = nand_flash_ReadStatus();		
}

void nand_flash_ReadPage(unsigned short col, unsigned short row, unsigned char* buf, unsigned int buf_size)
{
	unsigned int cmd_addr, data_addr, i, status;
	unsigned int buffer_bytes[4];
	generate_address(col, row, buffer_bytes);
	cmd_addr = nand_read_page();
	data_addr = nand_data_phase(0, 0, 0);
	*(volatile unsigned char*)cmd_addr = buffer_bytes[0];
	*(volatile unsigned char*)cmd_addr = buffer_bytes[1];
	*(volatile unsigned char*)cmd_addr = buffer_bytes[2];
	*(volatile unsigned char*)cmd_addr = buffer_bytes[3];
	while (EBC_GetBusyStatus() != SET) {;}
		for(i = 0; i < buf_size - 1; i++)
		{
			buf[i]=*(volatile unsigned char*)data_addr;	
		}
		data_addr = nand_data_phase(READ_2nd, 1, 0);
		buf[buf_size - 1] = *(volatile unsigned char*)data_addr;
}

void nand_flash_BlockErase(unsigned int block)
{
	unsigned int cmd_addr, data_addr, i, status;	
	unsigned int buffer_bytes[4];
	cmd_addr = nand_block_erase();
	data_addr = nand_data_phase(BLOCK_ERASE_STOP,1,0);
	generate_address(block, 0, buffer_bytes);
	*(volatile unsigned char*)cmd_addr = buffer_bytes[0];
	*(volatile unsigned char*)cmd_addr = buffer_bytes[1];
	while (EBC_GetBusyStatus() != SET) {;}
	*(volatile unsigned char*)data_addr = 0;
	
}	