#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"
#include "MDR32F9Qx_ebc.h"
#include "nand_flash.h"

#include "itmr.h"

#define READ_1st	0x00
#define READ_2nd	0x30

#define READ_FOR_COPY_BACK_1st	0x00
#define READ_FOR_COPY_BACK_2nd	0x35

#define READ_ID	0x90

#define RESET	0xFF

#define PAGE_PROGRAM_1st	0x80
#define PAGE_PROGRAM_2nd	0x10

#define COPY_BACK_PROGRAM_1st	0x85
#define COPY_BACK_PROGRAM_2nd	0x10

#define BLOCK_ERASE_1st	0x60
#define BLOCK_ERASE_2nd	0xD0

#define RANDOM_DATA_IN	0x85

#define RANDOM_DATA_OUT_1st	0x05
#define RANDOM_DATA_OUT_2nd	0xE0

#define READ_STATUS	0x70

#define NF_PageSize 8
#define NF_Status_Fail 0x0
#define buffer_size NF_PageSize + 64

unsigned int buffer[buffer_size];

static unsigned int NF_AddressCycles[5];
static EBC_InitTypeDef EBC_InitStruct;
//static EBC_MemRegionInitTypeDef EBC_MemRegionInitStruct;

static unsigned int nand_cmd_phase_without_stop(unsigned char start_cmd, unsigned int cycle_cnt)
{
	return (0x77 << 24) | ((cycle_cnt & 0x7) << 21) | (start_cmd << 3);
}
 
/*
static unsigned int nand_cmd_phase_with_stop(unsigned char start_cmd, unsigned char stop_cmd, unsigned int cycle_cnt)
{
	return (0x77 << 24) | ((cycle_cnt & 0x7) << 21) | (1 << 20) | (stop_cmd << 11) | (start_cmd << 3);
}
*/
static unsigned int nand_data_phase(unsigned char stop_cmd)
{		
	return (0x77 << 24) | (1 << 19) | (stop_cmd << 11);
}

//------------------------------------------------------------
void nand_flash_Init(void)
{
	PORT_InitTypeDef PORT_InitStructure;
	
	RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTA | RST_CLK_PCLK_PORTC | RST_CLK_PCLK_PORTD | RST_CLK_PCLK_PORTE | RST_CLK_PCLK_PORTF, ENABLE);			
	// portA - data/cmd bus	
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_MAIN;
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;
	
	PORT_InitStructure.PORT_Pin = PORT_Pin_0 | PORT_Pin_1 | PORT_Pin_2 | PORT_Pin_3 |  
																PORT_Pin_4 | PORT_Pin_5 | PORT_Pin_6 | PORT_Pin_7;	
	PORT_Init(MDR_PORTA, &PORT_InitStructure);	
	
	// 0 - NWR, 1 - NRD, 2 - CLKO
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_MAIN;
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;
	
	PORT_InitStructure.PORT_Pin = PORT_Pin_0 | PORT_Pin_1 | PORT_Pin_2;	
	PORT_Init(MDR_PORTC, &PORT_InitStructure);	
	
	// BUSY
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_ALTER;
	PORT_InitStructure.PORT_Pin = PORT_Pin_15;	
	PORT_Init(MDR_PORTC, &PORT_InitStructure);	
	
	// cmd lanch
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_ALTER;
	PORT_InitStructure.PORT_Pin = PORT_Pin_1;	
	PORT_Init(MDR_PORTD, &PORT_InitStructure);	
	
	// CS
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	PORT_InitStructure.PORT_Pin = PORT_Pin_0;	
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;	
	PORT_Init(MDR_PORTE, &PORT_InitStructure);	
	PORT_WriteBit(MDR_PORTE, PORT_Pin_0, Bit_RESET);
	
	// write protect
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	PORT_InitStructure.PORT_Pin = PORT_Pin_2;	
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;	
	PORT_Init(MDR_PORTF, &PORT_InitStructure);	
	PORT_WriteBit(MDR_PORTF, PORT_Pin_2, Bit_SET);
	
	// ext bus config
	RST_CLK_PCLKcmd(RST_CLK_PCLK_EBC, ENABLE);
	EBC_StructInit(&EBC_InitStruct);
	EBC_InitStruct.EBC_Mode					 = EBC_MODE_NAND;
  EBC_InitStruct.EBC_NandTrc       = EBC_NAND_CYCLES_6HCLK;
  EBC_InitStruct.EBC_NandTwc       = EBC_NAND_CYCLES_6HCLK;
  EBC_InitStruct.EBC_NandTrea      = EBC_NAND_CYCLES_4HCLK;
  EBC_InitStruct.EBC_NandTwp       = EBC_NAND_CYCLES_3HCLK;
  EBC_InitStruct.EBC_NandTwhr      = EBC_NAND_CYCLES_6HCLK;
  EBC_InitStruct.EBC_NandTalea		 = EBC_NAND_CYCLES_10HCLK;
  EBC_InitStruct.EBC_NandTrr       = EBC_NAND_CYCLES_2HCLK;	
	//EBC_InitStruct.EBC_UseMemRegion  = ENABLE;
	EBC_Init(&EBC_InitStruct);	
/*
	EBC_MemRegionStructInit(&EBC_MemRegionInitStruct);
	EBC_MemRegionInit(&EBC_MemRegionInitStruct, EBC_MEM_REGION_70000000);	
*/	
}

unsigned int nand_flash_ReadID(void)	
//void nand_flash_ReadID(char write_id)	
{
	int id[5];
	//char write_id;
	unsigned int addr = nand_cmd_phase_without_stop(READ_ID, 1);	
	unsigned int addr_data = nand_data_phase(0);
 	*(volatile unsigned char*)addr = 0x00;
	
	
	id[0] = *(volatile unsigned char*)addr_data;
	id[1] = *(volatile unsigned char*)addr_data;
	id[2] = *(volatile unsigned char*)addr_data;
	id[3] = *(volatile unsigned char*)addr_data;
	id[4] = *(volatile unsigned char*)addr_data;

	
	 
	return (id[0] << 16) | (id[1] << 8) | id[2];
	//return id[0];
}
//status reading
unsigned int nand_flash_ReadStatus(void)
{
	unsigned int status;
	//command
	unsigned int addr = nand_cmd_phase_without_stop(READ_STATUS, 1);
	status = *((volatile unsigned char *)(0x77280000));
	return status;	
}
//Generate Address
bool nand_flash_TranslateAddress(unsigned int column_address, unsigned int block_num, unsigned int page_num, unsigned int buffer_bytes[5])
{ buffer_bytes[0] = 0;
	buffer_bytes[1] = 0;
	buffer_bytes[2] = 0;
	buffer_bytes[3] = 0;
	buffer_bytes[4] = 0;
	//5
	buffer_bytes[4] = (unsigned int)((block_num >> 10) & 0x07);
	//4
	buffer_bytes[3] = (unsigned int)((block_num >> 2) & 0xFF);
	//3
	buffer_bytes[2] = ((block_num << 6) & 0xC0) | (page_num & 0x3F);
	//2
	buffer_bytes[1] = (unsigned int)((column_address >> 8) & 0x0F);
	//1
	buffer_bytes[0] = (unsigned int)(column_address  & 0xFF);
	
	return true;
} 
void nand_flash_SetWP(BitAction BA)
{if (BA == Bit_RESET)
		PORT_ResetBits(MDR_PORTF, PORT_Pin_2);
	else
		PORT_SetBits(MDR_PORTF, PORT_Pin_2);		
}
	
//PageRead
void nand_flash_ReadPage(unsigned int block_num, unsigned int page_num, unsigned int * flash_data)
{ unsigned int page_pointer;
	
	nand_flash_TranslateAddress(0, block_num, page_num, NF_AddressCycles);
	//command
	*((volatile unsigned char *)(0x11B18000)) = NF_AddressCycles[0];
	*((volatile unsigned char *)(0x11B18000)) = NF_AddressCycles[1];
	*((volatile unsigned char *)(0x11B18000)) = NF_AddressCycles[2];
	*((volatile unsigned char *)(0x11B18000)) = NF_AddressCycles[3];
	*((volatile unsigned char *)(0x11B18000)) = NF_AddressCycles[4];
	// ADR_Cycle = 5;
	// SCMD = 0x00
	// ECMD = 0x30
	while (EBC_GetBusyStatus() != SET) {};
		//Data
		for(page_pointer = 0; page_pointer < NF_PageSize-1; page_pointer++)
		{flash_data[page_pointer] = *((volatile unsigned char *)(0x11080000));
		}
		//Last byte
		flash_data[page_pointer] = *((volatile unsigned char *)(0x11280000));
		//return (buffer[0] << 16)|(buffer[1]<<8)|(buffer[2]);		
		//return flash_data[0];
}
//block erase
void nand_flash_BlockErase(unsigned int block_num)
{
	unsigned int status;
	nand_flash_TranslateAddress(0, block_num, 0, NF_AddressCycles);
	//Set WP
	nand_flash_SetWP(Bit_SET);
	//Command
	*((volatile unsigned char *)(0x11768300)) = NF_AddressCycles[2];
	*((volatile unsigned char *)(0x11768301)) = NF_AddressCycles[3];
	*((volatile unsigned char *)(0x11768302)) = NF_AddressCycles[4];
	// ADR_Cycle = 3
	// ECMD = 0xD0
	// SCMD = 0x60
	// Address 1 cycle = 0x11
	// Address 2 cycle = 0x22
	// Address 3 cycle = 0x33
	while (EBC_GetBusyStatus() != SET) {};
		//read status
		status = nand_flash_ReadStatus();
		//unset WP
		nand_flash_SetWP(Bit_RESET);
		/*if (status && NF_Status_Fail == NF_Status_Fail)
		{ return false;
		}
		else return true;*/
}
// Write page
void nand_flash_WritePage(unsigned int block_num, unsigned int page_num, unsigned int * buffer)
{ unsigned int page_pointer;	
	unsigned int status;
	nand_flash_TranslateAddress(0,block_num,page_num,NF_AddressCycles);
	//set WP
	nand_flash_SetWP(Bit_SET);
	//command
	*((volatile unsigned char *)(0x11A00400)) = NF_AddressCycles[0];
	*((volatile unsigned char *)(0x11A00400)) = NF_AddressCycles[1];
	*((volatile unsigned char *)(0x11A00400)) = NF_AddressCycles[2];
	*((volatile unsigned char *)(0x11A00400)) = NF_AddressCycles[3];
	*((volatile unsigned char *)(0x11A00400)) = NF_AddressCycles[4];
	// ADR_cycle = 5
	// SCMD = 0x80
	//DATA
	for (page_pointer = 0; page_pointer < NF_PageSize-1; page_pointer++)
	{ 
		*((volatile unsigned char *)(0x11088000)) = buffer[page_pointer];		
	}
	// Last byte
	*((volatile unsigned char *)(0x11388000)) = buffer[NF_PageSize-1];
	// ECMD = 0x10
	// N from 1 to 528
	while (EBC_GetBusyStatus() != SET) {};
		
		//read status
		status = nand_flash_ReadStatus();
		//Unset WP
		nand_flash_SetWP(Bit_RESET);
		/*if (status && NF_Status_Fail == NF_Status_Fail)
		{
			return false;
		}
		else return true;	*/
	}		