#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"

#include "keyboard.h"

#include "cb.h"

//#define __DEBUG_KEYBOARD_H__
#ifdef __DEBUG_KEYBOARD_H__
	#include "led.h"
	#include "uart.h"
#endif

#define BUF_SIZE 128
static char KeyBuf[BUF_SIZE];
static TCB KeyCBIn;

void keyboard_Init(void)
{
	PORT_InitTypeDef PORT_InitStructure;
	//  
	RST_CLK_PCLKcmd(RST_CLK_PCLK_RST_CLK | RST_CLK_PCLK_PORTE, ENABLE);

	PORT_InitStructure.PORT_OE = PORT_OE_IN;	
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	
	PORT_InitStructure.PORT_Pin = PORT_Pin_8;	
	PORT_Init(MDR_PORTE, &PORT_InitStructure);	
	//  
	timcap1_Init();		
	cb_Init(&KeyCBIn, KeyBuf, BUF_SIZE);
}

//       
void keyboard_IRQProc(void)
{
	static unsigned char bit_pos = 0;	
	static short ch = 0;
	bool bit;
	//     
	bit = !!(PORT_ReadInputData(MDR_PORTE) & PORT_Pin_8);	
	
	if(bit){
		ch |= 1 << bit_pos;				
	}	
	bit_pos++;	
	
	if(bit_pos == 11){		
		bit_pos = 0;				
		ch >>= 1;
		ch &= 0xFF;				
		cb_AddByte(&KeyCBIn, (char)ch);				
		ch = 0;				
	}
}

/*
static unsigned int GetByteCnt(void)
{
	unsigned int bcnt;
	TIM1_LOCK();
	bcnt = cb_GetByteCnt(&KeyCBIn);
	TIM1_UNLOCK();
	return bcnt;
}
*/

static unsigned char GetByte(void)
{
	unsigned short key, res = 0;
	TIM1_LOCK();
	if(cb_GetByte(&KeyCBIn, (char*)&key)) res = 1;
	TIM1_UNLOCK();	
	return res ? key : 0;	
}

enum{
	KB_WAIT_BYTE,
	KB_WAIT_BYTE_SHIFT,
	KB_WAIT_MULTU_BYTE,
	KB_IGNORE_NEXT,
	KB_CNT
};
static unsigned char KBState = KB_WAIT_BYTE;
static unsigned char IgnoreNext, ShiftOn;

unsigned char keyboard_GetKey(void)
{
	unsigned char tkey;
	
	switch(KBState){
		case KB_WAIT_BYTE:
			if((tkey = GetByte()) != 0){
				switch(tkey){
					case 0xF0:
						if(!ShiftOn) IgnoreNext = 2;
						else{
							KBState = KB_WAIT_BYTE_SHIFT;
							if(IgnoreNext){tkey = 0; IgnoreNext--;}
							return 0;
						}
						break;
					case 0x12:	// L Shift
					case 0x59:	// R Shift
						ShiftOn = 1;
						tkey = 0;
						break;
					case 0x70: { tkey='0'; break;}                     
          case 0x69: { tkey='1'; break;}
          case 0x72: { tkey='2'; break;}
          case 0x7a: { tkey='3'; break;}
          case 0x6b: { tkey='4'; break;}
          case 0x73: { tkey='5'; break;}
          case 0x74: { tkey='6'; break;}
          case 0x6c: { tkey='7'; break;}
          case 0x75: { tkey='8'; break;}
          case 0x7d: { tkey='9'; break;}
            
          case 0x79: { tkey='+';  break;}
					case 0x7b: { tkey='-';  break;}
          case 0x7c: { tkey='*';  break;}
            
          case 0x45: { if (ShiftOn) tkey=')'; else tkey='0'; break;}
          case 0x16: { if (ShiftOn) tkey='!'; else tkey='1'; break;}
          case 0x1e: { if (ShiftOn) tkey='@'; else tkey='2'; break;}
          case 0x26: { if (ShiftOn) tkey='#'; else tkey='3'; break;}
          case 0x25: { if (ShiftOn) tkey='$'; else tkey='4'; break;}
          case 0x2e: { if (ShiftOn) tkey='%'; else tkey='5'; break;}
          case 0x36: { if (ShiftOn) tkey='^'; else tkey='6'; break;}
          case 0x3d: { if (ShiftOn) tkey='&'; else tkey='7'; break;}
          case 0x3e: { if (ShiftOn) tkey='*'; else tkey='8'; break;}
          case 0x46: { if (ShiftOn) tkey='('; else tkey='9'; break;}
          case 0x55: { if (ShiftOn) tkey='+'; else tkey='='; break;}
          case 0x4e: { if (ShiftOn) tkey='-'; else tkey='_'; break;}
          case 0x4a: { if (ShiftOn) tkey='?'; else tkey='/'; break;}
          case 0x41: { if (ShiftOn) tkey='<'; else tkey=','; break;}
          case 0x49: { if (ShiftOn) tkey='>'; else tkey='.'; break;}
                                               
					case 0x5a: { tkey='\n';  break;} // Enter
          case 0x66: { tkey=0x08;  break;} // Backspace.
          case 0x76: { tkey=0x1B;  break;} // Escape.
          case 0x71: { tkey=0x0F;  break;} // Delete
            
					case 0x29: { tkey=' '; break;}
					case 0x15: { if (ShiftOn) tkey='Q'; else tkey='q'; break;}
          case 0x1d: { if (ShiftOn) tkey='W'; else tkey='w'; break;}
          case 0x24: { if (ShiftOn) tkey='E'; else tkey='e'; break;}
          case 0x2d: { if (ShiftOn) tkey='R'; else tkey='r'; break;}
          case 0x2c: { if (ShiftOn) tkey='T'; else tkey='t'; break;}
          case 0x35: { if (ShiftOn) tkey='Y'; else tkey='y'; break;}
          case 0x3c: { if (ShiftOn) tkey='U'; else tkey='u'; break;}
          case 0x43: { if (ShiftOn) tkey='I'; else tkey='i'; break;}
          case 0x44: { if (ShiftOn) tkey='O'; else tkey='o'; break;}
          case 0x4d: { if (ShiftOn) tkey='P'; else tkey='p'; break;}

          case 0x1c: { if (ShiftOn) tkey='A'; else tkey='a'; break;}
          case 0x1b: { if (ShiftOn) tkey='S'; else tkey='s'; break;}
          case 0x23: { if (ShiftOn) tkey='D'; else tkey='d'; break;}
          case 0x2b: { if (ShiftOn) tkey='F'; else tkey='f'; break;}
          case 0x34: { if (ShiftOn) tkey='G'; else tkey='g'; break;}
          case 0x33: { if (ShiftOn) tkey='H'; else tkey='h'; break;}
          case 0x3b: { if (ShiftOn) tkey='J'; else tkey='j'; break;}
          case 0x42: { if (ShiftOn) tkey='K'; else tkey='k'; break;}
          case 0x4b: { if (ShiftOn) tkey='L'; else tkey='l'; break;}
            
          case 0x1a: { if (ShiftOn) tkey='Z'; else tkey='z'; break;}
          case 0x22: { if (ShiftOn) tkey='X'; else tkey='x'; break;}
          case 0x21: { if (ShiftOn) tkey='C'; else tkey='c'; break;}
          case 0x2a: { if (ShiftOn) tkey='V'; else tkey='v'; break;}
          case 0x32: { if (ShiftOn) tkey='B'; else tkey='b'; break;}
          case 0x31: { if (ShiftOn) tkey='N'; else tkey='n'; break;}
          case 0x3a: { if (ShiftOn) tkey='M'; else tkey='m'; break;}					
					
					case 0xe0:
						KBState = KB_WAIT_MULTU_BYTE;
						if(IgnoreNext){tkey = 0; IgnoreNext--;}
						return 0;
					default:
						if(IgnoreNext){tkey = 0; IgnoreNext--;}						
						return 0;
				}
				if(IgnoreNext){tkey = 0; IgnoreNext--;}
        return tkey;
			}
			else return 0;			
		case KB_WAIT_BYTE_SHIFT:
			if((tkey = GetByte()) != 0){
				if(tkey == 0x12 || tkey == 0x59)
					ShiftOn = 0;
				tkey = 0;					
				if(IgnoreNext){tkey = 0; IgnoreNext--;}
				KBState = KB_WAIT_BYTE;
        return tkey;
			}
			else return 0;			
		case KB_WAIT_MULTU_BYTE:
			if((tkey = GetByte()) != 0){
				if(tkey != 0xF0){
					switch(tkey){
						case 0x71: tkey=0x0F; break; 
						case 0x72: tkey=0x10; break; 
						case 0x75: tkey=0x11; break; 
						case 0x6b: tkey=0x12; break; 
						case 0x74: tkey=0x13; break; 
						case 0x5a: tkey='\n'; break; 
						case 0x4a: tkey='/'; break; 						            
					}					
				}
				else IgnoreNext = 2;
				if(IgnoreNext){tkey = 0; IgnoreNext--;}
				KBState = KB_WAIT_BYTE;
				return tkey;
			}
			else return 0;	
		default:
			return 0;			
	}
}

