#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"
#include "MDR32F9Qx_ssp.h"

#define __DEBUG_EXP__

#include "mux.h"
#include "mcp23s18.h"

#ifdef __DEBUG_EXP__
#include <stdio.h>
#include "itmr.h"
#include "led.h"
#include "disp.h"
static unsigned int stime;
static unsigned int period = 100;
static unsigned short Pin, Val;
#endif

#define ADDR_MCP23S18		0x20
#define READ_CMD	(((ADDR_MCP23S18) << 1) | 1)
#define WRITE_CMD	((ADDR_MCP23S18) << 1)

static uint16_t GPIOState = 0x0000;

static void HardwareInit(void)
{	
	PORT_InitTypeDef PORT_InitStructure;
	SSP_InitTypeDef sSSP;
	
	RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTC, ENABLE);	
	RST_CLK_PCLKcmd(RST_CLK_PCLK_SSP1, ENABLE);	
	
	PORT_InitStructure.PORT_OE = PORT_OE_IN;	
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;
	PORT_InitStructure.PORT_PULL_UP = PORT_PULL_UP_ON; 
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_ALTER;
	
	PORT_InitStructure.PORT_Pin = PORT_Pin_6; // SPI
	PORT_Init(MDR_PORTC, &PORT_InitStructure);	
	
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;			
	PORT_InitStructure.PORT_Pin = (PORT_Pin_5 | PORT_Pin_7); // SPI
	PORT_Init(MDR_PORTC, &PORT_InitStructure);	

	// SPI
	SSP_DeInit(MDR_SSP1);
	SSP_BRGInit(MDR_SSP1, SSP_HCLKdiv128); // 
	
	SSP_StructInit(&sSSP);

	sSSP.SSP_SCR = 0;
	sSSP.SSP_CPSDVSR = 2;
	sSSP.SSP_Mode = SSP_ModeMaster;
	sSSP.SSP_WordLength = SSP_WordLength8b;
	sSSP.SSP_SPH = SSP_SPH_2Edge;
	sSSP.SSP_SPO = SSP_SPO_High;
	sSSP.SSP_FRF = SSP_FRF_SPI_Motorola;
	sSSP.SSP_HardwareFlowControl = SSP_HardwareFlowControl_SSE;
	SSP_Init(MDR_SSP1, &sSSP);
	
	SSP_Cmd(MDR_SSP1, ENABLE);
	
	//mux_SetAddr(MUX_EXP_ADDR);	
}

static unsigned char SendAndRcvByte(unsigned char byte)
{
	char str[8];
	unsigned short rb;
	while (SSP_GetFlagStatus(MDR_SSP1, SSP_FLAG_TFE ) == RESET);
	SSP_SendData(MDR_SSP1, byte);
	while (SSP_GetFlagStatus(MDR_SSP1, SSP_FLAG_RNE ) == RESET);
	rb = SSP_ReceiveData(MDR_SSP1);	
	return rb;
}

static void WriteByte(uint8_t addr, uint8_t data){
	mux_SetAddr(MUX_EXP_ADDR);
	SendAndRcvByte(MCP23S18WRADDR);	
	SendAndRcvByte(addr);
	SendAndRcvByte(data);
	mux_ClearAddr();
}

static void WriteWord(uint8_t addr, uint16_t data){
	mux_SetAddr(MUX_EXP_ADDR);
	SendAndRcvByte(MCP23S18WRADDR);	
	SendAndRcvByte(addr);	
	SendAndRcvByte(data & 0xFF);	//LOW
	SendAndRcvByte(data >> 8);		// HIGH	
	mux_ClearAddr();//endSend();
}

static uint16_t ReadAddress(void){
	uint8_t l, h;
	mux_SetAddr(MUX_FLASH_ADDR);	
	//SendAndRcvByte(MCP23S18RDADDR);	
	SendAndRcvByte(0x9F/*GPIOA_BANK0*/);	
	l = SendAndRcvByte(0);
	h = SendAndRcvByte(0);
	mux_ClearAddr();
	return l | (h << 8);	
}

void mcp23s18_Init(void)
{
	#ifdef __DEBUG_EXP__
	Pin = 0; Val = 1;
	stime = itmr_GetTick();	
	#endif
	HardwareInit();	
	//WriteByte(IOCONA_BANK0, IOCON_MIRROR|IOCON_SEQOP);
}

// чтение состояния входов
uint16_t mcp23s18_ReadGPIO(void)
{
	return ReadAddress();
}

void mcp23s18_WriteGPIOPin(uint8_t pin, uint8_t value){
	/*
	if (pin < 16){//0...15
		!value ? (GPIOState &= ~(1 << pin)) : (GPIOState |= (1 << pin));
		WriteWord(GPIO, GPIOState);
	}*/
}

void mcp23s18_Test(void)
{	
	#ifdef __DEBUG_EXP__	
	char text[16];
	
	if(itmr_IsTime(stime, period)){							
		Val = mcp23s18_ReadGPIO();
		snprintf(text, 16, "val=%d", Val);
		disp_ShowStr(1, 1, text);		
		stime = itmr_GetTick();		
		led_Toggle(0);		
		
	}			
	#endif
}
