#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"

#include "stdio.h"
#include <string.h>

#include "disp.h"
#include "misc.h"
#include "itmr.h"
#include "cb.h"
#include "mouse.h"

#define __DEBUG_MOUSE
#ifdef __DEBUG_MOUSE	
	#include "uart.h"	
#endif

#define CMD_RESET					0xFF
#define CMD_STREAM_MODE		0xEA
#define CMD_ENABLE   			0xF4

enum{
	MODE_REQ,
	MODE_ACK,
	MODE_RES
};

enum{
	MW_SEND_RESET,
	MW_SEND_EN,	
	MW_RCV_ACK,
	MW_IDLE
};

static char MWState, MWNextState;
static unsigned char RcvByteNum; //    
static PORT_InitTypeDef PORT_InitStructure;
static unsigned char Mode, Cmd;
#define BUF_SIZE 6
static char AnswBuf[BUF_SIZE];
static TCB AnswCBIn;
#define POLL_PERIOD	(5000)
static unsigned int stime;
static bool RcvAfterSend = true;

static char LastXPos, LastYPos;
static int CurXPos, CurYPos;
static int CurXChPos, CurYChPos;

typedef bool (*TRcvCmdAnswer)(char* answ, unsigned char answ_size);
static TRcvCmdAnswer RcvCmdAnswer;

void mouse_Init(void)
{
	Mode = MODE_RES;
	MWState = MW_SEND_RESET;
	stime = itmr_GetTick();
	CurXPos = disp_XSize() >> 1;
	CurYPos = disp_YSize() >> 1;
	cb_Init(&AnswCBIn, AnswBuf, BUF_SIZE);	
}

//    
void mouse_SendRequest(unsigned char cmd, bool rcv_after_send)
{			
	//  
	RST_CLK_PCLKcmd(RST_CLK_PCLK_RST_CLK | RST_CLK_PCLK_PORTE, ENABLE);
  
	mouse_Init();
	
	TIM1_LOCK();
	//     
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;	
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	
	PORT_InitStructure.PORT_Pin = PORT_Pin_5 | PORT_Pin_8;	
	PORT_Init(MDR_PORTE, &PORT_InitStructure);	
	PORT_WriteBit(MDR_PORTE, PORT_Pin_5 | PORT_Pin_8, Bit_SET);	
	
	//     "0"
	PORT_WriteBit(MDR_PORTE, PORT_Pin_5, Bit_RESET);	
	delay(DELAY_100US);
	//     "0"
	PORT_WriteBit(MDR_PORTE, PORT_Pin_8, Bit_RESET);	
	//     "1" -       
	PORT_WriteBit(MDR_PORTE, PORT_Pin_5, Bit_SET);	
	//      
	Mode = MODE_REQ;	
	Cmd	= cmd;
	timcap1_Init();	
	RcvAfterSend = rcv_after_send;
}

unsigned int GetByteCnt(void)
{
	unsigned int res;
	TIM1_LOCK();
	res = cb_GetByteCnt(&AnswCBIn);
	TIM1_UNLOCK();	
	return res;
}

char GetByte(void)
{
	char ch;
	TIM1_LOCK();
	cb_GetByte(&AnswCBIn, &ch);
	TIM1_UNLOCK();	
	return ch;
}

// 
bool mouse_GetRespond(char* answ_buf, unsigned char byte_cnt)
{
	bool res = false;	
	
	if(GetByteCnt() >= byte_cnt){
		while(byte_cnt--){			
			*answ_buf++ = GetByte();
		}
		res = true;
	}
	
	return res;
}

void mouse_IRQProc(void)
{	
	static unsigned char bit_pos = 0, parity = 1;
	static short ch = 0;
		
	switch(Mode){
		case MODE_REQ:			
			if(!bit_pos) parity = 1;
			if(bit_pos < 8){												
				if((Cmd >> bit_pos++) & 0x01){
					PORT_WriteBit(MDR_PORTE, PORT_Pin_8, Bit_SET);
					parity ^= 1;
				}
				else{
					PORT_WriteBit(MDR_PORTE, PORT_Pin_8, Bit_RESET);					
					parity ^= 0;
				}
				return;
			}
			
			if(bit_pos == 8){
				if(parity){
					PORT_WriteBit(MDR_PORTE, PORT_Pin_8, Bit_SET);							
				}
				else{
					PORT_WriteBit(MDR_PORTE, PORT_Pin_8, Bit_RESET);											
				}
				bit_pos++;
				parity = 1;
				return;
			}
									
			if(bit_pos == 9){							
				//    
				PORT_WriteBit(MDR_PORTE, PORT_Pin_8, Bit_SET);				
				bit_pos = 0;
				Mode = MODE_ACK;
				//     
				PORT_InitStructure.PORT_OE = PORT_OE_IN;	
				PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
				PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
				PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;	
				PORT_InitStructure.PORT_Pin = PORT_Pin_8;	
				PORT_Init(MDR_PORTE, &PORT_InitStructure);												
			}
			break;
		case MODE_ACK:
			//    			
			while(PORT_ReadInputDataBit(MDR_PORTE, PORT_Pin_8) == Bit_SET);
			RcvAfterSend ? (Mode = MODE_RES) : (Mode = MODE_REQ);
			break;
		case MODE_RES:									
			//     				
			if(!!(PORT_ReadInputData(MDR_PORTE) & PORT_Pin_8))
				ch |= 1 << bit_pos;
			bit_pos++;				
			if(bit_pos == 11){											
				bit_pos = 0;				
				ch >>= 1;
				ch &= 0xFF;				
				cb_AddByte(&AnswCBIn, (char)ch);						
				ch = 0;				
			}
			break;
		default:
			break;
	}
}

static bool ResetAnswer(char* answ, unsigned char answ_size)
{
	if(answ_size == 3 && answ[0] == 0xFA && answ[1] == 0xAA && answ[2] == 0x00)
		return true;		
	return false;
}

char mouse_GetState(unsigned int* state)
{
	char answ_buf[6];			
	
	switch(MWState){		
		case MW_SEND_RESET:						
			mouse_SendRequest(CMD_RESET, true);
			RcvByteNum = 3;
			MWState = MW_RCV_ACK;
			stime = itmr_GetTick();
			MWNextState = MW_SEND_EN;
			RcvCmdAnswer = ResetAnswer;
			return 0;			
		case MW_SEND_EN:			
			mouse_SendRequest(CMD_ENABLE, true);
			RcvByteNum = 1;
			MWState = MW_RCV_ACK;
			stime = itmr_GetTick();
			MWNextState = MW_IDLE;//MW_SEND_GETPOS;
			RcvCmdAnswer = NULL;//RemoteModeAnswer;
			return 0;
		case MW_RCV_ACK:
			if(itmr_IsTime(stime, 2 * POLL_PERIOD)){
				MWState = MW_SEND_RESET;
				return 0;
			}			
			if(!mouse_GetRespond(answ_buf, RcvByteNum))
				return 0;
			//   callback -  
			if(RcvCmdAnswer != NULL){
				if(!RcvCmdAnswer(answ_buf, RcvByteNum))
					MWState = MW_SEND_RESET;
			}
			MWState = MWNextState;
			return 0;
		case MW_IDLE:						
			if(!mouse_GetRespond(answ_buf, 3))
				return 0;																							
			if(LastXPos != answ_buf[1]){
				LastXPos = answ_buf[1];
				if(LastXPos > 127)
					CurXPos += (int)LastXPos - 256;
				else
					CurXPos += LastXPos;
				if(CurXPos < 0) CurXPos = 0; else 
				if(CurXPos > disp_XPixel()) CurXPos = disp_XPixel();											
			}
			
			if(LastYPos != answ_buf[2]){
				LastYPos = answ_buf[2];
				if(LastYPos > 127)
					CurYPos += (int)LastYPos - 256;
				else
					CurYPos += LastYPos;
				if(CurYPos < 0) CurYPos = 0; else 
				if(CurYPos > disp_YPixel()) CurYPos = disp_YPixel();										
			}						
			disp_CharPosition(CurXPos, CurYPos, &CurXChPos, &CurYChPos);
						
			*state = (unsigned int)answ_buf[0];
			return 1;
		default:
			return 0;
	}	
}

TMousePos mouse_GetPos(void)
{
	TMousePos curpos;
	curpos.x = CurXChPos;
	curpos.y = CurYChPos;
	return curpos;
}

