/*
      .

  01234567890123456789
  --------------------

     : 01.01
    : 23:59:59

  --------------------

     ""     .

      - 
     .
*/

#include <string.h>
#include "misc.h"
#include "disp.h"
#include "button.h"
#include "console.h"
#include "wnds.h"
#include "rtc.h"
#include "itmr.h"
#include "keyboard.h"

#define __WND_TIME_SET_DBG__
#ifdef __WND_TIME_SET_DBG__
#include "led.h"
#include "uart.h"
#endif

static TTime Time;
static unsigned int stime;
static unsigned char blink = 0;

static bool EventProc(unsigned int code, unsigned int data);

static TWnd Wnd = { WID_TIME_SET, 0, EventProc, 0 };

    //   //

//  () :
enum { WE_DAY, WE_MON, WE_YEAR, WE_HOUR, WE_MIN, WE_SEC, WE_CNT };

//    (x, y):
static unsigned int ElemCoord[WE_CNT][2] = {
  { 9, 1 }, { 12, 1 }, { 15, 1 },
  { 9, 2 }, { 12, 2 }, { 15, 2 }
};

static unsigned int CurElem;  //    
static unsigned int CurPos;   //      (0 -  )

    //

//------------------------------------------------------------------------------
unsigned char Bin2BCD(unsigned char b)
{
  return (unsigned char)(((b / 10) << 4) | (b % 10));
}

//------------------------------------------------------------------------------
unsigned char BCD2Bin(unsigned char bcd)
{
  return (unsigned char)((((bcd >> 4) & 0x0F) * 10) + (bcd & 0x0F));
}

//------------------------------------------------------------------------------
//      Time   .
// args: ei -  .
static unsigned char* ElemValPtr(unsigned int ei)
{
  unsigned char* vp = 0;

  switch (ei) {
    case WE_DAY:  vp = (unsigned char*)&Time.date;   break;
    case WE_MON:  vp = (unsigned char*)&Time.month;   break;
		case WE_YEAR: vp = (unsigned char*)&Time.year;   break;
    case WE_HOUR: vp = (unsigned char*)&Time.hour;  break;
    case WE_MIN:  vp = (unsigned char*)&Time.min;   break;
    case WE_SEC:  vp = (unsigned char*)&Time.sec;   break;
  }

  return vp;
}

//------------------------------------------------------------------------------
//  .
static void DrawElem(unsigned int  ei, unsigned char en_blink)
{
  unsigned int x = ElemCoord[ei][0] + (2 - 1);
  unsigned int y = ElemCoord[ei][1];

  unsigned char val = *ElemValPtr(ei);
  unsigned int i;

  for (i = 0; i < 2; i++) {
    char ch = (val & 0x0F) + '0';    		
		disp_ShowCharEx(x, y, ch, en_blink ? blink : 0);				
    val >>= 4;
    x--;
  }
	blink = ~blink;
}

//------------------------------------------------------------------------------
//    .
static void GoNextPos(void)
{
  if (CurPos == 1) {
    CurPos = 0;
    DrawElem(CurElem, 1);
  }
  else {
    unsigned int prev_elem = CurElem;
    CurPos = 1;

    if (CurElem < (WE_CNT - 1)) CurElem++;
    else CurElem = 0;		
		
    DrawElem(prev_elem, 0);
    DrawElem(CurElem, 1);
  }
}

//------------------------------------------------------------------------------
//    .
static void GoPrevPos(void)
{
  if (CurPos == 0) {
    CurPos = 1;
    DrawElem(CurElem, 1);
  }
  else {
    unsigned int prev_elem = CurElem;
    CurPos = 0;

    if (CurElem != 0) CurElem--;
    else CurElem = (WE_CNT - 1);
		
    DrawElem(prev_elem, 0);
    DrawElem(CurElem, 1);
  }
}

//------------------------------------------------------------------------------
//     (     ).
static void ProcessDigit(int digit)
{
  unsigned char* vp = ElemValPtr(CurElem);
  unsigned char val = *vp;

  if (CurPos == 0) {
    val &= ~0x0F;
    val |= (unsigned char)digit;
  }
  else {
    val &= ~0xF0;
    val |= ((unsigned char)digit << 4);
  }

  //     :
  switch (CurElem) {
    case WE_DAY:
      if (val < 0x01) val = 0x01;
      if (val > 0x31) val = 0x31;

      //        ,
      //   :
      if (BCD2Bin(val) > rtc_GetDayInMonth(BCD2Bin(val))) {
        Time.month = 0x01;
        DrawElem(WE_MON, 0);   // !!!
      }

      break;

    case WE_MON:
    {
      unsigned char max_day;

      if (val < 0x01) val = 0x01;
      if (val > 0x12) val = 0x12;

      //     :
      max_day = Bin2BCD(rtc_GetDayInMonth(BCD2Bin(val)));			

      if (Time.date > max_day) {
        Time.date = max_day;
        DrawElem(WE_DAY, 0);   // !!!
      }

      break;
    }
		
		case WE_YEAR:
      if (val > 0x99) val = 0x99;
      break;

    case WE_HOUR:
      if (val > 0x23) val = 0x23;
      break;

    case WE_MIN:
      if (val > 0x59) val = 0x59;
      break;

    case WE_SEC:
      if (val > 0x59) val = 0x59;
      break;
  }

  *vp = val;
  DrawElem(CurElem, 0);
  GoNextPos();
}

static void SetTime(void){
	//      :
  Time.msec = 0;  // ( )

  Time.sec  = BCD2Bin(Time.sec);
  Time.min  = BCD2Bin(Time.min);
  Time.hour = BCD2Bin(Time.hour);
  Time.date  = BCD2Bin(Time.date);
  Time.month  = BCD2Bin(Time.month);
	Time.year = BCD2Bin(Time.year) + 2000;
	rtc_TimeSet(&Time);
}
//------------------------------------------------------------------------------
//  .
static bool EventProc(unsigned int code, unsigned int data)
{
  unsigned int i;  

  switch (code) {
    case WE_DRAW:   //  :      
			disp_ShowStr(0, 0, " ");	
			disp_ShowStr(2, 1, " :");
      disp_ShowChar(11, 1, '.');
			disp_ShowChar(14, 1, '.');
			disp_ShowStr(2, 2, ":");
      disp_ShowChar(11, 2, ':');
      disp_ShowChar(14, 2, ':');
      for (i = 0; i < WE_CNT; i++) DrawElem(i, 0);
      break;
		//------------------------------------------------------------
		//  
    case WE_BUTTON:
      switch (data) {
        case SELECT:
					SetTime();
          console_CloseWnd(0);
          break;
        case LEFT: 		GoPrevPos(); break;
        case RIGHT: 	GoNextPos(); break;				
        case UP:								
				case DOWN: 		console_CloseWnd(0); break;								
        default: 			break;
      }
      break;		
		//------------------------------------------------------------
		//  
		case WE_KEYS:
			if(data >= '0' && data <= '9'){
				data -= '0';
				ProcessDigit(data);
			}
			else{
				switch(data){
					case KEY_ENTER: 	SetTime(); 									// enter
					case KEY_ESC: 		console_CloseWnd(0); break; // Esc
					case KEY_UP:			break;  										// UP
					case KEY_DOWN: 		break;											// DOWN
					case KEY_LEFT: 		GoPrevPos(); break;					// LEFT
					case KEY_RIGTH: 	GoNextPos(); break;					// RIGTH
					default:												
						break;
				}
			}
			break;
		//------------------------------------------------------------
		//    
		case WE_CYCLE:
			if(itmr_IsTime(stime, 500ul)){
				DrawElem(CurElem, 1);
				stime = itmr_GetTick();
			}
			break;		
  }

  return true;  // !!!
}

//------------------------------------------------------------------------------
//  .
void wnd_TimeSet(void)
{  
	stime = itmr_GetTick();
	Time = rtc_GetTime();
	Time.year -= 2000;
	//      
  Time.sec  = Bin2BCD(Time.sec);
  Time.min  = Bin2BCD(Time.min);
  Time.hour = Bin2BCD(Time.hour);
  Time.date  = Bin2BCD(Time.date);
  Time.month  = Bin2BCD(Time.month);
	Time.year = Bin2BCD(Time.year);

  CurElem = WE_DAY;
  CurPos = 1;

  console_OpenWnd(&Wnd);
}
