#include <stdio.h>
#include <string.h>
#include "itmr.h"
#include "rtc.h"
#include "buzzer.h"
#include "button.h"
#include "keyboard.h"
#include "mouse.h"
#include "encoder.h"
#include "disp.h"
#include "console.h"
#include "error.h"
#include "main.h"


//#define __CONSOLE_DBG__
#ifdef __CONSOLE_DBG__
#include "led.h"
#include "uart.h"
#endif

static TWnd* CurWnd;
static unsigned int stime, stime_rtc;

#ifdef __CONSOLE_DBG__
static char test[128];
#endif

static bool fCurWndVisible;

#define CHAR_WE_MRK_EN        '>'    
#define CHAR_WE_MRK_DIS       '-'
#define CHAR_WE_MRK_EDITING   '*'

#define IsEditing()						(CurWnd->editing)
#define IsWndElemActive(ei)   (CurWnd->elem_table[ei].num != NO_WND_ELEM_NUM)
#define IsWndElemEnabled(ei)  !(CurWnd->elem_table[ei].attr & WEA_DISABLED)
#define IsWndElemVisible(ei)	!(CurWnd->elem_table[ei].attr & WEA_HIDDEN)

void console_CloseWnd(unsigned short res);

//------------------------------------------------------------------------------
static void DrawElemMarker(unsigned int ei, bool draw)
{
  TWndElem* we = &CurWnd->elem_table[ei];

  if (!(we->attr & WEA_NO_NUM_MARK)) {
    char ch = draw ? IsWndElemEnabled(ei) ? CHAR_WE_MRK_EN : CHAR_WE_MRK_DIS : ' ';

    if (CurWnd->editing && (ei == CurWnd->cur_elem))  ch = CHAR_WE_MRK_EDITING;

    disp_ShowChar((we->x + 1), (we->y - CurWnd->y_top), ch);
  }
}

//------------------------------------------------------------------------------
static void DrawElemNum(unsigned int ei)
{
  TWndElem* we = &CurWnd->elem_table[ei];

  if (!(we->attr & WEA_NO_NUM_MARK))
		disp_ShowChar(we->x, (we->y - CurWnd->y_top), (char)(we->num + '0'));				  				
}

//------------------------------------------------------------------------------
static void DrawWndElem(unsigned int ei)
{
  TWndElem* we = &CurWnd->elem_table[ei];
  unsigned int tlen = we->text ? strlen(we->text) : 0;  
  bool elem_edit = ((ei == CurWnd->cur_elem) && CurWnd->editing);  
  
  int x = we->x;
  int y = we->y - CurWnd->y_top;

  if ((y < 0) || (y >= disp_YSize()) || (we->attr & WEA_HIDDEN)) return; 
  
  if (IsWndElemActive(ei) && !(we->attr & WEA_NO_NUM_MARK)) {
    DrawElemNum(ei);
    if (ei == CurWnd->cur_elem) DrawElemMarker(ei, true);
    x += 2;  // !!!
  }
  
  if (tlen) {
    disp_ShowStr(x, y, we->text);
    x += tlen;  // !!!
  }

  switch (we->type) {
    case WET_SELECTOR:
    {
      TWESelector* sel = (TWESelector*)we->data;
      disp_ShowStr(x, y, sel->val_text[sel->cur_val]);			
      break;
    }

    case WET_NUM_EDIT:
    {
      TWENumEdit* ne = (TWENumEdit*)we->data;
      unsigned char* ptr = ne->bcd;
      bool disp_zeroes = (!elem_edit && (we->attr & WEA_LEADING_ZEROES));
      unsigned int i;
      
      for (i = 0; i < ne->num_pos; i++) {        
        unsigned char digit = *ptr++;
        char ch = (char)(digit + '0');

        if ((digit != 0) || (i == (ne->num_pos - 1))) disp_zeroes = true;

        if ((digit == 0) && !disp_zeroes) ch = ' ';

        disp_ShowChar(x++, y, ch);				
      }
      break;
    }  
  }
}

//------------------------------------------------------------------------------
static void DrawWnd(unsigned int reason)
{
  TWndElem* we;
  unsigned int ei = 0;	

  fCurWndVisible = true;
		
	disp_ClearRegion(0, 0, disp_XSize(), disp_YSize());
  
  CurWnd->event_proc(WE_DRAW, (unsigned short)reason);

  if (!CurWnd->elem_table) return;    

  for (we = CurWnd->elem_table; (we->type != WET_NONE); we++, ei++) DrawWndElem(ei);
}

//------------------------------------------------------------------------------
static int MakeElemVisible(unsigned int ei)
{
  int y = CurWnd->elem_table[ei].y - CurWnd->y_top;

  if (y < 0) {
    CurWnd->y_top += y;
    return 1;
  }
  else {
    if (y >= disp_YSize()) {
      CurWnd->y_top += (y - disp_YSize()) + 1;
      return 1;
    }
    else  return 0;
  }
}

//------------------------------------------------------------------------------
static void ChangeCurElem(unsigned int new_cur_elem)
{
  if (new_cur_elem == CurWnd->cur_elem) return;

  if (MakeElemVisible(new_cur_elem) == 0) {
    DrawElemMarker(CurWnd->cur_elem, false);
    DrawElemMarker(new_cur_elem, true);
    CurWnd->cur_elem = new_cur_elem;
  }
  else {
    CurWnd->cur_elem = new_cur_elem;
    DrawWnd(1);
  }
}

//------------------------------------------------------------------------------
static void EnterEdit(void)
{
  CurWnd->editing = true;

  DrawWndElem(CurWnd->cur_elem);
}

//------------------------------------------------------------------------------
static void LeaveEdit(bool ok)
{
  TWnd* wnd = CurWnd;

  CurWnd->editing = false;
  CurWnd->event_proc(WE_EDIT_END, (unsigned short)ok);  
  if (wnd == CurWnd) DrawWndElem(CurWnd->cur_elem);
}

//------------------------------------------------------------------------------
static void ButtonProc(unsigned int code, unsigned int data)
{
  if(code == WE_BUTTON){		
		if (data == SELECT) CurWnd->event_proc(WE_ELEM_ACT, 0);		
	}
	if(code == WE_KEYS){		
		if (data == KEY_ENTER) CurWnd->event_proc(WE_ELEM_ACT, 0);		
	}
}

//------------------------------------------------------------------------------
static void SelectorProc(unsigned int code, unsigned int data)
{
  TWndElem* we = &CurWnd->elem_table[CurWnd->cur_elem];
  TWESelector* sel = (TWESelector*)we->data;
	
  if(code == WE_BUTTON){
		switch (data) {
			case SELECT:
				if (!IsEditing()) {
					EnterEdit();
					sel->saved_val = sel->cur_val;
				}
				else LeaveEdit(true);
				return;   // !!!

			case LEFT:  
				sel->cur_val = sel->saved_val;
				LeaveEdit(false);
				return;   // !!!

			case UP:        
				if (sel->cur_val > 0) sel->cur_val--;
				else                  sel->cur_val = sel->val_cnt - 1;

				break;

			case DOWN:      
				if (sel->cur_val < (sel->val_cnt - 1)) sel->cur_val++;
				else                                   sel->cur_val = 0;
				break;

			default: return;
		}
		

		if (!CurWnd->event_proc(WE_EDIT_CHANGE, (data == UP) ? 0 : 1)){
			DrawWndElem(CurWnd->cur_elem);
		}
	}

	if(code == WE_KEYS){
		switch (data) {
			case KEY_ENTER:				
				if (!IsEditing()) {
					EnterEdit();
					sel->saved_val = sel->cur_val;
				}
				else LeaveEdit(true);

				return;   // !!!

			case KEY_ESC:  				
				sel->cur_val = sel->saved_val;
				LeaveEdit(false);
				return;   // !!!

			case KEY_UP:
				if (sel->cur_val > 0) sel->cur_val--;
				else                  sel->cur_val = sel->val_cnt - 1;				
				break;

			case KEY_DOWN:      				
				if (sel->cur_val < (sel->val_cnt - 1)) sel->cur_val++;
				else                                   sel->cur_val = 0;				
				break;

			default: return;
		}
		
		if (!CurWnd->event_proc(WE_EDIT_CHANGE, (data == KEY_UP) ? 0 : 1)){			
			DrawWndElem(CurWnd->cur_elem);
		}
	}
}

//------------------------------------------------------------------------------
void NumEdit_SetVal(TWENumEdit* ne, unsigned short val)
{
  unsigned int left_cnt = ne->num_pos;
  unsigned char* ptr = ne->bcd + left_cnt;

  ne->cur_val = val;                    // !!!

  for ( ; left_cnt; left_cnt--) {
    *(--ptr) = (unsigned char)(val % 10);
    val = val / 10;
    if (!val)  { left_cnt--; break; }
  }
  
  while (left_cnt--)  *(--ptr) = 0;
}

//------------------------------------------------------------------------------
void NumEdit_UpdateVal(TWENumEdit* ne)
{
  unsigned short val = 0;
  unsigned int left_cnt = ne->num_pos;
  unsigned char* ptr = ne->bcd;

  while (left_cnt--) {
    val *= 10;
    val += *ptr++;
  }

  ne->cur_val = val;
}

//------------------------------------------------------------------------------
static void NumEditProc(unsigned int code, unsigned int data)
{
  TWndElem* we = &CurWnd->elem_table[CurWnd->cur_elem];
  TWENumEdit* ne = (TWENumEdit*)we->data;
	unsigned int cnt;
	unsigned char* ptr;

  if(code == WE_BUTTON){
		switch (data) {
			case SELECT:
				if (!IsEditing()) {
					ne->edit_state = 0;   // !!!
					EnterEdit();
					ne->saved_val = ne->cur_val;
					button_SetFastMode(1);
				}
				else LeaveEdit(true);				
				return;   // !!!

			case LEFT:
				NumEdit_SetVal(ne, ne->saved_val);
				LeaveEdit(false);
				button_SetFastMode(0);
				return;   // !!!

			case UP:
				if (ne->cur_val == ne->max_val)  return;  
				NumEdit_SetVal(ne, (unsigned short)(ne->cur_val + 1));
				ne->edit_state = 0;      
				if (CurWnd->event_proc(WE_EDIT_CHANGE, 0))  return;
				break;

			case DOWN:    
				if (ne->cur_val == ne->min_val) return;   
				NumEdit_SetVal(ne, (unsigned short)(ne->cur_val - 1));
				ne->edit_state = 0;      
				if (CurWnd->event_proc(WE_EDIT_CHANGE, 0))  return;
				break;

			default:
				break;
		}
	}
	
	if(code == WE_KEYS){		
		switch(data){
			case KEY_ESC:
				NumEdit_SetVal(ne, ne->saved_val);
				LeaveEdit(false);
				return;   // !!!				
			case KEY_ENTER:
				if (!IsEditing()) {
					ne->edit_state = 0;   // !!!
					EnterEdit();
					ne->saved_val = ne->cur_val;
				}
				else LeaveEdit(true);
				return;   // !!!
			case KEY_UP:
				if (ne->cur_val == ne->max_val)  return;  
				NumEdit_SetVal(ne, (unsigned short)(ne->cur_val + 1));
				ne->edit_state = 0;      
				if (CurWnd->event_proc(WE_EDIT_CHANGE, 0))  return;
				break;
				
			case KEY_DOWN:
				if (ne->cur_val == ne->min_val) return;   
				NumEdit_SetVal(ne, (unsigned short)(ne->cur_val - 1));
				ne->edit_state = 0;      
				if (CurWnd->event_proc(WE_EDIT_CHANGE, 0))  return;
				break;
			default:
				if(data >= '0' && data <= '9'){
					data -= '0';												
					cnt = ne->num_pos - 1;
					ptr = ne->bcd;

					if (ne->edit_state == 0) {
						for ( ; cnt--; ptr++) *ptr = 0;
					}
					else {
						for ( ; cnt--; ptr++) *ptr = *(ptr + 1);
					}

					*ptr = (unsigned char)data;

					if (++ne->edit_state == ne->num_pos) ne->edit_state = 0;		
			
					NumEdit_UpdateVal(ne);
			
					if (ne->cur_val < ne->min_val) {			
						NumEdit_SetVal(ne, ne->min_val);
						ne->edit_state = 0;
					}
					else if (ne->cur_val > ne->max_val) {			
						NumEdit_SetVal(ne, ne->max_val);
						ne->edit_state = 0;
					}
        
					if (CurWnd->event_proc(WE_EDIT_CHANGE, 0)) return;											
			}
			else return;	
		}				
	}
  DrawWndElem(CurWnd->cur_elem);
}

//------------------------------------------------------------------------------
static bool ElemProc(unsigned int code, unsigned int data)
{
  if(code == WE_BUTTON){
		if (CurWnd->editing || (data == SELECT)) {
			if ((CurWnd->cur_elem != NO_WND_CUR_ELEM) && IsWndElemEnabled(CurWnd->cur_elem)){
				switch (CurWnd->elem_table[CurWnd->cur_elem].type) {
					case WET_BUTTON:
						ButtonProc(code, data);
						break;
					case WET_SELECTOR:						
						SelectorProc(code, data);
						break;
					case WET_NUM_EDIT:						
						NumEditProc(code, data);
						break;       
				}
			}
			return true;
		}
		else  return false;
	}
	
	if(code == WE_KEYS){		
		if (CurWnd->editing || (data == KEY_ENTER)) {
			if ((CurWnd->cur_elem != NO_WND_CUR_ELEM) && IsWndElemEnabled(CurWnd->cur_elem)){
				switch (CurWnd->elem_table[CurWnd->cur_elem].type) {
					case WET_BUTTON:
						ButtonProc(code, data);
						break;
					case WET_SELECTOR:
						SelectorProc(code, data);
						break;
					case WET_NUM_EDIT:						
						NumEditProc(code, data);
						break;					
				}				
			}
			return true;
		}
		else return false;
	}
	
	return false;
}

//------------------------------------------------------------------------------
static void WndProc(unsigned int code, unsigned int data)
{
  unsigned int ei;

  if(code == WE_BUTTON){
		switch (data) {
			case LEFT:
				if (CurWnd->next) {        
					console_CloseWnd(0);
					return;
				}
				break;

			case UP:
				if (CurWnd->cur_elem == NO_WND_CUR_ELEM)  return;

				for (ei = CurWnd->cur_elem; ei != 0; ) {
					ei--;
					if (IsWndElemActive(ei) && IsWndElemVisible(ei)) {
						ChangeCurElem(ei);
						return;
					}
				}
				break;

			case DOWN:
				if (CurWnd->cur_elem == NO_WND_CUR_ELEM)  return;

				for (ei = CurWnd->cur_elem + 1; ei != CurWnd->elem_cnt; ei++) {
					if (IsWndElemActive(ei) && IsWndElemVisible(ei)) {
						ChangeCurElem(ei);
						return;
					}
				}
				break;		
		}
	}

	if(code == WE_KEYS){		
		switch (data) {
			case KEY_ESC:
				if (CurWnd->next) {        
					console_CloseWnd(0);
					return;
				}
				break;

			case KEY_UP:
				if (CurWnd->cur_elem == NO_WND_CUR_ELEM)  return;

				for (ei = CurWnd->cur_elem; ei != 0; ) {
					ei--;
					if (IsWndElemActive(ei) && IsWndElemVisible(ei)) {
						ChangeCurElem(ei);						
						return;
					}
				}
				break;

			case KEY_DOWN:
				if (CurWnd->cur_elem == NO_WND_CUR_ELEM)  return;

				for (ei = CurWnd->cur_elem + 1; ei != CurWnd->elem_cnt; ei++) {
					if (IsWndElemActive(ei) && IsWndElemVisible(ei)) {
						ChangeCurElem(ei);						
						return;
					}
				}
				break;		
		}
	}
	
}

//------------------------------------------------------------------------------
static bool CloseWnd(void)
{
  if (CurWnd && CurWnd->next) {
    CurWnd = CurWnd->next;
    fCurWndVisible = false;   // !!!

    return true;
  }
  else return false;
}

//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
// 
void console_Init(void)
{
	CurWnd = 0;
	fCurWndVisible = false;
	stime = stime_rtc = itmr_GetTick();
}

//------------------------------------------------------------------------------
//  
void console_OpenWnd(TWnd* wnd)
{
  wnd->editing = false;
  wnd->next = CurWnd;
  CurWnd = wnd;
  
  wnd->y_top = 0;
  wnd->cur_elem = NO_WND_CUR_ELEM;
  
  wnd->elem_cnt = 0;

  if (wnd->elem_table) {
    TWndElem* we;
    unsigned int ei = 0, cur_num = 1;    

    for (we = wnd->elem_table; (we->type != WET_NONE); we++, ei++) {
      if (we->type != WET_STATIC) {
        we->num = cur_num;
        if (++cur_num == 10) cur_num = 0;

        if ((wnd->cur_elem == NO_WND_CUR_ELEM) && !(we->attr & WEA_HIDDEN))
          wnd->cur_elem = ei;
      }
      else we->num = NO_WND_ELEM_NUM;
    }

    wnd->elem_cnt = ei;
	}

  CurWnd->event_proc(WE_SET_CONTEXT, 0);
  DrawWnd(0);
}

//------------------------------------------------------------------------------
void console_CloseWnd(unsigned short res)
{
  if (CloseWnd()) {
    CurWnd->event_proc(WE_PREV_WND_CLOSED, res);
    DrawWnd(0);
  }
}

//------------------------------------------------------------------------------
//   
void console_Work(void)
{
  TTime CurTime;	
	char time[32];
	bool event = false;
	//   
	unsigned short button = button_Press();
	//     
	unsigned short key;
	//  
	unsigned int enc;
	
	unsigned int mouse_state;
	
	
	if (button != NOPRESS){  			
		event = true;		
		if (CurWnd && !CurWnd->event_proc(WE_BUTTON, (unsigned int)button)){
			if (!ElemProc(WE_BUTTON, button)) WndProc(WE_BUTTON, button);
		}
	}
	
	else if(((key = keyboard_GetKey()) != 0) && (wnd_choose == 1)){
		event = true;		
		if (CurWnd && !CurWnd->event_proc(WE_KEYS, (unsigned int)key)){    
			if (!ElemProc(WE_KEYS, key)) WndProc(WE_KEYS, key);
		}		
	}
	
	else if(encoder_GetValue(&enc)){
		event = true;		
		if (CurWnd && !CurWnd->event_proc(WE_ENCODER, enc)){    
			if (!ElemProc(WE_ENCODER, key)) WndProc(WE_ENCODER, key);
		}
	}
	
	else if(mouse_GetState(&mouse_state) && wnd_choose == 2){
		event = true;		
		if (CurWnd && !CurWnd->event_proc(WE_MOUSE, mouse_state)){    
			if (!ElemProc(WE_MOUSE, mouse_state)) WndProc(WE_MOUSE, mouse_state);
		}
	}
	
	if(CurWnd) CurWnd->event_proc(WE_CYCLE, 0);	
	
	//   
	if(event){
		BUZZER;											// 
		disp_LedOn();							// 
		stime = itmr_GetTick();
	}
	
	//  
	if(itmr_IsTime(stime, 10000ul)){		
		disp_LedOff();
	}	
	
	if(itmr_IsTime(stime_rtc, 1000ul)){
		CurTime = rtc_GetTime();							
		sprintf(time, "%02d.%02d.%02d %02d:%02d:%02d%s", 
						CurTime.date, 
						CurTime.month, 
						CurTime.year - 2000, 
						CurTime.hour, 
						CurTime.min, 
						CurTime.sec,
						error_Is() ? " !" : "  ");
		disp_ShowStr(0, disp_YSize(), time);	
		stime_rtc = itmr_GetTick();
	}
}

//------------------------------------------------------------------------------
void console_DrawWndElem(TWnd* wnd, unsigned int ei)
{
  if ((wnd != CurWnd) || !fCurWndVisible) return;

  DrawWndElem(ei);
}

//------------------------------------------------------------------------------
void console_RemoveWnd(unsigned short res)
{
  CloseWnd();
  CurWnd->event_proc(WE_PREV_WND_CLOSED, res);
}

//------------------------------------------------------------------------------
void console_RestoreWnd(unsigned int wid)
{
  while ((CurWnd->id != wid) && CloseWnd()) ;

  DrawWnd(0);
}

//------------------------------------------------------------------------------
void console_DrawWnd(void)
{
  DrawWnd(0);
}

//------------------------------------------------------------------------------
void console_SetCurWndElem(unsigned int ei)
{
  CurWnd->cur_elem = ei;
  MakeElemVisible(ei);
}

//------------------------------------------------------------------------------
void console_ActivateElem(unsigned int ei)
{
  ChangeCurElem(ei);    
  ElemProc(WE_BUTTON, SELECT);  
}

//------------------------------------------------------------------------------
void console_ShowWndElem(unsigned int ei, bool show)
{
  if (show)  CurWnd->elem_table[ei].attr &= ~WEA_HIDDEN;
  else       CurWnd->elem_table[ei].attr |=  WEA_HIDDEN;
}

//------------------------------------------------------------------------------
void console_EnableWndElem(unsigned int ei, bool enable)
{
  if (enable) CurWnd->elem_table[ei].attr &= ~WEA_DISABLED;
  else        CurWnd->elem_table[ei].attr |=  WEA_DISABLED;
}

//------------------------------------------------------------------------------
bool console_IsEditing(void)
{
  return IsEditing();
}

//------------------------------------------------------------------------------
void console_SetEditState(bool set)
{
  CurWnd->editing = set;
}

//------------------------------------------------------------------------------
void console_ParsChanged(void)
{
  if (CurWnd) CurWnd->event_proc(WE_PARS_CHANGED, 0);
}
