#include "disp.h"

#ifdef __HY28B__

#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"

#define __SPI_HDW__

#include "MDR32F9Qx_ssp.h"

#include "itmr.h"
#include "hy28b_drv.h"
#include "hy28b_font.h"
#include "jdt1800_font.h"

#define __DEBUG_HY28B__

#ifdef __DEBUG_HY28B__
#include "led.h"
#include "uart.h"
#endif

#define hy28b_CS_LOW		PORT_WriteBit(MDR_PORTD, PORT_Pin_8, Bit_RESET)
#define hy28b_CS_HIGH		PORT_WriteBit(MDR_PORTD, PORT_Pin_8, Bit_SET)

static char DispBuffer[MAX_Y / HY_FONT_Y][MAX_X / HY_FONT_X];

void hy28b_HardwareInit(void)
{						
	unsigned int i, j;
	PORT_InitTypeDef PORT_InitStructure;
#ifdef __SPI_HDW__
	SSP_InitTypeDef sSSP;
#endif	
	
	RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTB | RST_CLK_PCLK_PORTC | RST_CLK_PCLK_PORTD | RST_CLK_PCLK_PORTF, ENABLE);	
#ifdef __SPI_HDW__
	RST_CLK_PCLKcmd(RST_CLK_PCLK_SSP1, ENABLE);	
#endif
	
	PORT_InitStructure.PORT_OE = PORT_OE_IN;	
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;
	PORT_InitStructure.PORT_PULL_UP = PORT_PULL_UP_ON; 
#ifdef __SPI_HDW__
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_ALTER;
#else	
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
#endif
	PORT_InitStructure.PORT_Pin = PORT_Pin_6; // SPI
	PORT_Init(MDR_PORTC, &PORT_InitStructure);	
	
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;			
	PORT_InitStructure.PORT_Pin = (PORT_Pin_5 | PORT_Pin_7); // SPI
	PORT_Init(MDR_PORTC, &PORT_InitStructure);	

#ifdef __SPI_HDW__
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
#endif	
	PORT_InitStructure.PORT_Pin =  PORT_Pin_8 | PORT_Pin_11;	//  
	PORT_Init(MDR_PORTD, &PORT_InitStructure);		
	PORT_WriteBit(MDR_PORTD, PORT_Pin_8, Bit_SET);
	PORT_WriteBit(MDR_PORTD, PORT_Pin_11, Bit_SET); //    TP
		
	PORT_InitStructure.PORT_Pin =  PORT_Pin_14;	//  
	PORT_Init(MDR_PORTF, &PORT_InitStructure);		
	PORT_WriteBit(MDR_PORTF, PORT_Pin_14, Bit_SET);	
	
	PORT_InitStructure.PORT_Pin =  PORT_Pin_3;	//  
	PORT_Init(MDR_PORTB, &PORT_InitStructure);					
	
#ifdef __SPI_HDW__
	// SPI
	SSP_DeInit(MDR_SSP1);
	SSP_BRGInit(MDR_SSP1, SSP_HCLKdiv2); // 
	
	SSP_StructInit(&sSSP);

	sSSP.SSP_SCR = 0;
	sSSP.SSP_CPSDVSR = 2;
	sSSP.SSP_Mode = SSP_ModeMaster;
	sSSP.SSP_WordLength = SSP_WordLength8b;
	sSSP.SSP_SPH = SSP_SPH_2Edge;
	sSSP.SSP_SPO = SSP_SPO_High;
	sSSP.SSP_FRF = SSP_FRF_SPI_Motorola;
	sSSP.SSP_HardwareFlowControl = SSP_HardwareFlowControl_SSE;
	SSP_Init(MDR_SSP1, &sSSP);
	
	SSP_Cmd(MDR_SSP1, ENABLE);
#endif

	for(i = 0; i < MAX_Y / HY_FONT_Y; i++){
		for(j = 0; j < MAX_X / HY_FONT_X; j++){
			DispBuffer[i][j] = ' ';
		}
	}

	// 
	PORT_WriteBit(MDR_PORTB, PORT_Pin_3, Bit_SET);
	itmr_Delay(100); //
	PORT_WriteBit(MDR_PORTB, PORT_Pin_3, Bit_RESET);
	itmr_Delay(100); //
	PORT_WriteBit(MDR_PORTB, PORT_Pin_3, Bit_SET);
	itmr_Delay(100); //		
}

static unsigned char hy28b_SendAndRcvByte(unsigned char byte)
{
#ifndef	__SPI_HDW__
	unsigned char bit, rcv_byte = 0;
	//  -  
	PORT_WriteBit(MDR_PORTC, PORT_Pin_7, Bit_SET);
	for(bit = 0; bit < 8; bit++){
		//  
		if((byte >> (7 - bit)) & 1)
			PORT_WriteBit(MDR_PORTC, PORT_Pin_5, Bit_SET);
		else
			PORT_WriteBit(MDR_PORTC, PORT_Pin_5, Bit_RESET);		
		//  -  
		PORT_WriteBit(MDR_PORTC, PORT_Pin_7, Bit_RESET);	
		//  -  
		PORT_WriteBit(MDR_PORTC, PORT_Pin_7, Bit_SET);
		
		rcv_byte |= PORT_ReadInputDataBit(MDR_PORTC, PORT_Pin_6) << bit;
	}
	return rcv_byte;	
#else			
	unsigned short rb;
	while (SSP_GetFlagStatus(MDR_SSP1, SSP_FLAG_TFE ) == RESET);
	SSP_SendData(MDR_SSP1, byte);
	while (SSP_GetFlagStatus(MDR_SSP1, SSP_FLAG_RNE ) == RESET);
	rb = SSP_ReceiveData(MDR_SSP1);
	//uart_SendByte(rb);
	return rb;
#endif	
}

void hy28b_WriteIndex(unsigned char index)
{
	hy28b_CS_LOW;
        
	hy28b_SendAndRcvByte(SPI_START | SPI_WR | SPI_INDEX);   /* Write : RS = 0, RW = 0       */
	hy28b_SendAndRcvByte(0);
	hy28b_SendAndRcvByte(index);

	hy28b_CS_HIGH; 
}

void hy28b_WriteData(unsigned short data)
{
	hy28b_CS_LOW;

	hy28b_SendAndRcvByte(SPI_START | SPI_WR | SPI_DATA);    /* Write : RS = 1, RW = 0       */
	hy28b_SendAndRcvByte((data >> 8));                    	/* Write D8..D15                */
	hy28b_SendAndRcvByte((data & 0xFF));                    /* Write D0..D7                 */

	hy28b_CS_HIGH; 
}

void hy28b_WriteDataStart(void) 
{
  hy28b_SendAndRcvByte(SPI_START | SPI_WR | SPI_DATA);    /* Write : RS = 1, RW = 0       */
}

void hy28b_WriteDataOnly(unsigned short data) 
{
  hy28b_SendAndRcvByte((data >> 8));      /* Write D8..D15                */
  hy28b_SendAndRcvByte((data & 0xFF));      /* Write D0..D7                 */
}

unsigned short hy28b_ReadData(void)
{ 
	unsigned short value;

	hy28b_CS_LOW;

	hy28b_SendAndRcvByte(SPI_START | SPI_RD | SPI_DATA);    /* Read: RS = 1, RW = 1         */
	hy28b_SendAndRcvByte(0);                                /* Dummy read 1                 */
	value   = hy28b_SendAndRcvByte(0);                      /* Read D8..D15                 */
	value <<= 8;
	value  |= hy28b_SendAndRcvByte(0);                      /* Read D0..D7                  */

	hy28b_CS_HIGH; 
	
	return value;
}

void hy28b_WriteReg( unsigned short LCD_Reg, unsigned short LCD_RegValue)
{
	/* Write 16-bit Index, then Write Reg */  
	hy28b_WriteIndex(LCD_Reg);         
	/* Write 16-bit Reg */
	hy28b_WriteData(LCD_RegValue);  
}

unsigned short hy28b_ReadReg( unsigned short LCD_Reg)
{		
	/* Write 16-bit Index (then Read Reg) */
	hy28b_WriteIndex(LCD_Reg);
	/* Read 16-bit Reg */	
	return hy28b_ReadData();    
}

static void hy28b_SetCursor( unsigned short Xpos, unsigned short Ypos )
{
	/* 0x9320 */
	hy28b_WriteReg(0x0020, Xpos );     
	hy28b_WriteReg(0x0021, Ypos );           
}

void hy28b_Init(void)
{
	hy28b_HardwareInit();
	hy28b_WriteReg(0x00e7,0x0010);      		
	hy28b_WriteReg(0x0000,0x0001);  	// start internal osc 
	hy28b_WriteReg(0x0001,0x0100);     
	hy28b_WriteReg(0x0002,0x0700); 	// power on sequence 
	hy28b_WriteReg(0x0003,(1<<12)|(1<<4)); 	// importance 
	hy28b_WriteReg(0x0004,0x0000);                                   
	hy28b_WriteReg(0x0008,0x0207);	           
	hy28b_WriteReg(0x0009,0x0000);         
	hy28b_WriteReg(0x000a,0x0000); 	// display setting    
	hy28b_WriteReg(0x000c,0x0001);	// display setting         
	hy28b_WriteReg(0x000d,0x0000); 			        
	hy28b_WriteReg(0x000f,0x0000);
	// Power On sequence 
	hy28b_WriteReg(0x0010,0x0000);   
	hy28b_WriteReg(0x0011,0x0007);
	hy28b_WriteReg(0x0012,0x0000);                                                                 
	hy28b_WriteReg(0x0013,0x0000);                 
	itmr_Delay(50);
	hy28b_WriteReg(0x0010,0x1590);   
	hy28b_WriteReg(0x0011,0x0227);
	itmr_Delay(50);
	hy28b_WriteReg(0x0012,0x009c);                  
	itmr_Delay(50);
	hy28b_WriteReg(0x0013,0x1900);   
	hy28b_WriteReg(0x0029,0x0023);
	hy28b_WriteReg(0x002b,0x000e);
	itmr_Delay(50);
	hy28b_WriteReg(0x0020,0x0000);                                                            
	hy28b_WriteReg(0x0021,0x0000);           
	itmr_Delay(50);
	hy28b_WriteReg(0x0030,0x0007); 
	hy28b_WriteReg(0x0031,0x0707);   
	hy28b_WriteReg(0x0032,0x0006);
	hy28b_WriteReg(0x0035,0x0704);
	hy28b_WriteReg(0x0036,0x1f04); 
	hy28b_WriteReg(0x0037,0x0004);
	hy28b_WriteReg(0x0038,0x0000);        
	hy28b_WriteReg(0x0039,0x0706);     
	hy28b_WriteReg(0x003c,0x0701);
	hy28b_WriteReg(0x003d,0x000f);
	itmr_Delay(50);
	hy28b_WriteReg(0x0050,0x0000);        
	hy28b_WriteReg(0x0051,0x00ef);   
	hy28b_WriteReg(0x0052,0x0000);     
	hy28b_WriteReg(0x0053,0x013f);
	hy28b_WriteReg(0x0060,0xa700);        
	hy28b_WriteReg(0x0061,0x0001); 
	hy28b_WriteReg(0x006a,0x0000);
	hy28b_WriteReg(0x0080,0x0000);
	hy28b_WriteReg(0x0081,0x0000);
	hy28b_WriteReg(0x0082,0x0000);
	hy28b_WriteReg(0x0083,0x0000);
	hy28b_WriteReg(0x0084,0x0000);
	hy28b_WriteReg(0x0085,0x0000);
	hy28b_WriteReg(0x0090,0x0010);     
	hy28b_WriteReg(0x0092,0x0000);  
	hy28b_WriteReg(0x0093,0x0003);
	hy28b_WriteReg(0x0095,0x0110);
	hy28b_WriteReg(0x0097,0x0000);        
	hy28b_WriteReg(0x0098,0x0000);  
	// display on sequence
	hy28b_WriteReg(0x0007,0x0133);		
	hy28b_WriteReg(0x0020,0x0000);         
	hy28b_WriteReg(0x0021,0x0000);	
	itmr_Delay(100);	
}

void hy28b_Clear(unsigned short color)
{
	unsigned int index=0;
	
	hy28b_SetCursor(0,0); 

	hy28b_WriteIndex(0x0022);

 	hy28b_CS_LOW; 
	hy28b_WriteDataStart();

	for( index = 0; index < MAX_X * MAX_Y; index++ )	
		hy28b_WriteDataOnly(color);	
	hy28b_CS_HIGH;
}

void hy28b_SetPoint(unsigned short Xpos, unsigned short Ypos, unsigned short point)
{
	if( Xpos >= MAX_X || Ypos >= MAX_Y )	
		return;
	hy28b_SetCursor(Ypos, MAX_X - Xpos);	
	hy28b_WriteReg(0x0022,point);
}

void hy28b_PrintByteScale(unsigned short Xpos, unsigned short Ypos, unsigned char byte, unsigned short charColor, unsigned short bkColor )	
{
	unsigned short i;
	
	for( i = 0; i < 8; i++ ){				
		if((byte >> (7 - i)) & 0x01){            
			hy28b_SetPoint( 3 * Xpos + 1 - 1, 3 * (Ypos + i) + HY_FONT_Y + 1 - 1, charColor );  /* Character color */            
			hy28b_SetPoint( 3 * Xpos + 1 - 1, 3 * (Ypos + i) + HY_FONT_Y +1 + 1, charColor );  /* Character color */            
			hy28b_SetPoint( 3 * Xpos + 1 + 1, 3 * (Ypos + i) + HY_FONT_Y +1 - 1, charColor );  /* Character color */            			
			hy28b_SetPoint( 3 * Xpos + 1 + 1, 3 * (Ypos + i) + HY_FONT_Y +1 + 1, charColor );  /* Character color */            
			
			hy28b_SetPoint( 3 * Xpos + 1, 3 * (Ypos + i) + 1 + HY_FONT_Y , charColor );  /* Character color */            
			
			hy28b_SetPoint( 3 * Xpos + 1 - 1, 3 * (Ypos + i) + HY_FONT_Y +1, charColor );  /* Character color */            			
			hy28b_SetPoint( 3 * Xpos + 1 + 1, 3 * (Ypos + i) + HY_FONT_Y +1, charColor );  /* Character color */            
			hy28b_SetPoint( 3 * Xpos + 1, 3 * (Ypos + i) + 1 + HY_FONT_Y - 1, charColor );  /* Character color */            
			hy28b_SetPoint( 3 * Xpos + 1, 3 * (Ypos + i) + 1 + HY_FONT_Y + 1, charColor );  /* Character color */            									
		} 
		else{
			hy28b_SetPoint( 3 * Xpos + 1 - 1, 3 * (Ypos + i) + HY_FONT_Y + 1 - 1, bkColor );  /* Character color */            
			hy28b_SetPoint( 3 * Xpos + 1 - 1, 3 * (Ypos + i) + HY_FONT_Y + 1 + 1, bkColor );  /* Character color */            
			hy28b_SetPoint( 3 * Xpos + 1 + 1, 3 * (Ypos + i) + HY_FONT_Y + 1 - 1, bkColor );  /* Character color */            			
			hy28b_SetPoint( 3 * Xpos + 1 + 1, 3 * (Ypos + i) + HY_FONT_Y + 1 + 1, bkColor );  /* Character color */            
			
			hy28b_SetPoint( 3 * Xpos + 1, 3 * (Ypos + i) + HY_FONT_Y + 1, bkColor );  /* Character color */            
			
			hy28b_SetPoint( 3 * Xpos + 1 - 1, 3 * (Ypos + i) + HY_FONT_Y + 1, bkColor );  /* Character color */            			
			hy28b_SetPoint( 3 * Xpos + 1 + 1, 3 * (Ypos + i) + HY_FONT_Y + 1, bkColor );  /* Character color */            
			hy28b_SetPoint( 3 * Xpos + 1, 3 * (Ypos + i) + HY_FONT_Y + 1 - 1, bkColor );  /* Character color */            
			hy28b_SetPoint( 3 * Xpos + 1, 3 * (Ypos + i) + HY_FONT_Y + 1 + 1, bkColor );  /* Character color */            						
		}
	}
}

void hy28b_PrintChar(unsigned short Xpos, unsigned short Ypos, unsigned char ASCII, unsigned short charColor, unsigned short bkColor )
{
	unsigned short i, j;
	unsigned int xx, yy;
	unsigned char tmp_char1, tmp_char2;  
	
	if(Xpos == 1 && Ypos == 3)
		DispBuffer[Ypos][Xpos] = 0xFF;
	
	if(DispBuffer[Ypos][Xpos] == ASCII) return;
	
	xx = Xpos;
	yy = Ypos;
	Xpos *= HY_FONT_X;
	Ypos *= HY_FONT_Y;	
	
	for( i=0; i< 2 * HY_FONT_Y; i += 2 ){
		tmp_char1 = Font16x20Table[ASCII][i+1];			
		for( j=0; j<HY_FONT_X/2; j++ ){			
				if( (tmp_char1 >> (7 - j)) & 0x01)            
					hy28b_SetPoint( Xpos + j, Ypos + (i >> 1), charColor );  /* Character color */            
				else	            
					hy28b_SetPoint( Xpos + j, Ypos + (i >> 1), bkColor );  /* Background color */            			
		}
		
		tmp_char2 = Font16x20Table[ASCII][i];						
		for( j=HY_FONT_X/2; j<HY_FONT_X; j++ ){			
				if( (tmp_char2 >> (15 - j)) & 0x01)            
					hy28b_SetPoint( Xpos + j, Ypos + (i >> 1), charColor );  /* Character color */            
				else	            
					hy28b_SetPoint( Xpos + j, Ypos + (i >> 1), bkColor );  /* Background color */            			
		}			
	}	
	DispBuffer[yy][xx] = ASCII;
}

void hy28b_PrintString(uint16_t Xpos, uint16_t Ypos, uint8_t *str,uint16_t Color, uint16_t bkColor)
{
    unsigned char tmp;
    do
    { 				
				tmp = *str++;  							
        hy28b_PrintChar( Xpos, Ypos, tmp, Color, bkColor );    
				uart_SendByte(tmp);
        if( Xpos < MAX_X - 1 ) Xpos++;
    }
    while ( *str != 0 );
}

#endif
