#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"

//#include "disp.h"

#include "itmr.h"

//#define __ETH_LED_DEBUG__

#define LED_PORTS_CLK   (RST_CLK_PCLK_PORTA | RST_CLK_PCLK_PORTB | RST_CLK_PCLK_PORTC | \
                         RST_CLK_PCLK_PORTD)
#ifndef __HY28B__
static unsigned int stime1;
static unsigned int period1 = 100;
static unsigned int stime2;	
static unsigned int period2 = 1000;
#endif

void led_Toggle(unsigned char num);
void led_Write(unsigned char num, bool on_off);

void led_Init(void)
{					
#ifndef __HY28B__
	PORT_InitTypeDef PORT_InitStructure;
	
	RST_CLK_PCLKcmd(LED_PORTS_CLK, ENABLE);	
#ifdef __ETH_LED_DEBUG__
	RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTE, ENABLE);	
#endif	
	
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;
	
	PORT_InitStructure.PORT_Pin = (PORT_Pin_9 | PORT_Pin_13);	
	PORT_Init(MDR_PORTA, &PORT_InitStructure);	
	PORT_InitStructure.PORT_Pin = (PORT_Pin_0);	
	PORT_Init(MDR_PORTB, &PORT_InitStructure);	
	PORT_InitStructure.PORT_Pin = (PORT_Pin_8 | PORT_Pin_9);	
	PORT_Init(MDR_PORTC, &PORT_InitStructure);			
	PORT_InitStructure.PORT_Pin = (PORT_Pin_0 | PORT_Pin_7 | PORT_Pin_12);	
	PORT_Init(MDR_PORTD, &PORT_InitStructure);			

#ifdef __ETH_LED_DEBUG__
	PORT_InitStructure.PORT_Pin = (PORT_Pin_6 | PORT_Pin_7);	
	PORT_Init(MDR_PORTE, &PORT_InitStructure);
#endif

	stime1 = itmr_GetTick();	
	stime2 = itmr_GetTick();	
	#endif
}

void led_Work(void)
{			
#ifndef __HY28B__
		static char CurLed1 = 0;
		static char CurLed2 = 4;

		if(itmr_IsTime(stime1, period1)){					
			led_Toggle(CurLed1++);
			if(CurLed1 > 3) CurLed1 = 0;
			led_Toggle(CurLed1);
			stime1 = itmr_GetTick();
		}			

		if(itmr_IsTime(stime2, period2)){					
			led_Write(CurLed2++, true);
			if(CurLed2 > 7) CurLed2 = 4;
			led_Write(CurLed2, false);
			stime2 = itmr_GetTick();
		}					
#endif
}

void led_Toggle(unsigned char num)
{
#ifndef __HY28B__
	switch(num){
		case 0:
			PORT_Toggle(MDR_PORTA, PORT_Pin_9);
			break;
		case 1:
			PORT_Toggle(MDR_PORTA, PORT_Pin_13);
			break;
		case 2:
			PORT_Toggle(MDR_PORTC, PORT_Pin_8);
			break;
		case 3:
			PORT_Toggle(MDR_PORTC, PORT_Pin_9);
			break;
		case 4:
			PORT_Toggle(MDR_PORTD, PORT_Pin_0);
			break;
		case 5:
			PORT_Toggle(MDR_PORTD, PORT_Pin_7);
			break;		
		case 6:
			PORT_Toggle(MDR_PORTD, PORT_Pin_12);
			break;
		case 7:
			PORT_Toggle(MDR_PORTB, PORT_Pin_0);
			break;			
		default:			
			break;
	}		
#endif	
}

void led_Write(unsigned char num, bool on_off)
{
#ifndef __HY28B__
	switch(num){
		case 0:
			PORT_WriteBit(MDR_PORTA, PORT_Pin_9, on_off ? Bit_SET : Bit_RESET);
			break;
		case 1:
			PORT_WriteBit(MDR_PORTA, PORT_Pin_13, on_off ? Bit_SET : Bit_RESET);
			break;
		case 2:
			PORT_WriteBit(MDR_PORTC, PORT_Pin_8, on_off ? Bit_SET : Bit_RESET);
			break;
		case 3:
			PORT_WriteBit(MDR_PORTC, PORT_Pin_9, on_off ? Bit_SET : Bit_RESET);
			break;
		case 4:
			PORT_WriteBit(MDR_PORTD, PORT_Pin_0, on_off ? Bit_SET : Bit_RESET);
			break;
		case 5:
			PORT_WriteBit(MDR_PORTD, PORT_Pin_7, on_off ? Bit_SET : Bit_RESET);
			break;		
		case 6:
			PORT_WriteBit(MDR_PORTD, PORT_Pin_12, on_off ? Bit_SET : Bit_RESET);
			break;
		case 7:
			PORT_WriteBit(MDR_PORTB, PORT_Pin_0, on_off ? Bit_SET : Bit_RESET);
			break;			
		default:			
			break;
	}		
#endif	
}
