#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"

#define __DEBUG_MUX__

#include "mux.h"
#include "itmr.h"

#ifdef __DEBUG_MUX__
#include "led.h"
static unsigned int stime;
static unsigned int period = 4000;
static unsigned char addr;
#endif 

// управление переключением линий CS для разных микросхем доп.платы FUNC
/*
PE11 - A0
PE12 - A1
PE13 - A2
PE14 - A3
PE15 - A4 // всегда 1 - выборка мультиплексора

адреса:
14 - экспандер
13 - FLASH
11 - датчик давления
10 - MRAM
9 - misroSD
8 - FRAM
7 - EEPROM
6 - компас
*/

static uint32_t PinArr[] = {PORT_Pin_11, PORT_Pin_12, PORT_Pin_13, PORT_Pin_14};
static uint8_t Addr[8] = {	MUX_EXP_ADDR, MUX_FLASH_ADDR, MUX_PRESS_ADDR, MUX_MRAM_ADDR,
													MUX_MICRO_SD_ADDR, MUX_FRAM_ADDR, MUX_EEPROM_ADDR, MUX_COMPAS_ADDR};
void mux_Init(void)
{
	PORT_InitTypeDef PORT_InitStructure;
	
	RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTE, ENABLE);	
	
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	

	PORT_InitStructure.PORT_Pin = (PORT_Pin_11 | PORT_Pin_12 | PORT_Pin_13 | PORT_Pin_14 | PORT_Pin_15);
	PORT_Init(MDR_PORTE, &PORT_InitStructure);
	
	// выборка мультиплексора
	PORT_WriteBit(MDR_PORTE, PORT_Pin_15, Bit_RESET);
	
	#ifdef __DEBUG_MUX__
	stime = itmr_GetTick();	
	#endif
}

void mux_SetAddr(uint8_t addr)
{
	uint8_t i;
	
	addr &= 0x0f; // старших ниббл отсутствует
	
	for(i = 0; i < 4; i++){
			if(addr & (1 << i))
				PORT_WriteBit(MDR_PORTE, PinArr[i], Bit_SET);
			else 
				PORT_WriteBit(MDR_PORTE, PinArr[i], Bit_RESET);
	}
	itmr_Delay(100);
}

void mux_ClearAddr(void)
{
	uint8_t i;
	for(i = 0; i < 4; i++)
		PORT_WriteBit(MDR_PORTE, PinArr[i], Bit_RESET);	
}

void mux_Test(void)
{
	#ifdef __DEBUG_MUX__	
	
	if(itmr_IsTime(stime, period)){			
			mux_SetAddr(Addr[addr]);						
			addr++;
			if(addr > 7) addr = 0;
			stime = itmr_GetTick();		
			led_Toggle(0);
		}			
	#endif
}
