#include <string.h>
#include "MDR32F9Qx_uart.h"
#include "itmr.h"
#include "console.h"
#include "wnds.h"
#include "uart.h"

static TUARTSettings UARTSettings; 

#define SPEED_SEL_CNT 		(6)
#define DATA_BITS_SEL_CNT (4)
#define PAR_SEL_CNT 			(5)
#define STOP_SEL_CNT 			(2)

static char* selSpeedText[SPEED_SEL_CNT] = { "4800", "9600", "19200", "38400", "57600", "115200" };
static TWESelector selSpeed = { selSpeedText, SPEED_SEL_CNT, 1 };

static char* selDataBitsText[DATA_BITS_SEL_CNT] = { "5", "6", "7", "8" };
static TWESelector selDataBits = { selDataBitsText, DATA_BITS_SEL_CNT, 3 };

static char* selParText[PAR_SEL_CNT] = { "", "", "", "0", "1"};
static TWESelector selPar = { selParText, PAR_SEL_CNT, 0 };

static char* selStopText[PAR_SEL_CNT] = { "1", "2"};
static TWESelector selStop = { selStopText, STOP_SEL_CNT, 0 };

BEGIN_WE_TABLE(ElemTable)
	DEFINE_WE(WET_SELECTOR,	0, 0, 0, ": ", &selSpeed)
	DEFINE_WE(WET_SELECTOR, 0, 1, 0, " : ", &selDataBits )  	
	DEFINE_WE(WET_SELECTOR, 0, 2, 0, ": ", &selPar)  
	DEFINE_WE(WET_SELECTOR, 0, 3, 0, " : ", &selStop)		  
END_WE_TABLE()

static bool EventProc(unsigned int code, unsigned int data);

static TWnd Wnd = { WID_UART, ElemTable, EventProc, 0 };

static unsigned int GetSpeed(void)
{
	switch(selSpeed.cur_val){
		case 0:
			return 4800;
		case 1:
			return 9600;
		case 2:
			return 19200;
		case 3:
			return 38400;
		case 4:
			return 57600; 
		case 5:
			return 115200;
		default:
			return 9600;
	}
}

static unsigned int GetSpeedPos(unsigned int speed)
{
	switch(speed){
		case 4800:
			return 0;
		case 9600:
			return 1;
		case 19200:
			return 2;
		case 38400:
			return 3;
		case 57600:
			return 4; 
		case 115200:
			return 5;
		default:
			return 1;
	}
}

static unsigned short GetDataBits(void)
{
	switch(selDataBits.cur_val){
		case 0:
			return UART_WordLength5b;
		case 1:
			return UART_WordLength6b;		
		case 2:
			return UART_WordLength7b;		
		case 3:
		default:
			return UART_WordLength8b;
		
	}
}

static unsigned short GetDataBitsPos(unsigned short databits)
{
	switch(databits){
		case UART_WordLength5b:
			return 0;
		case UART_WordLength6b:
			return 1;		
		case UART_WordLength7b:
			return 2;		
		case UART_WordLength8b:
		default:
			return 3;
		
	}
}
	
static unsigned short GetParity(void)
{
	switch(selPar.cur_val){
		case 0:
			return UART_Parity_No;
		case 1:
			return UART_Parity_Even;		
		case 2:
			return UART_Parity_Odd;		
		case 3:
			return UART_Parity_0;
		case 4:
			return UART_Parity_1;
		default:
			return UART_Parity_No;	
	}
}

static unsigned short GetParityPos(unsigned short parity)
{
	switch(parity){
		case UART_Parity_No:
			return 0;
		case UART_Parity_Even:
			return 1;		
		case UART_Parity_Odd:
			return 2;		
		case UART_Parity_0:
			return 3;
		case UART_Parity_1:
			return 4;
		default:
			return 0;	
	}
}

static unsigned short GetStopBitsCnt(void)
{
	switch(selStop.cur_val){
		case 0:
			return UART_StopBits1;
		case 1:
			return UART_StopBits2;
		default:
			return UART_StopBits1;		
	}
}

static unsigned short GetStopBitsCntPos(unsigned short stopbits)
{
	switch(stopbits){
		case UART_StopBits1:
			return 0;
		case UART_StopBits2:
			return 1;
		default:
			return 0;		
	}
}

static bool EventProc(unsigned int code, unsigned int data)
{
  switch (code) {
    case WE_DRAW:
      if (data == 0) {				
				selSpeed.cur_val = GetSpeedPos(UARTSettings.speed); 
				selDataBits.cur_val = GetDataBitsPos(UARTSettings.bit_cnt);
				selPar.cur_val = GetParityPos(UARTSettings.parity);
				selStop.cur_val = GetStopBitsCntPos(UARTSettings.stop_bits); 
      }
      break;				
		case WE_EDIT_END:
			if(data){
				 switch(Wnd.cur_elem){
					 case 0:
						 UARTSettings.speed	= GetSpeed();				 
						 break;
					 case 1:
						 UARTSettings.bit_cnt	= GetDataBits();
						 break;
					 case 2:
						 UARTSettings.parity	= GetParity();
						 break;
					 case 3:
						 UARTSettings.stop_bits = GetStopBitsCnt();
						 break;
					 default:
						 break;
				 }
				 uart_Init(UARTSettings.speed, UARTSettings.bit_cnt, UARTSettings.stop_bits, UARTSettings.parity);
			}
			break;
  }

  return false;
}

void wnd_Uart(void)
{ 
	uart_GetSettings(&UARTSettings);
	console_OpenWnd(&Wnd);
}
